@@ -32,6 +32,7 @@
 #include "system.h"
 #include "gretl_restrict.h"
 #include "gretl_func.h"
+#include "gretl_help.h"
 #include "libset.h"
 #include "cmd_private.h"
 #include "flow_control.h"
@@ -745,6 +746,7 @@ static int cli_exec_line (ExecState *s, int id, DATASET *dset,
 	    set_gretl_errno(err);
 	    catch = 1;
 	}
+	echo_command(cmd, dset, line, prn);
         errmsg(err, prn);
 	return (catch)? 0 : err;
     }
