@@ -155,28 +155,7 @@ struct open_data_code {
 
 struct open_data_code open_data_codes[] = {
     { OPEN_DATA,       "OpenData" },    
-    { OPEN_GDT,        "OpenGdt" },
-    { APPEND_GDT,      "AppendGdt" },
-    { OPEN_CSV,        "OpenCSV" },
-    { APPEND_CSV,      "AppendCSV" },
-    { OPEN_OCTAVE,     "OpenOctave" },
-    { APPEND_OCTAVE,   "AppendOctave" },
-    { OPEN_GNUMERIC,   "OpenGnumeric" },
-    { APPEND_GNUMERIC, "AppendGnumeric" },
-    { OPEN_XLS,        "OpenXLS" },
-    { APPEND_XLS,      "AppendXLS" },
-    { OPEN_WF1,        "OpenWF1" },
-    { APPEND_WF1,      "AppendWF1" },
-    { OPEN_DTA,        "OpenDTA" },
-    { APPEND_DTA,      "AppendDTA" },
-    { OPEN_SAV,        "OpenSAV" },
-    { APPEND_SAV,      "AppendSAV" },
-    { OPEN_SAS,        "OpenSAS" },
-    { APPEND_SAS,      "AppendSAS" },
-    { OPEN_JMULTI,     "OpenJMulTi" },
-    { APPEND_JMULTI,   "AppendJMulTi" },
-    { OPEN_ODS,        "OpenODS" },
-    { APPEND_ODS,      "AppendODS" },
+    { APPEND_DATA,     "AppendData" },
     { OPEN_MARKERS,    "AddMarkers" },
     { OPEN_RATS_DB,    "RATSDB" },
     { OPEN_PCGIVE_DB,  "PcGiveDB" },
@@ -201,17 +180,6 @@ void open_data (GtkAction *action)
     if (!dataset_locked()) {
 	int code = open_data_code(gtk_action_get_name(action));
 
-#if 1
-	if (code == OPEN_CSV || code == APPEND_CSV) {
-	    /* allow specification of separator */
-	    int resp = csv_options_dialog(code, GRETL_OBJ_DSET, NULL);
-
-	    if (canceled(resp)) {
-		return;
-	    }
-	}
-#endif
-
 	file_selector(code, FSEL_DATA_NONE, NULL);
     }
 }
@@ -768,13 +736,11 @@ static int nist_verbosity (GtkAction *action)
 
 void do_nistcheck (GtkAction *action)
 {
-    void *handle;
     int (*run_nist_tests)(const char *, const char *, int);
     gchar *datadir = NULL;
     gchar *fname = NULL;
     
-    run_nist_tests = gui_get_plugin_function("run_nist_tests", 
-					     &handle);
+    run_nist_tests = gui_get_plugin_function("run_nist_tests");
     if (run_nist_tests == NULL) {
 	return;
     }
@@ -784,8 +750,6 @@ void do_nistcheck (GtkAction *action)
 
     (*run_nist_tests)(datadir, fname, nist_verbosity(action));
 
-    close_plugin(handle);
-
     view_file(fname, 0, 1, 78, 400, VIEW_CODEBOOK);
 
     g_free(datadir);
@@ -797,15 +761,12 @@ void do_nistcheck (GtkAction *action)
 void send_file (char *fullname)
 {
     int (*email_file) (const char *, const char *);
-    void *handle;
 
-    email_file = gui_get_plugin_function("email_file", &handle);
-    if (email_file == NULL) {
-        return;
+    email_file = gui_get_plugin_function("email_file");
+
+    if (email_file != NULL) {
+	email_file(fullname, gretl_dotdir());
     }
-    
-    email_file(fullname, gretl_dotdir());
-    close_plugin(handle);
 }
 
 #endif
