@@ -108,7 +108,7 @@ static void gretl_clipboard_get (GtkClipboard *clip,
     }
 
     if (info != TARGET_UTF8_STRING) {
-	/* remove any Unicode minuses */
+	/* remove any Unicode minuses (?) */
 	strip_unicode_minus(str);
     }
 
@@ -180,27 +180,6 @@ static void gretl_clipboard_set (int fmt)
     }
 }
 
-/* We call this when a buffer to be copied as RTF contains
-   non-ASCII characters but validates as UTF-8. Note that the 
-   UTF-8 minus sign does not recode correctly into Windows 
-   codepages, so if it's present we have to remove it first, 
-   then see if there's anything non-ASCII left to handle.
-*/
-
-static char *strip_utf8 (char *buf)
-{
-    strip_unicode_minus(buf);
-
-    if (string_is_utf8((const unsigned char *) buf)) {
-#if CLIPDEBUG
-	fprintf(stderr, "strip_utf8: stage 2, recoding\n");
-#endif
-	return utf8_to_cp(buf);
-    } else {
-	return buf;
-    }
-}
-
 /* note: there's a Windows-specific counterpart to this
    in gretlwin32.c
 */
@@ -208,9 +187,7 @@ static char *strip_utf8 (char *buf)
 int prn_to_clipboard (PRN *prn, int fmt)
 {
     char *buf = gretl_print_steal_buffer(prn);
-    const char *cset = NULL;
-    gchar *trbuf = NULL;
-    int utf8_ok = 1;
+    char *modbuf = NULL;
     int err = 0;
 
     if (buf == NULL || *buf == '\0') {
@@ -219,39 +196,21 @@ int prn_to_clipboard (PRN *prn, int fmt)
 
     gretl_clipboard_free();
 
-    if (fmt == GRETL_FORMAT_RTF || fmt == GRETL_FORMAT_RTF_TXT) {
-	/* UTF-8 will never work in RTF */
-	utf8_ok = 0;
-    } else if (fmt == GRETL_FORMAT_TXT && !g_get_charset(&cset)) {
-	utf8_ok = 0;
-    }
+    err = maybe_post_process_buffer(buf, fmt, W_COPY, &modbuf);
 
-#if CLIPDEBUG
-    fprintf(stderr, "prn_to_clipboard, fmt = %d, utf8_ok = %d\n", fmt, utf8_ok);
-#endif
-
-    if (!utf8_ok && string_is_utf8((const unsigned char *) buf)) {
-	trbuf = strip_utf8(buf);
-    } else {
-	trbuf = buf;
-    }
-
-    if (fmt == GRETL_FORMAT_RTF_TXT) {
-	clipboard_buf = dosify_buffer(trbuf, fmt);
-    } else {
-	clipboard_buf = gretl_strdup(trbuf);
-    }
-
-    if (trbuf != buf) {
-	g_free(trbuf);
-    }
-
-    if (clipboard_buf == NULL) {
-	err = 1;
-    } else {
+    if (!err) {
+	if (modbuf != NULL) {
+	    clipboard_buf = modbuf;
+	} else {
+	    clipboard_buf = buf;
+	}
 	gretl_clipboard_set(fmt);
     }
 
+    if (buf != clipboard_buf) {
+	free(buf);
+    }
+
     return err;
 }
 
