@@ -417,7 +417,8 @@ static int dont_go_there (const char *s)
    non-zero only if something show-stopping occurs
 */
 
-static int seek_file_collections (const char *basedir, int type,
+static int seek_file_collections (const char *basedir, 
+				  SearchType stype,
 				  int *err)
 {
     char *path = NULL;
@@ -425,14 +426,19 @@ static int seek_file_collections (const char *basedir, int type,
     struct dirent *dirent;
     int n_coll = 0;
 
+#if COLL_DEBUG
+    fprintf(stderr, "*** seek_file_collections: basedir='%s', type=%d\n", 
+	    basedir, stype);
+#endif
+
     if (*err) {
 	/* a fatal error occurred already, skip it */
 	return 0;
     }
 
-    if (type == DATA_SEARCH) {
+    if (stype == DATA_SEARCH) {
 	path = gretl_strdup_printf("%sdata", basedir);
-    } else if (type == SCRIPT_SEARCH) {
+    } else if (stype == SCRIPT_SEARCH) {
 	path = gretl_strdup_printf("%sscripts", basedir);
     } else {
 	/* USER_SEARCH */
@@ -447,7 +453,7 @@ static int seek_file_collections (const char *basedir, int type,
     }
 
 #if COLL_DEBUG
-    fprintf(stderr, "*** seeking file collections in '%s'\n", path);
+    fprintf(stderr, "*** seek_file_collections: path='%s'\n", path);
 #endif
 
     while (!*err && (dirent = readdir(topdir))) {
@@ -455,6 +461,9 @@ static int seek_file_collections (const char *basedir, int type,
 	    char *subpath;
 	    DIR *subdir;
 
+#if COLL_DEBUG > 1
+	    fprintf(stderr, " dname = '%s'\n", dirent->d_name);
+#endif
 	    if (strcmp(dirent->d_name, ".")) {
 		subpath = full_path(path, dirent->d_name);
 	    } else {
@@ -537,6 +546,8 @@ static int build_file_collections (void)
 	    wd = maybe_get_default_workdir();
 	    if (wd != NULL) {
 		n += seek_file_collections(wd, USER_SEARCH, &err);
+		n += seek_file_collections(wd, DATA_SEARCH, &err);
+		n += seek_file_collections(wd, SCRIPT_SEARCH, &err);
 	    }
 	}
 	if (!err && n == 0) {
@@ -713,7 +724,7 @@ void browser_open_data (GtkWidget *w, gpointer data)
 
     set_datapage(collection->title);
 
-    verify_open_data(vwin, OPEN_GDT);
+    verify_open_data(vwin, OPEN_DATA);
 }
 
 void browser_open_ps (GtkWidget *w, gpointer data)
@@ -1386,8 +1397,8 @@ static GretlToolItem files_items[] = {
     { N_("Edit"),           GTK_STOCK_EDIT,       G_CALLBACK(browser_edit_func), BTN_EDIT },
     { N_("Info"),           GTK_STOCK_INFO,       NULL,                          BTN_INFO },
     { N_("View code"),      GTK_STOCK_PROPERTIES, G_CALLBACK(show_function_code), BTN_CODE },
-    { N_("List series"),    GTK_STOCK_INDEX,      NULL,                          BTN_INDX },
-    { N_("Install"),        GTK_STOCK_SAVE,       NULL,                          BTN_INST },
+    { N_("List series"),    GTK_STOCK_INDEX,      NULL,                           BTN_INDX },
+    { N_("Install"),        GTK_STOCK_SAVE,       NULL,                           BTN_INST },
     { N_("Execute"),        GTK_STOCK_EXECUTE,    G_CALLBACK(browser_call_func), BTN_EXEC },
     { N_("Add to menu"),    GTK_STOCK_ADD,        G_CALLBACK(add_func_to_menu),  BTN_ADD },
     { N_("Delete"),         GTK_STOCK_DELETE,     G_CALLBACK(browser_del_func),  BTN_DEL },
@@ -2043,7 +2054,6 @@ gint populate_func_list (windata_t *vwin, struct fpkg_response *fresp)
 		closedir(dir);
 	    }
 	}
-
 	strings_array_free(dnames, n_dirs);
     } else {
 	dnames = get_plausible_search_dirs(FUNCS_SEARCH, &n_dirs);
@@ -2056,7 +2066,6 @@ gint populate_func_list (windata_t *vwin, struct fpkg_response *fresp)
 		closedir(dir);
 	    }
 	}
-
 	strings_array_free(dnames, n_dirs);
     }	    
 
