@@ -29,6 +29,14 @@
 
 #include "gretl_panel.h"
 
+/* The code here answers to (1) the "/Data/Dataset structure" menu item
+   in the main gretl window; (2) at one remove, the "/File/New data set"
+   main-window item; and (3) finalize_data_open() in gui_utils.c. where
+   a newly opened data file has been considered simply as undated,
+   non-panel data. In each case we're offering the user the chance to
+   impose time-series or panel structure.
+*/
+
 #define DWDEBUG 0
 
 #define PD_SPECIAL -1
@@ -86,7 +94,7 @@ struct dw_opts_ {
     int tid;            /* panel: ID number of "period" variable */
     int *setvar;        /* pointer to variable currently being set */
     int *extra;         /* additional pointer to int variable */
-    GtkWidget *spinner; /* used for setting starting observation */
+    GtkWidget *pdspin;  /* used for setting custom time-series frequency */
     GList *vlist;       /* panel: list of candidates for uid, tid */
 };
 
@@ -112,8 +120,11 @@ static const char *wizcode_string (int code)
 
 static int translate_panel_vars (dw_opts *opts, int *uv, int *tv);
 
-/* initialize the "dummy" DATASET structure dwinfo, based
-   on the current data info */
+/* Initialize the "dummy" DATASET structure @dwinfo, based
+   on the current data info. This will just be a cross-section
+   unless the user has decided to modify an already-structured
+   dataset under /Data/Dataset structure.
+*/
 
 static void dwinfo_init (DATASET *dwinfo)
 {
@@ -529,18 +540,18 @@ static void dwiz_set_radio_opt (GtkWidget *w, dw_opts *opts)
 {
     int val = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "action"));
 
-    if (opts->spinner != NULL) {
+    if (opts->pdspin != NULL) {
 	if (val == PD_SPECIAL) {
 	    GtkAdjustment *adj = 
-		gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(opts->spinner));
+		gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(opts->pdspin));
 
-	    gtk_widget_set_sensitive(opts->spinner, TRUE);
+	    gtk_widget_set_sensitive(opts->pdspin, TRUE);
 	    val = (int) gtk_adjustment_get_value(adj);
 	    if (opts->extra != NULL) {
 		*opts->extra = SPECIAL_TIME_SERIES;
 	    }
 	} else {
-	    gtk_widget_set_sensitive(opts->spinner, FALSE);
+	    gtk_widget_set_sensitive(opts->pdspin, FALSE);
 	    if (opts->extra != NULL) {
 		*opts->extra = TIME_SERIES;
 	    }
@@ -620,7 +631,7 @@ static int default_start_decade (void)
     return d;
 }
 
-static void compute_default_ts_info (DATASET *dwinfo, int newdata)
+static void compute_default_ts_info (DATASET *dwinfo)
 {
 #if DWDEBUG
     char obsstr[OBSLEN];
@@ -635,12 +646,9 @@ static void compute_default_ts_info (DATASET *dwinfo, int newdata)
     if (dwinfo->pd < 0) {
 	dwinfo->pd = 1;
     }
-    
-    if (dwinfo->structure == CROSS_SECTION) {
-	dwinfo->n = 500;
-	dwinfo->t1 = 0;
-	strcpy(dwinfo->stobs, "1");
-    } else if (dwinfo->structure == SPECIAL_TIME_SERIES) {
+
+    if (dwinfo->structure == SPECIAL_TIME_SERIES) {
+	/* non-standard time series */
 	dwinfo->n = 500;
 	dwinfo->t1 = 0;
 	if (dwinfo->pd > 1) {
@@ -655,9 +663,9 @@ static void compute_default_ts_info (DATASET *dwinfo, int newdata)
 	    strcpy(dwinfo->stobs, "1");
 	}
     } else if (dwinfo->pd == 1) {
-	strcpy(dwinfo->stobs, "1500");
-	dwinfo->n = 600; 
-	dwinfo->t1 = 450; /* 1950, rollable back to 1500 */
+	strcpy(dwinfo->stobs, "1");
+	dwinfo->n = 2100; 
+	dwinfo->t1 = 1959; /* 1960 */
     } else if (dwinfo->pd == 10) {
 	int dd = default_start_decade();
 
@@ -671,12 +679,12 @@ static void compute_default_ts_info (DATASET *dwinfo, int newdata)
 	}
     } else if (dwinfo->pd == 4) {
 	strcpy(dwinfo->stobs, "1700:1");
-	dwinfo->n = 1300;
-	dwinfo->t1 = 1000;
+	dwinfo->n = 1400;
+	dwinfo->t1 = 1040; /* 1960:1 */
     } else if (dwinfo->pd == 12) {
 	strcpy(dwinfo->stobs, "1700:01");
-	dwinfo->n = 3900;
-	dwinfo->t1 = 3360;
+	dwinfo->n = 4200;
+	dwinfo->t1 = 3120; /* 1960:01 */
     } else if (dwinfo->pd == 24) {
 	strcpy(dwinfo->stobs, "1:01");
 	dwinfo->n = 1500;
@@ -694,23 +702,22 @@ static void compute_default_ts_info (DATASET *dwinfo, int newdata)
     } else if (dwinfo->pd == 5 ||
 	       dwinfo->pd == 6 ||
 	       dwinfo->pd == 7) {
-	strcpy(dwinfo->stobs, "1900/01/01");
+	strcpy(dwinfo->stobs, "1900-01-01");
 	dwinfo->n = 40000;
+	/* set default start to 1960-01-01 (a Friday) */
 	if (dwinfo->pd == 5) {
-	    dwinfo->t1 = 13046;
-	} else if (dwinfo->pd == 6) {
 	    dwinfo->t1 = 15654;
+	} else if (dwinfo->pd == 6) {
+	    dwinfo->t1 = 18784;
 	} else {
-	    dwinfo->t1 = 18263;
+	    dwinfo->t1 = 21914;
 	}
     }
 
     dwinfo->sd0 = get_date_x(dwinfo->pd, dwinfo->stobs);
 
-    if (newdata) {
-	dwinfo->t2 = dwinfo->t1 + 49;
-    } else if (dataset->structure == TIME_SERIES && 
-	       dataset->pd == dwinfo->pd) {
+    if (dataset->structure == TIME_SERIES && 
+	dataset->pd == dwinfo->pd) {
 	/* make the current start the default */
 	dwinfo->t1 = dateton(dataset->stobs, dwinfo);
     }
@@ -1103,46 +1110,50 @@ static void dw_set_t1 (GtkWidget *w, DATASET *dwinfo)
 #endif
 }
 
-/* spinner for either time-series starting observation or
-   custom time-series frequency */
+/* spinner for time-series starting observation */
 
-static GtkWidget *dwiz_spinner (GtkWidget *hbox, DATASET *dwinfo, int step)
+static void add_startobs_spinner (GtkWidget *vbox,
+				  DATASET *dwinfo,
+				  int direction)
+
+{
+    GtkWidget *hbox, *label, *spin;
+    GtkAdjustment *adj;
+
+    if (direction == DW_FORWARD) {
+	compute_default_ts_info(dwinfo);
+    }
+
+    adj = (GtkAdjustment *) gtk_adjustment_new(dwinfo->t1, 
+					       0, dwinfo->n - 1,
+					       1, 10, 0);
+    g_signal_connect(G_OBJECT(adj), "value-changed", 
+		     G_CALLBACK(dw_set_t1), dwinfo);
+    spin = data_start_button(adj, dwinfo);
+    gtk_entry_set_activates_default(GTK_ENTRY(spin), TRUE);
+
+    hbox = gtk_hbox_new(FALSE, 5);
+    label = gtk_label_new(_(ts_frequency_string(dwinfo)));
+    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(hbox), spin, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
+    gtk_widget_show_all(hbox);
+}
+
+/* spinner for selecting custom time-series frequency */
+
+static GtkWidget *frequency_spinner (GtkWidget *hbox, DATASET *dwinfo)
 {
     GtkAdjustment *adj;
     GtkWidget *spin;
-    int spinmin, spinmax, spinstart;
+    int pdmax = 1000; /* arbitrary */
 
-    if (step == DW_STARTING_OBS) {
-	GtkWidget *label = gtk_label_new(_(ts_frequency_string(dwinfo)));
-
-	gtk_widget_show(label);
-	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
-
-	compute_default_ts_info(dwinfo, 0);
-	spinmin = 0;
-	spinmax = dwinfo->n - 1;
-	spinmax = INT_MAX; /* new, FIXME */
-	spinstart = dwinfo->t1;
-    } else {
-	/* custom time-series frequency */
-	spinmin = 1;
-	spinmax = 100; /* arbitrary */
-	spinstart = dwinfo->pd;
-    } 
-
-    /* appropriate step size? */
-    adj = (GtkAdjustment *) gtk_adjustment_new(spinstart, spinmin, spinmax,
+    adj = (GtkAdjustment *) gtk_adjustment_new(dwinfo->pd, 1, pdmax,
 					       1, 10, 0);
 
-    if (step == DW_STARTING_OBS) {
-	g_signal_connect(G_OBJECT(adj), "value-changed", 
-			 G_CALLBACK(dw_set_t1), dwinfo);
-	spin = data_start_button(adj, dwinfo);
-    } else {	
-	g_signal_connect(G_OBJECT(adj), "value-changed", 
-			 G_CALLBACK(dw_set_custom_frequency), dwinfo);
-	spin = gtk_spin_button_new(adj, 1, 0);
-    } 
+    g_signal_connect(G_OBJECT(adj), "value-changed", 
+		     G_CALLBACK(dw_set_custom_frequency), dwinfo);
+    spin = gtk_spin_button_new(adj, 1, 0);
 
     gtk_entry_set_activates_default(GTK_ENTRY(spin), TRUE);
     gtk_box_pack_start(GTK_BOX(hbox), spin, FALSE, FALSE, 0);
@@ -1151,16 +1162,6 @@ static GtkWidget *dwiz_spinner (GtkWidget *hbox, DATASET *dwinfo, int step)
     return spin;
 }
 
-static void dwiz_startobs_spinner (DATASET *dwinfo,
-				   GtkWidget *vbox)
-{
-    GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
-
-    dwiz_spinner(hbox, dwinfo, DW_STARTING_OBS);
-    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
-    gtk_widget_show(hbox);
-}
-
 /* Panel: callback for setting the number of cross-sectional units, n,
    and the number of time periods, T, via spin buttons.  We allow the
    user to vary either n or T, subject to the constraint that n * T
@@ -1325,7 +1326,7 @@ static void set_up_dw_opts (dw_opts *opts, int step,
 {
     opts->setvar = NULL;
     opts->extra = NULL;
-    opts->spinner = NULL;
+    opts->pdspin = NULL;
     opts->n_radios = 0;
 
     opts->deflt = dwiz_radio_default(dwinfo, step);
@@ -1392,10 +1393,10 @@ static void dwiz_build_radios (int step, DATASET *dwinfo,
 
 	if (step == DW_TS_FREQUENCY && i == opts->n_radios - 1) {
 	    /* time series, "other" (custom) frequency: need spinner */
-	    GtkWidget *freqspin = dwiz_spinner(hbox, dwinfo, step);
+	    GtkWidget *freqspin = frequency_spinner(hbox, dwinfo);
 
 	    gtk_widget_set_sensitive(freqspin, FALSE);
-	    opts->spinner = freqspin;
+	    opts->pdspin = freqspin;
 	} 
 
 	g_signal_connect(G_OBJECT(button), "clicked",
@@ -1617,6 +1618,7 @@ static void add_editing_option (GtkWidget *vbox, gretlopt *flags)
 
 static void dwiz_prepare_page (GtkNotebook *nb,
 			       gint step,
+			       gint direction,
 			       DATASET *dwinfo)
 {
     GtkWidget *page = gtk_notebook_get_nth_page(nb, step);
@@ -1647,7 +1649,7 @@ static void dwiz_prepare_page (GtkNotebook *nb,
 	}
 
 	if (step == DW_STARTING_OBS) {
-	    dwiz_startobs_spinner(dwinfo, page);
+	    add_startobs_spinner(page, dwinfo, direction);
 	    if (dataset != NULL && dataset->Z != NULL && 
 		dataset_is_daily(dwinfo)) {
 		maybe_add_missobs_purger(page, &opts->flags);
@@ -1706,7 +1708,7 @@ static void dwiz_back (GtkWidget *b, GtkWidget *dlg)
     dw_opts *opts = g_object_get_data(G_OBJECT(dlg), "opts");
 
     pg = dwiz_compute_step(pg, DW_BACK, dwinfo, opts);
-    dwiz_prepare_page(nb, pg, dwinfo);
+    dwiz_prepare_page(nb, pg, DW_BACK, dwinfo);
     gtk_notebook_set_current_page(nb, pg);
 }
 
@@ -1723,14 +1725,14 @@ static void dwiz_forward (GtkWidget *b, GtkWidget *dlg)
     if (pg == DW_SET_TYPE && any_panel(dwinfo) && !panel_possible(opts)) {
 	/* special case: called for panel but it won't work */
 	dwinfo->structure = dataset->structure;
-	dwiz_prepare_page(nb, DW_SET_TYPE, dwinfo);
+	dwiz_prepare_page(nb, DW_SET_TYPE, DW_BACK, dwinfo);
 	gtk_notebook_set_current_page(nb, DW_SET_TYPE);
 	return;
     }
 
     newpg = dwiz_compute_step(pg, DW_FORWARD, dwinfo, opts);
     if (newpg != pg) {
-	dwiz_prepare_page(nb, newpg, dwinfo);
+	dwiz_prepare_page(nb, newpg, DW_FORWARD, dwinfo);
 	gtk_notebook_set_current_page(nb, newpg);
     }
 }
@@ -1922,7 +1924,7 @@ static void data_structure_wizard (int create)
     build_dwiz_buttons(dialog, dwinfo);
     gtk_widget_show(nb);
 
-    dwiz_prepare_page(GTK_NOTEBOOK(nb), 0, dwinfo);
+    dwiz_prepare_page(GTK_NOTEBOOK(nb), 0, DW_FORWARD, dwinfo);
     gtk_notebook_set_current_page(GTK_NOTEBOOK(nb), 0);
 
     /* note: we can't use gtk_widget_show_all() here
