@@ -240,11 +240,9 @@ static void script_window_update (windata_t *vwin, const char *fname)
 static void 
 save_editable_content (int action, const char *fname, windata_t *vwin)
 {
-    const gchar *cset;
+    gchar *buf = textview_get_text(vwin->text);
     FILE *fp;
-    gchar *buf;
 
-    buf = textview_get_text(vwin->text);
     if (buf == NULL) {
 	errbox("Couldn't retrieve buffer");
 	return;
@@ -257,12 +255,18 @@ save_editable_content (int action, const char *fname, windata_t *vwin)
 	return;
     }
 
+#if 1 /* 2014-09-11: just keep any UTF-8 */
+    system_print_buf(buf, fp);
+#else /* what we used to do */
     if (action == SAVE_SCRIPT) {
 	/* don't mess with encoding */
 	system_print_buf(buf, fp);
     } else {
 	gchar *trbuf;
 
+	/* FIXME which cases come here and do
+	   they want this treatment? */
+
 	if (!g_get_charset(&cset)) {
 	    /* UTF-8 minuses not wanted for locale */
 	    strip_unicode_minus(buf);
@@ -274,6 +278,7 @@ save_editable_content (int action, const char *fname, windata_t *vwin)
 	    g_free(trbuf);
 	}
     }
+#endif
 
     g_free(buf);
     fclose(fp);
@@ -293,7 +298,18 @@ save_editable_content (int action, const char *fname, windata_t *vwin)
 
 static void filesel_save_prn_buffer (PRN *prn, const char *fname)
 {
-    FILE *fp = gretl_fopen(fname, "w");
+    int fmt = prn_format(prn);
+    FILE *fp;
+
+    /* if @prn is in RTF, use binary mode to avoid messing
+       with pre-set line endings 
+    */
+
+    if ((fmt & GRETL_FORMAT_RTF) || fmt == GRETL_FORMAT_RTF_TXT) {
+	fp = gretl_fopen(fname, "wb");
+    } else {
+	fp = gretl_fopen(fname, "w");
+    }
 
     if (fp == NULL) {
 	file_write_errbox(fname);
@@ -374,9 +390,13 @@ static char *suggested_savename (const char *fname)
 
     sfx = strrchr(s, '.');
 
-    if (sfx != NULL && (strlen(sfx) == 4 || !strcmp(sfx, ".gnumeric"))) {
-	if (strcmp(sfx, ".gdt")) {
-	    strcpy(sfx, ".gdt");
+    if (sfx != NULL) {
+	if (strlen(sfx) == 4 || 
+	    !strcmp(sfx, ".xlsx") ||
+	    !strcmp(sfx, ".gnumeric")) {
+	    if (strcmp(sfx, ".gdt")) {
+		strcpy(sfx, ".gdt");
+	    }
 	}
     }
 
