@@ -27,6 +27,7 @@
 #include "cmd_private.h"
 #include "gretl_www.h"
 #include "gretl_xml.h"
+#include "gretl_typemap.h"
 #include "database.h"
 #include "guiprint.h"
 #include "ssheet.h"
@@ -227,20 +228,33 @@ static GtkWidget *label_hbox (call_info *cinfo, GtkWidget *w,
 {
     GtkWidget *hbox, *lbl;
     gchar *label = NULL;
+    gchar *vstr = NULL;
     gchar *buf = NULL;
 
     hbox = gtk_hbox_new(FALSE, 5);
     gtk_box_pack_start(GTK_BOX(w), hbox, FALSE, FALSE, 5);
 
-    function_package_get_properties(cinfo->pkg, "label",
-				    &label, NULL);
-    buf = g_markup_printf_escaped("<span weight=\"bold\">%s</span>", 
-				  (label != NULL)? label : fallback);
+    if (cinfo->label == NULL) {
+	function_package_get_properties(cinfo->pkg, 
+					"label", &label,
+					"version", &vstr,
+					NULL);
+	cinfo->label = label;
+    } else {
+	label = cinfo->label;
+	function_package_get_properties(cinfo->pkg, 
+					"version", &vstr,
+					NULL);
+    }
+
+    buf = g_markup_printf_escaped("<span weight=\"bold\">%s %s</span>", 
+				  (label != NULL)? label : fallback,
+				  vstr);
     lbl = gtk_label_new(NULL);
     gtk_label_set_markup(GTK_LABEL(lbl), buf);
-    g_free(buf);
 
-    cinfo->label = label;
+    g_free(buf);
+    g_free(vstr);
 
     gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 5);
     gtk_widget_show(lbl);
@@ -694,7 +708,7 @@ static void launch_matrix_maker (GtkWidget *button, call_info *cinfo)
 {
     int n = n_user_matrices();
 
-    gui_new_matrix(cinfo->dlg);
+    fncall_add_matrix(cinfo->dlg);
 
     if (n_user_matrices() > n) {
 	GtkWidget *sel = g_object_get_data(G_OBJECT(button), "combo");
@@ -1261,7 +1275,7 @@ static void function_call_dialog (call_info *cinfo)
 				     (desc != NULL)? desc :
 				     parname);
 	} else {
-	    const char *astr = gretl_arg_type_name(ptype);
+	    const char *astr = gretl_type_get_name(ptype);
 
 	    if (desc != NULL && strstr(desc, astr)) {
 		argtxt = g_strdup_printf("%s", desc);
@@ -1352,7 +1366,7 @@ static void function_call_dialog (call_info *cinfo)
 	label = gtk_label_new(_("selection (or new variable)"));
 	add_table_cell(tbl, label, 1, 2, ++row);
 
-	label = gtk_label_new(gretl_arg_type_name(cinfo->rettype));
+	label = gtk_label_new(gretl_type_get_name(cinfo->rettype));
 	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
 	add_table_cell(tbl, label, 0, 1, ++row);
 
@@ -1571,50 +1585,69 @@ static void set_genr_model_from_vwin (windata_t *vwin)
     set_genr_model(vwin->data, type);
 }
 
-static int real_GUI_function_call (call_info *cinfo, PRN *prn)
+/* Compose the command line that calls a packaged function with the
+   appropriate arguments, if any, and possible assignment of the
+   return value, if any.
+*/
+
+static void compose_fncall_line (char *line, 
+				 call_info *cinfo,
+				 const char *funname,
+				 char **tmpname,
+				 int *grab_bundle)
 {
-    ExecState state;
-    GtkWidget *hbox;
-    char fnline[MAXLINE];
-    char *tmpname = NULL;
-    const char *funname;
-    gretl_bundle *bundle = NULL;
-    int grab_bundle = 0;
-    int show = 1;
-    int orig_v = dataset->v;
-    int i, err = 0;
-
-    funname = user_function_name_by_index(cinfo->iface);
-    *fnline = 0;
-
-    /* compose the function command-line */
-
+    *line = '\0';
+    
     if (cinfo->ret != NULL) {
-	strcat(fnline, cinfo->ret);
-	strcat(fnline, " = ");
+	strcat(line, cinfo->ret);
+	strcat(line, " = ");
     } else if (cinfo->rettype == GRETL_TYPE_BUNDLE) {
-	/* make a special arrangement to grab the returned
-	   bundle for GUI purposes 
+	/* the function offers a bundle return but this has not been
+	   assigned by the user; make a special arrangement to grab
+	   the bundle for GUI purposes
 	*/
-	tmpname = temp_name_for_bundle();
-	strcat(fnline, tmpname);
-	strcat(fnline, " = ");
-	grab_bundle = 1;
+	*tmpname = temp_name_for_bundle();
+	strcat(line, *tmpname);
+	strcat(line, " = ");
+	*grab_bundle = 1;
     }	
 
-    strcat(fnline, funname);
-    strcat(fnline, "(");
+    strcat(line, funname);
+    strcat(line, "(");
 
     if (cinfo->args != NULL) {
+	int i;
+
 	for (i=0; i<cinfo->n_params; i++) {
-	    strcat(fnline, cinfo->args[i]);
+	    strcat(line, cinfo->args[i]);
 	    if (i < cinfo->n_params - 1) {
-		strcat(fnline, ", ");
+		strcat(line, ", ");
 	    }
 	}
     }
 
-    strcat(fnline, ")");
+    strcat(line, ")");
+}
+
+static int real_GUI_function_call (call_info *cinfo, PRN *prn)
+{
+    windata_t *outwin = NULL;
+    ExecState state;
+    char fnline[MAXLINE];
+    char *tmpname = NULL;
+    const char *funname;
+    const char *title;
+    gretl_bundle *bundle = NULL;
+    int grab_bundle = 0;
+    int show = 1;
+    int orig_v = dataset->v;
+    int err = 0;
+
+    funname = user_function_name_by_index(cinfo->iface);
+    title = cinfo->label != NULL ? cinfo->label : funname;
+
+    compose_fncall_line(fnline, cinfo, funname,
+			&tmpname, &grab_bundle);
 
     if (!grab_bundle && strncmp(funname, "GUI", 3)) {
 	pprintf(prn, "? %s\n", fnline);
@@ -1635,24 +1668,25 @@ static int real_GUI_function_call (call_info *cinfo, PRN *prn)
 	set_genr_model_from_vwin(cinfo->vwin);
     }
 
-    hbox = cinfo->top_hbox;
+    show = !user_func_is_noprint(cinfo->func);
 
-    if (hbox != NULL) {
-	start_wait_for_output(hbox, FALSE);
+    if (close_on_OK) {
+	gtk_widget_hide(cinfo->dlg);
     }
 
-    err = gui_exec_line(&state, dataset);
-
-    if (hbox != NULL) {
-	stop_wait_for_output(hbox);
+    if (show) {
+	/* allow the "flush" mechanism to operate */
+	err = exec_line_with_output_handler(&state, dataset,
+					    title, &outwin);
+    } else {
+	/* execute "invisibly" */
+	err = gui_exec_line(&state, dataset);
     }
 
     if (cinfo->flags & MODEL_CALL) {
 	unset_genr_model();
     }
 
-    /* destroy any "ARG" vars or matrices that were created? */
-
     if (!err && cinfo->rettype == GRETL_TYPE_BUNDLE) {
 	if (grab_bundle) {
 	    bundle = get_bundle_by_name(tmpname);
@@ -1667,8 +1701,6 @@ static int real_GUI_function_call (call_info *cinfo, PRN *prn)
 	}
     }
 
-    show = !user_func_is_noprint(cinfo->func);
-
     if (!err && bundle != NULL && !show) {
 	gretl_print_reset_buffer(prn);
 	if (try_exec_bundle_print_function(bundle, prn)) {
@@ -1688,13 +1720,19 @@ static int real_GUI_function_call (call_info *cinfo, PRN *prn)
 
     if (!err && !show) {
 	gretl_print_destroy(prn);
-    } else {
-	const char *title = cinfo->label != NULL ?
-	    cinfo->label : funname;
-
+    } else if (outwin == NULL) {
+	/* output window not already in place */
 	view_buffer(prn, 80, 400, title, 
 		    (bundle == NULL)? PRINT : VIEW_BUNDLE,
 		    bundle);
+    } else {
+	/* an output window has already been opened
+	   via "flush" */
+	if (bundle != NULL) {
+	    finalize_script_output_window(VIEW_BUNDLE, bundle);
+	} else {
+	    finalize_script_output_window(0, NULL); 
+	}
     }
 
     free(tmpname);
@@ -1799,7 +1837,8 @@ static void fncall_exec_callback (GtkWidget *w, call_info *cinfo)
     }
 }
 
-static void maybe_set_gui_interface (const char *gname,
+static void maybe_set_gui_interface (fnpkg *pkg,
+				     const char *gname,
 				     call_info *cinfo)
 {
     const char *iface;
@@ -1810,6 +1849,9 @@ static void maybe_set_gui_interface (const char *gname,
 	if (iface != NULL && !strcmp(iface, gname)) {
 	    cinfo->iface = cinfo->publist[i];
 	    cinfo->flags |= SHOW_GUI_MAIN;
+	    function_package_get_properties(pkg, "name",
+					    &cinfo->label,
+					    NULL);
 	    break;
 	}
     }
@@ -1899,7 +1941,7 @@ void call_function_package (const char *fname, windata_t *vwin,
 	function_package_get_properties(pkg, "gui-main",
 					&gmain, NULL);
 	if (gmain != NULL) {
-	    maybe_set_gui_interface(gmain, cinfo);
+	    maybe_set_gui_interface(pkg, gmain, cinfo);
 	    free(gmain);
 	}
     }
@@ -1979,12 +2021,23 @@ int exec_bundle_plot_function (gretl_bundle *b, const char *aname)
     int argc, iopt = -1;
     int err = 0;
 
-    if (strchr(aname, ':') != NULL) {
-	/* extract option */
-	sscanf(aname, "%31[^:]:%d", funname, &iopt);
+    if (aname != NULL) {
+	if (strchr(aname, ':') != NULL) {
+	    /* extract option */
+	    sscanf(aname, "%31[^:]:%d", funname, &iopt);
+	} else {
+	    /* name but no option present */
+	    strcpy(funname, aname);
+	}
     } else {
-	/* no option present */
-	strcpy(funname, aname);
+	gchar *pf = get_bundle_plot_function(b);
+
+	if (pf == NULL) {
+	    return E_DATA;
+	} else {
+	    strcpy(funname, pf);
+	    g_free(pf);
+	}
     }
 
     func = get_user_function_by_name(funname);
@@ -2485,6 +2538,7 @@ static void add_package_to_menu (addon_info *addon,
 	NULL, NULL, NULL, NULL, NULL, G_CALLBACK(gfn_menu_callback)
     };
     GtkActionGroup *actions;
+    char *fixed_label = NULL;
     guint merge_id;
 
 #if PKG_DEBUG
@@ -2498,6 +2552,19 @@ static void add_package_to_menu (addon_info *addon,
     } else {
 	pkg_item.label = addon->pkgname;
     }
+    
+    if (strchr(pkg_item.label, '_')) {
+	const char *s = pkg_item.label;
+	int n = 0;
+
+	while (*s && (s = strchr(s, '_')) != NULL) {
+	    n++;
+	    s++;
+	}
+	fixed_label = malloc(strlen(pkg_item.label) + n + 1);
+	double_underscores(fixed_label, pkg_item.label);
+	pkg_item.label = fixed_label;
+    }
 
     merge_id = gtk_ui_manager_new_merge_id(vwin->ui);
 
@@ -2514,6 +2581,8 @@ static void add_package_to_menu (addon_info *addon,
     gtk_action_group_add_actions(actions, &pkg_item, 1, vwin);
     gtk_ui_manager_insert_action_group(vwin->ui, actions, 0);
     g_object_unref(actions);
+
+    free(fixed_label);
 }
 
 /* run a package's gui-precheck function to determine if
