@@ -1,4 +1,4 @@
-/* 
+/*
  *  gretl -- Gnu Regression, Econometrics and Time-series Library
  *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
  * 
@@ -39,6 +39,7 @@
 #endif
 
 #include "gretl_func.h"
+#include "gretl_typemap.h"
 
 #define PKG_DEBUG 0
 #define NENTRIES 4
@@ -837,6 +838,11 @@ static void gfn_to_script_callback (GtkWidget *w, function_info *finfo)
 
 static void gfn_to_spec_callback (GtkWidget *w, function_info *finfo)
 {
+    if (finfo->pkg == NULL) {
+	warnbox(_("Please save your package first"));
+	return;
+    }
+
     file_selector_with_parent(SAVE_GFN_SPEC, FSEL_DATA_MISC, 
 			      finfo, finfo->dlg);
 }
@@ -1673,6 +1679,11 @@ static void extra_properties_dialog (GtkWidget *w, function_info *finfo)
 	return;
     }
 
+    if (finfo->pkg == NULL) {
+	warnbox(_("Please save your package first"));
+	return;
+    }
+
     finfo->maintree = finfo->modeltree = NULL;
     finfo->currtree = finfo->alttree = NULL;
     finfo->treewin = NULL;
