@@ -71,7 +71,6 @@
 #endif
 
 /* update.c */
-extern int silent_update_query (void);
 extern int update_query (void); 
 
 /* functions private to gretl.c */
@@ -81,7 +80,6 @@ static gboolean main_popup_handler (GtkWidget *w, GdkEventButton *event,
 				    gpointer data);
 static GtkWidget *make_main_menu (void);
 static void start_R_callback (void);
-static void start_gnuplot_callback (void);
 static void auto_store (void);
 static void restore_sample_callback (void);
 static void show_sample_callback (void);
@@ -160,7 +158,6 @@ int data_status, orig_vars;
 float gui_scale;
 
 /* defaults for some options */
-int updater = FALSE;
 int winsize = FALSE;
 int main_x = -1;
 int main_y = -1;
@@ -239,7 +236,37 @@ static void new_package_callback (GtkAction *action, gpointer p)
 #ifdef ENABLE_MAILER
 static void email_data (gpointer p, guint u, GtkWidget *w)
 {
-    send_file(datafile);
+    char gdttmp[FILENAME_MAX];
+    char *title = NULL;
+    int err;
+
+    /* We need to handle the cases where (a) we have unsaved
+       data or (b) the dataset is saved as a binary file. To
+       do this we write out a temporary copy of the current
+       dataset (gzipped XML).
+    */
+
+    if (*datafile != '\0') {
+	const char *base = path_last_element(datafile);
+	int len = strcspn(base, ".");
+
+	if (len > 0) {
+	    title = g_strndup(base, len);
+	}
+    }
+    
+    if (title == NULL) {
+	sprintf(gdttmp, "%suntitled.gdt", gretl_dotdir());
+    } else {
+	sprintf(gdttmp, "%s%s.gdt", gretl_dotdir(), title);
+	g_free(title);
+    }
+
+    err = gretl_write_gdt(gdttmp, NULL, dataset, OPT_Z, 0);
+    if (!err) {
+	send_file(gdttmp);
+    }
+    gretl_remove(gdttmp);
 }
 #endif
 
@@ -515,7 +542,6 @@ int main (int argc, char **argv)
 #ifdef G_OS_WIN32
     char *callname = argv[0];
 #endif
-    int ftype = 0;
     char auxname[MAXLEN];
     char filearg[MAXLEN];
     GError *opterr = NULL;
@@ -609,7 +635,6 @@ int main (int argc, char **argv)
     }
 
     library_command_init();
-
     helpfile_init();
     session_init();
     init_fileptrs();
@@ -630,89 +655,17 @@ int main (int argc, char **argv)
 #endif
 
     if (argc > 1) {
-	/* Process what is presumably a filename argument
+	/* Record what is presumably a filename argument
 	   given on the command line (by now any options will
 	   have been extracted from the argv array).
 	*/
-	PRN *prn; 
-	int err = 0;
-
-	prn = gretl_print_new(GRETL_PRINT_STDERR, &err);
-	if (err) {
-	    exit(EXIT_FAILURE);
-	}
-
-	*datafile = '\0';
-
 #ifdef G_OS_WIN32
-	if (filename_to_win32(datafile, filearg)) {
+	if (filename_to_win32(tryfile, filearg)) {
 	    exit(EXIT_FAILURE);
 	}
 #else
-	record_filearg(datafile, filearg);
+	record_filearg(tryfile, filearg);
 #endif
-
-	/* keep a copy of input filename */
-	strcpy(tryfile, datafile);
-
-	ftype = detect_filetype(datafile, OPT_P);
-
-	switch (ftype) {
-	case GRETL_NATIVE_DATA:
-	    err = gretl_get_data(datafile, dataset, OPT_NONE, prn);
-	    break;
-	case GRETL_XML_DATA:
-	    err = gretl_read_gdt(datafile, dataset, OPT_NONE, prn);
-	    break;
-	case GRETL_CSV:
-	    err = import_csv(datafile, dataset, OPT_NONE, prn);
-	    break;
-	case GRETL_XLS:
-	case GRETL_XLSX:    
-	case GRETL_GNUMERIC:
-	case GRETL_ODS:
-	case GRETL_DTA:
-	case GRETL_SAV:
-	case GRETL_SAS:
-	case GRETL_JMULTI:
-	case GRETL_OCTAVE:
-	case GRETL_WF1:
-	    err = get_imported_data(datafile, ftype, 0);
-	    break;
-	case GRETL_SCRIPT:
-	case GRETL_SESSION:
-	    get_runfile(datafile);
-	    *datafile = '\0';
-	    break;
-	case GRETL_NATIVE_DB:
-	case GRETL_RATS_DB:  
-	case GRETL_PCGIVE_DB:
-	    strcpy(auxname, datafile);
-	    *tryfile = '\0';
-	    *datafile = '\0';
-	    maybe_fix_dbname(auxname);
-	    optdb = auxname;
-	    break;
-	case GRETL_UNRECOGNIZED:
-	default:
-	    fprintf(stderr, "%s: unrecognized file type", tryfile);
-	    exit(EXIT_FAILURE);
-	    break;
-	}
-
-	if (err == E_CANCEL) {
-	    err = 0;
-	    ftype = 0;
-	    *tryfile = '\0';
-	    *datafile = '\0';
-	}
-
-	if (ftype != GRETL_SCRIPT && err) {
-	    errmsg(err, prn);
-	    exit(EXIT_FAILURE);
-	}
-
-	gretl_print_destroy(prn);
     }
 
 #if GUI_DEBUG
@@ -738,11 +691,6 @@ int main (int argc, char **argv)
     fprintf(stderr, "done make_main_window\n");
 #endif
 
-    if (have_data()) {
-	/* redundant? */
-	set_sample_label(dataset);
-    }
-
     add_files_to_menus();
 
 #if GUI_DEBUG
@@ -762,29 +710,8 @@ int main (int argc, char **argv)
     fprintf(stderr, "done setting GUI state\n");
 #endif
 
-    if (have_data()) {
-	register_startup_data(tryfile);
-	maybe_display_string_table();
-	*tryfile = '\0';
-    }
-
-    /* opening a script or session from the command line? */
-    if (*tryfile != '\0') { 
-	if (gretl_is_pkzip_file(tryfile)) {
-	    ftype = GRETL_SESSION;
-	}
-	if (ftype == GRETL_SESSION) {
-	    do_open_session();
-	} else if ((ftype = script_type(tryfile))) {
-	    do_open_script(ftype);
-	} else {
-	    do_open_script(EDIT_SCRIPT);
-	}
-    }
-
-    /* check for program updates? */
-    if (updater) {
-	silent_update_query(); 
+    if (*tryfile != '\0') {
+	real_open_tryfile();
     }
 
     /* try opening specified database or package */
@@ -1539,38 +1466,14 @@ GtkActionEntry main_entries[] = {
     { "OpenDataMenu", NULL, N_("_Open data"), NULL, NULL, NULL }, 
     { "OpenData",     GTK_STOCK_OPEN, N_("_User file..."), NULL, NULL, G_CALLBACK(open_data) },
     { "DisplayDataFiles", GTK_STOCK_OPEN, N_("_Sample file..."), "", NULL, G_CALLBACK(show_files) },
-    { "ImportData",   NULL, N_("_Import"), NULL, NULL, NULL }, 
-    { "OpenCSV",      NULL, N_("_text/CSV..."), NULL, NULL, G_CALLBACK(open_data) },
-    { "OpenOctave",   NULL, N_("_Octave..."), NULL, NULL, G_CALLBACK(open_data) },
-    { "OpenGnumeric", NULL, N_("_Gnumeric..."), NULL, NULL, G_CALLBACK(open_data) },
-    { "OpenXLS",      NULL, N_("_Excel..."), NULL, NULL, G_CALLBACK(open_data) },
-    { "OpenODS",      NULL, N_("_Open Document..."), NULL, NULL, G_CALLBACK(open_data) },
-    { "OpenWF1",      NULL, N_("_Eviews..."), NULL, NULL, G_CALLBACK(open_data) },
-    { "OpenDTA",      NULL, N_("_Stata..."), NULL, NULL, G_CALLBACK(open_data) },
-    { "OpenSAV",      NULL, N_("_SPSS..."), NULL, NULL, G_CALLBACK(open_data) },
-    { "OpenSAS",      NULL, N_("_SAS (xport)..."), NULL, NULL, G_CALLBACK(open_data) },
-    { "OpenJMulTi",   NULL, N_("_JMulTi..."), NULL, NULL, G_CALLBACK(open_data) },
-
-    { "AppendDataMenu", NULL, N_("_Append data"), NULL, NULL, NULL }, 
-    { "AppendGdt",      NULL, N_("_Standard format..."), NULL, NULL, G_CALLBACK(open_data) },
-    { "AppendCSV",      NULL, N_("_text/CSV..."), NULL, NULL, G_CALLBACK(open_data) },
-    { "AppendOctave",   NULL, N_("_Octave..."), NULL, NULL, G_CALLBACK(open_data) },
-    { "AppendGnumeric", NULL, N_("_Gnumeric..."), NULL, NULL, G_CALLBACK(open_data) },
-    { "AppendXLS",      NULL, N_("_Excel..."), NULL, NULL, G_CALLBACK(open_data) },
-    { "AppendODS",      NULL, N_("_Open Document..."), NULL, NULL, G_CALLBACK(open_data) },
-    { "AppendWF1",      NULL, N_("_Eviews..."), NULL, NULL, G_CALLBACK(open_data) },
-    { "AppendDTA",      NULL, N_("_Stata..."), NULL, NULL, G_CALLBACK(open_data) },
-    { "AppendSAV",      NULL, N_("_SPSS..."), NULL, NULL, G_CALLBACK(open_data) },
-    { "AppendSAS",      NULL, N_("_SAS (xport)..."), NULL, NULL, G_CALLBACK(open_data) },
-    { "AppendJMulTi",   NULL, N_("_JMulTi..."), NULL, NULL, G_CALLBACK(open_data) },
-
+    { "AppendData", NULL, N_("_Append data..."), NULL, NULL, G_CALLBACK(open_data) }, 
     { "SaveData",  GTK_STOCK_SAVE, N_("_Save data"), NULL, NULL, G_CALLBACK(auto_store) },
     { "SaveDataAs", GTK_STOCK_SAVE_AS, N_("Save data _as..."), NULL, NULL, G_CALLBACK(fsave_callback) }, 
     { "ExportData", NULL, N_("_Export data..."), NULL, NULL, G_CALLBACK(fsave_callback) },
-
     { "MailData", GRETL_STOCK_MAIL, N_("Send To..."), NULL, NULL, G_CALLBACK(email_data) },
     { "NewData", GTK_STOCK_NEW, N_("_New data set"), NULL, NULL, G_CALLBACK(newdata_callback) },
     { "ClearData", GTK_STOCK_CLEAR, N_("C_lear data set"), NULL, NULL, G_CALLBACK(verify_clear_data) },
+
     { "WorkingDir", NULL, N_("_Working directory..."), NULL, NULL, G_CALLBACK(working_dir_dialog) },
     { "ScriptFiles", NULL, N_("_Script files"), NULL, NULL, NULL },
     { "OpenScript", GTK_STOCK_OPEN, N_("_User file..."), "", NULL, G_CALLBACK(open_script_callback) },
@@ -1596,11 +1499,6 @@ GtkActionEntry main_entries[] = {
     { "PcGiveDB", GTK_STOCK_OPEN, N_("_PcGive..."), "", NULL, G_CALLBACK(open_data) },
     { "RemoteDB", GTK_STOCK_NETWORK, N_("On database _server..."), NULL, NULL, G_CALLBACK(show_files) },
 
-    { "FunctionFiles", NULL, N_("_Function files"), NULL, NULL, NULL },
-    { "LocalGfn", GTK_STOCK_OPEN, N_("On _local machine..."), "", NULL, G_CALLBACK(show_files) },
-    { "RemoteGfn", GTK_STOCK_NETWORK, N_("On _server..."), NULL, NULL, G_CALLBACK(show_files) },
-    { "EditGfn", GTK_STOCK_EDIT, N_("Edit package..."), NULL, NULL, G_CALLBACK(edit_package_callback) },
-    { "NewGfn", GTK_STOCK_NEW, N_("_New package"), "", NULL, G_CALLBACK(new_package_callback) },
     { "Quit", GTK_STOCK_QUIT, NULL, NULL, NULL,  G_CALLBACK(menu_exit_check)},
 
     /* Tools */
@@ -1614,8 +1512,13 @@ GtkActionEntry main_entries[] = {
     { "SetSeed", NULL, N_("_Seed for random numbers"), NULL, NULL, G_CALLBACK(rand_seed_dialog) },
     { "CommandLog", NULL, N_("_Command log"), NULL, NULL, G_CALLBACK(view_command_log) },
     { "ShowConsole", NULL, N_("_Gretl console"), NULL, NULL, G_CALLBACK(gretl_console) },
-    { "Gnuplot", NULL, N_("_Gnuplot"), NULL, NULL, G_CALLBACK(start_gnuplot_callback) },
+    { "Gnuplot", NULL, N_("_Gnuplot"), NULL, NULL, G_CALLBACK(launch_gnuplot_interactive) },
     { "StartR", NULL, N_("Start GNU _R"), NULL, NULL, G_CALLBACK(start_R_callback) },
+    { "FunctionFiles", NULL, N_("_Function packages"), NULL, NULL, NULL },
+    { "LocalGfn", GTK_STOCK_OPEN, N_("On _local machine..."), "", NULL, G_CALLBACK(show_files) },
+    { "RemoteGfn", GTK_STOCK_NETWORK, N_("On _server..."), NULL, NULL, G_CALLBACK(show_files) },
+    { "EditGfn", GTK_STOCK_EDIT, N_("Edit package..."), NULL, NULL, G_CALLBACK(edit_package_callback) },
+    { "NewGfn", GTK_STOCK_NEW, N_("_New package"), "", NULL, G_CALLBACK(new_package_callback) },
     { "NistTest", NULL, N_("_NIST test suite"), NULL, NULL, NULL },
     { "NistBasic", NULL, N_("_Basic"), NULL, NULL, G_CALLBACK(do_nistcheck) },
     { "NistVerbose", NULL, N_("_Verbose"), NULL, NULL, G_CALLBACK(do_nistcheck) },
@@ -1704,8 +1607,8 @@ GtkActionEntry main_entries[] = {
     { "SMPLRAND", NULL, N_("R_andom sub-sample..."), NULL, NULL, G_CALLBACK(sample_range_dialog) },
     { "SampleWReplace", NULL, N_("_Resample with replacement..."), NULL, NULL, 
       G_CALLBACK(gui_resample_data) },
-    { "DropMissing", NULL, N_("Drop all obs with _missing values"), NULL, NULL, 
-      G_CALLBACK(drop_all_missing) },
+    { "DropMissing", NULL, N_("Drop observations with _missing values..."), NULL, NULL, 
+      G_CALLBACK(drop_missing_data) },
     { "CountMissing", NULL, N_("_Count missing values"), NULL, NULL, G_CALLBACK(count_missing) },
 
     /* Variable */
@@ -1812,6 +1715,7 @@ GtkActionEntry main_entries[] = {
     { "UserGuide", GRETL_STOCK_PDF, N_("_User's guide"), NULL, NULL, G_CALLBACK(display_pdf_help) },
     { "PDFCmdRef", GRETL_STOCK_PDF, N_("_Command reference"), NULL, NULL, G_CALLBACK(display_pdf_help) },
     { "KbdRef", GRETL_STOCK_PDF, N_("_Keyboard shortcuts"), NULL, NULL, G_CALLBACK(display_pdf_help) },
+    { "Primer", GRETL_STOCK_PDF, N_("_Hansl primer"), NULL, NULL, G_CALLBACK(display_pdf_help) },
     { "UpdateCheck", GTK_STOCK_NETWORK, N_("Check for _updates"), NULL, NULL, G_CALLBACK(update_query) },
     { "SFAddons", NULL, N_("Check for _addons"), NULL, NULL, G_CALLBACK(show_files) },
     { "About", GTK_STOCK_ABOUT, N_("_About gretl"), NULL, NULL, G_CALLBACK(about_dialog) }
@@ -2215,11 +2119,6 @@ static void start_R_callback (void)
     start_R(NULL, 1, 1);
 }
 
-static void start_gnuplot_callback (void)
-{
-    launch_gnuplot_interactive(NULL);
-}
-
 #ifndef G_OS_WIN32
 
 int gretl_fork (const char *progvar, const char *arg)
@@ -2417,10 +2316,9 @@ static void auto_store (void)
     } else {
 	/* ensure there's no stale selection around */
 	set_selector_storelist(NULL);
-
-	if ((data_status & USER_DATA) && has_suffix(datafile, ".gdt")) {
+	if ((data_status & USER_DATA) && has_native_data_suffix(datafile)) {
 	    /* bypass filename selection */
-	    do_store(datafile, SAVE_DATA, NULL);
+	    do_store(datafile, AUTO_SAVE_DATA, NULL);
 	} else {
 	    file_selector(SAVE_DATA, FSEL_DATA_NONE, NULL);
 	}
@@ -2445,7 +2343,7 @@ static void mdata_text_received (GtkClipboard *cb,
 	err = user_fopen(CLIPTEMP, fullname, &prn);
 
 	if (!err) {
-	    int ci = append ? APPEND_CSV : OPEN_CSV;
+	    int ci = append ? APPEND_DATA : OPEN_DATA;
 
 	    pputs(prn, text);
 	    gretl_print_destroy(prn);
@@ -2526,7 +2424,6 @@ static int script_stopper (int set)
 	/* set the stop signal */
 	stop = 1;
     } else if (stop) {
-	warnbox(_("Execution aborted by request"));
 	ret = 1;
 	stop = 0;
     }
