@@ -322,13 +322,11 @@ void gretl_win32_init (const char *progname, int debug)
     }
 }
 
-static int win_copy_buf (char *buf, int fmt)
+int prn_to_clipboard (PRN *prn, int fmt)
 {
-    HGLOBAL winclip;
-    LPTSTR ptr;
-    char *winbuf = NULL;
-    unsigned clip_format;
-    size_t len;
+    char *buf = gretl_print_steal_buffer(prn);
+    char *modbuf = NULL;
+    int err = 0;
 
     if (buf == NULL || *buf == '\0') {
 	errbox(_("Copy buffer was empty"));
@@ -342,54 +340,59 @@ static int win_copy_buf (char *buf, int fmt)
 
     EmptyClipboard();
 
-    strip_unicode_minus(buf);
+    err = maybe_post_process_buffer(buf, fmt, W_COPY, &modbuf);
 
-    if ((fmt == GRETL_FORMAT_TXT || fmt == GRETL_FORMAT_RTF_TXT) &&
-	string_is_utf8((const unsigned char *) buf)) {
-	gchar *tr = my_locale_from_utf8(buf);
+    if (!err) {
+	HGLOBAL winclip;
+	LPTSTR ptr;
+	unsigned clip_format;
+	gunichar2 *ubuf = NULL;
+	char *winbuf;
+	glong wrote = 0;
+	size_t sz;
 
-	if (tr != NULL) {
-	    winbuf = dosify_buffer(tr, fmt);
-	    g_free(tr);
+	winbuf = modbuf != NULL ? modbuf : buf;
+
+	if (!rtf_format(prn) && !gretl_is_ascii(winbuf)) {
+	    /* for Windows clipboard, recode UTF-8 to UTF-16 */
+	    ubuf = g_utf8_to_utf16(winbuf, -1, NULL, &wrote, NULL);
+	}
+
+	if (ubuf != NULL) {
+	    winbuf = ubuf;
+	    sz = (wrote + 1) * sizeof(gunichar2);
+	} else {
+	    sz = strlen(winbuf) + 1;
+	}
+
+	winclip = GlobalAlloc(GMEM_MOVEABLE, sz);	
+	ptr = GlobalLock(winclip);
+	memcpy(ptr, winbuf, sz);
+	GlobalUnlock(winclip);
+
+	if (ubuf != NULL) {
+	    clip_format = CF_UNICODETEXT;
+	} else if (rtf_format(prn)) { 
+	    clip_format = RegisterClipboardFormat("Rich Text Format");
+	} else if (fmt == GRETL_FORMAT_CSV) {
+	    clip_format = RegisterClipboardFormat("CSV");
+	} else {
+	    clip_format = CF_TEXT;
+	}
+
+	SetClipboardData(clip_format, winclip);
+
+	if (ubuf != NULL) {
+	    g_free(ubuf);
 	}
-    } else {
-	winbuf = dosify_buffer(buf, fmt);
     }
 
-    if (winbuf == NULL) {
-	CloseClipboard();
-	return 1;
-    }
-
-    len = strlen(winbuf) + 1; 
-    winclip = GlobalAlloc(GMEM_MOVEABLE, len * sizeof(TCHAR));        
-
-    ptr = GlobalLock(winclip);
-    memcpy(ptr, winbuf, len);
-    GlobalUnlock(winclip); 
-
-    if (fmt == GRETL_FORMAT_RTF || fmt == GRETL_FORMAT_RTF_TXT) { 
-	clip_format = RegisterClipboardFormat("Rich Text Format");
-    } else if (fmt == GRETL_FORMAT_CSV) {
-	clip_format = RegisterClipboardFormat("CSV");
-    } else {
-	clip_format = CF_TEXT;
-    }
-
-    SetClipboardData(clip_format, winclip);
-
     CloseClipboard();
 
-    free(winbuf);
+    free(buf);
+    free(modbuf);
 
-    return 0;
-}
-
-int prn_to_clipboard (PRN *prn, int fmt)
-{
-    char *buf = gretl_print_steal_buffer(prn);
-
-    return win_copy_buf(buf, fmt);
+    return err;
 }
 
 static char *fname_from_fullname (char *fullname)
