@@ -74,8 +74,7 @@ enum {
 static GretlToolItem help_tools[] = {
     { N_("Larger"), GTK_STOCK_ZOOM_IN, G_CALLBACK(text_larger), 0},
     { N_("Smaller"), GTK_STOCK_ZOOM_OUT, G_CALLBACK(text_smaller), 0},
-    { N_("Show English help"), GRETL_STOCK_EN, G_CALLBACK(en_help_callback), 
-      EN_ITEM },
+    { N_("Show English help"), GRETL_STOCK_EN, G_CALLBACK(en_help_callback), EN_ITEM },
     { N_("Windows"), GRETL_STOCK_WINLIST, GNULL, 0 },
     { N_("Close"), GTK_STOCK_CLOSE, G_CALLBACK(delete_help_viewer), 0 }
 };
@@ -148,6 +147,7 @@ static struct gui_help_item gui_help_items[] = {
     { GUI_FUNCS,      "gui-funcs" },
     { MENU_ATTACH,    "menu-attach" },
     { REPROBIT,       "reprobit" },
+    { DAILY_PURGE,    "daily-purge" },
     { -1,             NULL },
 };
 
@@ -1860,7 +1860,8 @@ enum {
     GRETL_REF,
     GNUPLOT_REF,
     X12A_REF,
-    GRETL_KEYS
+    GRETL_KEYS,
+    HANSL_PRIMER
 };
 
 static int get_writable_doc_path (char *path, const char *fname)
@@ -1946,7 +1947,11 @@ static int get_x12a_doc_path (char *path, const char *fname)
 #if !defined(G_OS_WIN32) && !defined(OS_OSX)
 	if (!ret) {
 	    /* using gretl x12a package? */
-	    sprintf(path, "/opt/x12arima/docs/%s", fname);
+	    if (gretl_x12_is_x13()) {
+		sprintf(path, "/opt/x13as/docs/%s", fname);
+	    } else {
+		sprintf(path, "/opt/x12arima/docs/%s", fname);
+	    }
 	    ret = 1;
 	}
 #endif
@@ -1973,8 +1978,11 @@ static int find_or_download_pdf (int code, int i, char *fullpath)
 	"gretl-keys.pdf",
 	"gretl-keys-a4.pdf"
     };
+    const char *primer_files[] = {
+	"hansl-primer.pdf",
+	"hansl-primer-a4.pdf"
+    };    
     const char *fname;
-    char *tmp = NULL;
     FILE *fp;
     int gotit = 0;
     int err = 0;
@@ -1983,21 +1991,29 @@ static int find_or_download_pdf (int code, int i, char *fullpath)
 	i = 0;
     }
 
+    if (code == GRETL_KEYS || code == HANSL_PRIMER) {
+	/* no translations */
+	if (i > 1) i = 1;
+    }
+
     if (code == GRETL_GUIDE) {
 	fname = guide_files[i];
     } else if (code == GRETL_REF) {
 	fname = ref_files[i];
     } else if (code == GRETL_KEYS) {
-	if (i > 1) i = 1; /* no translations */
 	fname = kbd_files[i];
+    } else if (code == HANSL_PRIMER) {
+	fname = primer_files[i];
     } else if (code == GNUPLOT_REF) {
 	fname = "gnuplot.pdf";
     } else if (code == X12A_REF) {
-	fname = "x12adocV03.pdf";
+	fname = gretl_x12_is_x13() ? "docX13AS.pdf" : "x12adocV03.pdf";
     } else {
 	return E_DATA;
     }
 
+    fprintf(stderr, "pdf help: looking for %s\n", fname);
+
     /* is the file available in public dir? */
     sprintf(fullpath, "%sdoc%c%s", gretl_home(), SLASH, fname);
 
@@ -2048,8 +2064,6 @@ static int find_or_download_pdf (int code, int i, char *fullpath)
 	}
     }
 
-    free(tmp);
-
     return err;
 }
 
@@ -2074,11 +2088,11 @@ void display_pdf_help (GtkAction *action)
 	    code = GRETL_REF;
 	} else if (!strcmp(gtk_action_get_name(action), "KbdRef")) {
 	    code = GRETL_KEYS;
+	} else if (!strcmp(gtk_action_get_name(action), "Primer")) {
+	    code = HANSL_PRIMER;
 	}
     }
 
-    fprintf(stderr, "display_pdf_help: code = %d\n", code);
-
     err = find_or_download_pdf(code, get_manpref(), fname);
 
     if (!err) {
