@@ -132,7 +132,7 @@ void dataset_menubar_state (gboolean s)
 {
     if (mdata == NULL || mdata->ui == NULL) return;
 
-    flip(mdata->ui, "/menubar/File/AppendDataMenu", s);
+    flip(mdata->ui, "/menubar/File/AppendData", s);
     flip(mdata->ui, "/menubar/File/ClearData", s);
     flip(mdata->ui, "/menubar/File/SaveData", s);
     flip(mdata->ui, "/menubar/File/SaveDataAs", s);
@@ -680,9 +680,11 @@ static const char *get_pd_string (DATASET *dset)
 	case 52:
 	    pdstr = N_("Weekly"); break;
 	case 5:
+	    pdstr = N_("Daily (5 days)"); break;
 	case 6:
+	    pdstr = N_("Daily (6 days)"); break;
 	case 7:
-	    pdstr = N_("Daily"); break;
+	    pdstr = N_("Daily (7 days)"); break;
 	case 10:
 	    pdstr = N_("Decennial"); break;
 	default:
@@ -737,12 +739,17 @@ void set_sample_label (DATASET *dset)
 	char t1str[OBSLEN], t2str[OBSLEN];
 	const char *pdstr = get_pd_string(dset);
 
-	if (tsubset && calendar_data(dset)) {
+	if (calendar_data(dset) && tsubset) {
 	    /* it's too verbose to print both full range and sample */
 	    ntodate(t1str, dset->t1, dset);
 	    ntodate(t2str, dset->t2, dset);
 	    sprintf(tmp, _("%s; sample %s - %s"), _(pdstr), t1str, t2str);
 	    gtk_label_set_text(GTK_LABEL(mdata->status), tmp);
+	} else if (calendar_data(dset) && complex_subsampled()) {
+	    /* ditto, too verbose */
+	    sprintf(tmp, _("%s; sample %s - %s"), _(pdstr), dset->stobs, 
+		    dset->endobs);
+	    gtk_label_set_text(GTK_LABEL(mdata->status), tmp);
 	} else {
 	    ntodate(t1str, 0, dset);
 	    ntodate(t2str, dset->n - 1, dset);
