@@ -24,6 +24,6 @@ int maybe_save_graph (const char *name, int ci, PRN *prn);
 
 int save_text_buffer (const char *name, PRN *prn);
 
-int saved_object_action (const char *line, PRN *prn);
+int gui_saved_object_action (const char *line, PRN *prn);
 
 #endif /* OBJECTSAVE_H */
