@@ -33,6 +33,7 @@
 #include "libset.h"
 #include "texprint.h"
 #include "uservar.h"
+#include "gretl_foreign.h"
 
 #include <unistd.h>
 #include <sys/types.h>
@@ -48,9 +49,11 @@
 # include <windows.h>
 # include "gretlwin32.h"
 #else
+#if 0 /* not needed? */
 # include <sys/stat.h>
 # include <fcntl.h>
 # include <errno.h>
+#endif
 # if HAVE_GTK_FONT_CHOOSER
 #  include "fontfilter.h"
 # else
@@ -106,6 +109,7 @@ static int session_prompt = 1;
 static int keep_folder = 1;
 static int tabbed_editor;
 static int tabbed_models;
+static int script_output_policy;
 char gpcolors[64];
 static char datapage[24] = "Gretl";
 static char scriptpage[24] = "Gretl";
@@ -117,6 +121,14 @@ static char hc_tseri[5] = "HAC";
 static char hc_panel[9] = "Arellano";
 static char hc_garch[5] = "QML";
 
+#ifdef HAVE_MPI
+# ifdef G_OS_WIN32
+static char mpi_pref[8] = "MS-MPI";
+# else
+static char mpi_pref[8] = "OpenMPI";
+# endif
+#endif
+
 static int lcnumeric = 1;
 static double graph_scale = 1.0;
 
@@ -170,10 +182,6 @@ RCVAR rc_vars[] = {
       MACHSET | BROWSER, sizeof paths.gretldir, TAB_MAIN, NULL },
     { "userdir", N_("User's gretl directory"), NULL, paths.workdir, 
       INVISET, sizeof paths.workdir, TAB_MAIN, NULL },
-#if 0 /* not working anyway */
-    { "updater", N_("Tell me about gretl updates"), NULL, &updater, 
-      BOOLSET, 0, TAB_MAIN, NULL },
-#endif
 #ifndef G_OS_WIN32
     { "winsize", N_("Remember main window size"), NULL, &winsize, 
       BOOLSET, 0, TAB_MAIN, NULL },
@@ -262,6 +270,14 @@ RCVAR rc_vars[] = {
       MACHSET | BROWSER, sizeof paths.statapath, TAB_PROGS, NULL},
     { "python", N_("Path to Python executable"), NULL, paths.pypath, 
       MACHSET | BROWSER, sizeof paths.statapath, TAB_PROGS, NULL},
+#ifdef HAVE_MPI
+    { "mpiexec", N_("Path to mpiexec"), NULL, paths.mpiexec, 
+      MACHSET | BROWSER, sizeof paths.mpiexec, TAB_MPI, NULL},
+    { "mpi_hosts", N_("Path to MPI hosts file"), NULL, paths.mpi_hosts, 
+      MACHSET | BROWSER, sizeof paths.mpi_hosts, TAB_MPI, NULL},
+    { "mpi_pref", N_("Installed MPI variant"), NULL, mpi_pref, 
+      LISTSET, 8, TAB_MPI, NULL},
+#endif
     { "dbhost", N_("Database server name"), NULL, paths.dbhost, 
       USERSET, sizeof paths.dbhost, TAB_NET, NULL },
     { "dbproxy", N_("HTTP proxy"), NULL, http_proxy, 
@@ -294,6 +310,8 @@ RCVAR rc_vars[] = {
       INVISET | INTSET, 0, TAB_NONE, NULL },
     { "main_y", "main window y position", NULL, &main_y, 
       INVISET | INTSET, 0, TAB_NONE, NULL },
+    { "script_output_policy", "stickiness of output", NULL, &script_output_policy, 
+      INVISET | INTSET, 0, TAB_NONE, NULL },
     { "HC_by_default", N_("Use robust covariance matrix by default"), NULL,
       &hc_by_default, BOOLSET, 0, TAB_VCV, NULL },
     { "HC_xsect", N_("For cross-sectional data"), NULL, hc_xsect, 
@@ -379,6 +397,23 @@ int get_keep_folder (void)
     return keep_folder;
 }
 
+void set_script_output_policy (int p, windata_t *vwin)
+{
+    script_output_policy = p;
+
+    if (script_output_policy < 0 || 
+	script_output_policy > OUTPUT_APPEND) {
+	script_output_policy = OUTPUT_NEW_WINDOW;
+    }
+
+    set_reuseable_output_window(p, vwin);
+}
+
+int get_script_output_policy (void)
+{
+    return script_output_policy;
+}
+
 #ifdef G_OS_WIN32
 
 void set_use_wimp (int s)
@@ -393,12 +428,12 @@ int get_use_wimp (void)
 
 #endif
 
-static gretlopt set_paths_opt = OPT_NONE;
+static gretlopt update_paths_opt = OPT_NONE;
 
 void force_english_help (void)
 {
-    set_paths_opt |= OPT_N;
-    gretl_update_paths(&paths, set_paths_opt);
+    update_paths_opt |= OPT_N;
+    gretl_update_paths(&paths, update_paths_opt);
 }
 
 void set_fixed_font (const char *fontname)
@@ -499,7 +534,6 @@ void set_app_font (const char *fontname)
 	}
 	g_free(deffont);
 	if (noop) {
-	    fprintf(stderr, "set_app_font: no-op\n");
 	    return;
 	}
     }
@@ -653,6 +687,7 @@ static const char *get_reg_base (const char *key)
 #endif
 
 #ifdef OS_OSX
+
 static int alt_ok (const char *prog)
 {
     char *p, test[MAXSTR];
@@ -671,7 +706,7 @@ static int alt_ok (const char *prog)
          strcat(test, "x12arima/x12a");
     }
 
-    ok = check_for_prog(test);
+    ok = check_for_program(test);
     
     if (ok) {
         if (tr) {
@@ -683,6 +718,7 @@ static int alt_ok (const char *prog)
     
     return ok;
 }
+
 #endif
 
 #ifdef HAVE_TRAMO
@@ -706,7 +742,7 @@ static void set_tramo_status (void)
     if (*tramodir != '\0') {
 	const char *tramo = gretl_tramo();
 
-	ok = check_for_prog(tramo);
+	ok = check_for_program(tramo);
 # ifdef OS_OSX
 	if (!ok) {
 	    ok = alt_ok(tramo);
@@ -749,7 +785,7 @@ static void set_x12a_status (void)
     if (*x12adir != '\0') {
 	const char *x12a = gretl_x12_arima();
 
-	ok = check_for_prog(x12a);
+	ok = check_for_program(x12a);
 # ifdef OS_OSX    
 	if (!ok) {
 	    ok = alt_ok(x12a);
@@ -771,136 +807,7 @@ static void set_x12a_status (void)
 
 #endif /* HAVE_X12A */
 
-#ifdef G_OS_WIN32
-
-int check_for_prog (const char *prog)
-{
-    char tmp[MAXLEN];
-    WIN32_FIND_DATA find_data;
-    HANDLE hfind;
-    int ret = 1;
-
-    if (prog == NULL || *prog == '\0') {
-	return 0;
-    }
-
-    hfind = FindFirstFile(prog, &find_data);
-    if (hfind == INVALID_HANDLE_VALUE) {
-	ret = 0;
-    }
-    FindClose(hfind);
-
-    if (ret == 0) {
-	char *p;
-
-	ret = SearchPath(NULL, prog, NULL, MAXLEN, tmp, &p);
-    }
-
-    return ret;
-}
-
-#else /* !G_OS_WIN32 */
-
-int is_executable (const char *s, uid_t myid, gid_t mygrp)
-{
-    struct stat buf;
-    int ok = 0;
-
-    if (gretl_stat(s, &buf) != 0) {
-	return 0;
-    }
-
-    if (buf.st_mode & (S_IFREG|S_IFLNK)) {
-	if (buf.st_uid == myid && (buf.st_mode & S_IXUSR)) {
-	    ok = 1;
-	} else if (buf.st_gid == mygrp && (buf.st_mode & S_IXGRP)) {
-	    ok = 1;
-	} else if (buf.st_uid != myid && buf.st_gid != mygrp &&
-		   (buf.st_mode & S_IXOTH)) {
-	    ok = 1;
-	}
-    }
-
-    return ok;
-}
-
-int check_for_prog (const char *prog)
-{
-    uid_t myid = getuid();
-    gid_t mygrp = getgid();
-    char *path;
-    char *pathcpy;
-    char **dirs;
-    char *fullpath;
-    char *p;
-    int max_dlen = 0;
-    int found = 0;
-    int i, ndirs;
-
-    if (prog != NULL && *prog == '/') {
-	return is_executable(prog, myid, mygrp);
-    }
-
-    path = getenv("PATH");
-    if (path == NULL || *path == '\0') {
-	return 0;
-    }
-
-    pathcpy = gretl_strdup(path);
-    if (pathcpy == NULL) {
-	return 0;
-    }
-
-    ndirs = 1;
-    p = pathcpy;
-    while (*p) {
-	if (*p == ':') ndirs++;
-	p++;
-    }
-
-    dirs = malloc(ndirs * sizeof *dirs);
-    if (dirs == NULL) {
-	free(pathcpy);
-	return 0;
-    }
-
-    if (ndirs == 1) {
-	dirs[0] = pathcpy;
-	max_dlen = strlen(pathcpy);
-    } else {
-	for (i=0; i<ndirs; i++) {
-	    int dlen;
-
-	    dirs[i] = strtok((i == 0)? pathcpy : NULL, ":");
-	    if (dirs[i] == NULL) {
-		ndirs = i;
-		break;
-	    }
-	    dlen = strlen(dirs[i]);
-	    if (dlen > max_dlen) {
-		max_dlen = dlen;
-	    }
-	}
-    }
-
-    if (ndirs == 0 || 
-	(fullpath = malloc(max_dlen + strlen(prog) + 2)) == NULL) {
-	free(dirs);
-	free(pathcpy);
-	return 0;
-    }
-
-    for (i=0; i<ndirs && !found; i++) { 
-	sprintf(fullpath, "%s/%s", dirs[i], prog);
-	found = is_executable(fullpath, myid, mygrp);
-    }
-
-    free(dirs);
-    free(pathcpy);
-    free(fullpath);
-
-    return found;
-}
+#ifndef G_OS_WIN32
 
 static void root_check (void)
 {
@@ -929,7 +836,7 @@ int gretl_config_init (void)
     return err;
 }
 
-#endif /* *nix versus Windows */
+#endif /* !G_OS_WIN32 */
 
 static void highlight_options_entry (const char *vname)
 {
@@ -994,6 +901,9 @@ int options_dialog (int page, const char *varname, GtkWidget *parent)
     make_prefs_tab(notebook, TAB_MAIN);
     make_prefs_tab(notebook, TAB_NET);
     make_prefs_tab(notebook, TAB_PROGS);
+#ifdef HAVE_MPI
+    make_prefs_tab(notebook, TAB_MPI);
+#endif
     make_prefs_tab(notebook, TAB_VCV);
     make_prefs_tab(notebook, TAB_MAN);
 
@@ -1075,6 +985,13 @@ static void browse_button_callback (GtkWidget *w, RCVAR *rc)
     GtkWidget *parent = g_object_get_data(G_OBJECT(w), "parent");
     int code = SET_PROG;
 
+#ifdef HAVE_MPI
+    if (!strcmp(rc->key, "mpi_hosts")) {
+	file_selector_with_parent(SET_OTHER, FSEL_DATA_MISC, rc->var, parent);
+	return;
+    }
+#endif
+
     if (strstr(rc->description, "directory") != NULL) {
 	code = SET_DIR;
     }
@@ -1173,6 +1090,23 @@ static const char **get_list_setting_strings (void *var, int *n)
 	*n = sizeof garch_strs / sizeof garch_strs[0];
     } 
 
+#ifdef HAVE_MPI
+    else if (var == mpi_pref) {
+# ifdef G_OS_WIN32
+	static const char *mpi_strs[] = {
+	    "MS-MPI" /* FIXME? */
+	};
+# else
+	static const char *mpi_strs[] = {
+	    "OpenMPI", "MPICH"
+	};
+#endif
+
+	strs = mpi_strs;
+	*n = sizeof mpi_strs / sizeof mpi_strs[0];
+    }
+#endif
+
 #ifdef MAC_THEMING
     else if (var == themepref) {
 	static const char *theme_strs[] = {
@@ -1275,6 +1209,8 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 	w = gtk_label_new(_("Network"));
     } else if (tab == TAB_PROGS) {
 	w = gtk_label_new(_("Programs"));
+    } else if (tab == TAB_MPI) {
+	w = gtk_label_new(_("MPI"));
     } else if (tab == TAB_VCV) {
 	w = gtk_label_new(_("HCCME"));
     } else if (tab == TAB_MAN) {
@@ -1679,10 +1615,30 @@ static void rcvar_set_int (RCVAR *rcvar, int ival, int *changed)
     }
 }
 
+static int blank_ok (RCVAR *rcvar)
+{
+    if (!strcmp(rcvar->key, "mpi_hosts")) {
+	return 1;
+    } else if (!strcmp(rcvar->key, "dbproxy")) {
+	return 1;
+    } else {
+	return 0;
+    }
+}
+
 static void rcvar_set_string (RCVAR *rcvar, const char *sval, int *changed)
 {
     char *strvar = (char *) rcvar->var;
 
+    if (sval == NULL && blank_ok(rcvar)) {
+	/* allow "blanking out" of the item */
+	if (*strvar != '\0') {
+	    flag_changed(rcvar, changed);
+	    *strvar = '\0';
+	}
+	return;
+    }
+
     if (sval != NULL && *sval != '\0' && strcmp(sval, strvar)) {
 	flag_changed(rcvar, changed);
 	*strvar = '\0';
@@ -1722,15 +1678,11 @@ static void apply_changes (GtkWidget *widget, gpointer data)
 	    int bval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
 
 	    rcvar_set_int(rcvar, bval, &changed);
-	} else if ((rcvar->flags & USERSET) || 
-		   (rcvar->flags & MACHSET) ||
-		   (rcvar->flags & MACHSET)) {
-	    gchar *str = entry_box_get_trimmed_text(w);
+	} else if (rcvar->flags & (USERSET | MACHSET)) {
+	    gchar *sval = entry_box_get_trimmed_text(w);
 
-	    if (str != NULL) {
-		rcvar_set_string(rcvar, str, &changed);
-		g_free(str);
-	    } 
+	    rcvar_set_string(rcvar, sval, &changed);
+	    g_free(sval);
 	} else if (rcvar->flags & LISTSET) {
 	    GtkComboBox *box = GTK_COMBO_BOX(w);
 
@@ -1739,14 +1691,14 @@ static void apply_changes (GtkWidget *widget, gpointer data)
 
 		rcvar_set_int(rcvar, ival, &changed);
 	    } else {
-		gchar *str = combo_box_get_active_text(box);
+		gchar *sval = combo_box_get_active_text(box);
 
 		if (rcvar->flags & FLOATSET) {
-		    rcvar_set_double(rcvar, str, &changed);
+		    rcvar_set_double(rcvar, sval, &changed);
 		} else {
-		    rcvar_set_string(rcvar, str, &changed);
+		    rcvar_set_string(rcvar, sval, &changed);
 		}
-		g_free(str);
+		g_free(sval);
 	    }
 	}
     }
@@ -1759,6 +1711,10 @@ static void apply_changes (GtkWidget *widget, gpointer data)
     maybe_revise_tramo_x12a_status();
 #endif
 
+#ifdef HAVE_MPI
+    set_mpi_variant(mpi_pref);
+#endif
+
     if (use_proxy && *http_proxy == '\0') {
 	/* fix inconsistency */
 	use_proxy = 0;
@@ -1837,7 +1793,7 @@ int write_rc (void)
     rc_save_file_lists(fp);
     fclose(fp);
 
-    gretl_update_paths(&paths, set_paths_opt);
+    gretl_update_paths(&paths, update_paths_opt);
 
     return 0;
 }
@@ -1932,7 +1888,7 @@ static int common_read_rc_setup (void)
     set_panel_hccme(hc_panel);
     set_garch_robust_vcv(hc_garch);
 
-    err = gretl_set_paths(&paths, set_paths_opt);
+    err = gretl_set_paths(&paths);
     if (err) {
 	/* tell the user, then turn off the special alarm */
 	gui_errmsg(err);
@@ -2079,6 +2035,11 @@ static void win32_read_gretlrc (void)
 
     fp = gretl_fopen(rcfile, "r");
 
+#if 1
+    fprintf(stderr, "rcfile: '%s' (%s)\n", rcfile,
+	    fp == NULL ? "not found" : "found");
+#endif
+
     if (fp == NULL) {
 	/* not necessarily an error: may be starting from scratch */
 	return;
@@ -2166,7 +2127,7 @@ int read_win32_config (int debug)
 				  value);
 	} 
 
-	if (debug) {
+	if (debug && *value != '\0') {
 	    fprintf(stderr, "reg: err = %d, '%s' -> '%s'\n", regerr, rcvar->key,
 		    value);
 	}
