@@ -57,6 +57,8 @@ gchar *textview_get_current_line (GtkWidget *view);
 
 int textview_set_text (GtkWidget *view, const gchar *text);
 
+void textview_append_text (GtkWidget *view, const char *text);
+
 int textview_set_text_selected (GtkWidget *view, const gchar *text);
 
 int textview_set_cursor_at_line (GtkWidget *view, int line);
@@ -115,4 +117,6 @@ void textview_add_processing_message (GtkWidget *view);
 
 void textview_delete_processing_message (GtkWidget *view);
 
+void textview_format_paragraph (GtkWidget *view);
+
 #endif /* TEXTBUF_H */
