@@ -344,6 +344,21 @@ void text_replace (GtkWidget *w, windata_t *vwin)
     replace_string_dialog(vwin);
 }
 
+static int prep_prn_for_file_save (PRN *prn, int fmt)
+{
+    const char *orig = gretl_print_get_buffer(prn);
+    char *modbuf;
+    int err;
+    
+    err = maybe_post_process_buffer(orig, fmt, W_SAVE, &modbuf);
+
+    if (!err && modbuf != NULL) {
+	gretl_print_replace_buffer(prn, modbuf);
+    }
+
+    return err;
+}
+
 static int special_text_handler (windata_t *vwin, guint fmt, int what)
 {
     int cmd = vwin->role;
@@ -386,6 +401,7 @@ static int special_text_handler (windata_t *vwin, guint fmt, int what)
 				  get_tex_eqn_opt(), 
 				  prn);
 	} else {
+	    /* RTF or CSV */
 	    err = printmodel(pmod, dataset, OPT_NONE, prn);
 	}
     } else if (cmd == VAR || cmd == VECM) {
@@ -399,7 +415,8 @@ static int special_text_handler (windata_t *vwin, guint fmt, int what)
 	    err = E_DATA;
 	} else {
 	    GRETL_VAR *var = (GRETL_VAR *) parent->data;
-	    int h = vwin->active_var; /* here records preferred horizon */
+	    /* here active_var records preferred horizon */
+	    int h = vwin->active_var;
 
 	    if (cmd == VAR_IRF) {
 		gretl_VAR_print_all_impulse_responses(var, dataset, h, prn);
@@ -415,6 +432,10 @@ static int special_text_handler (windata_t *vwin, guint fmt, int what)
 	err = special_print_model_table(prn);
     } 
 
+    if (!err && what == W_SAVE) {
+	err = prep_prn_for_file_save(prn, fmt);
+    }
+
     if (err) {
 	gui_errmsg(err);
     } else {
@@ -533,27 +554,6 @@ static gchar *text_window_get_copy_buf (windata_t *vwin, int select)
     return cpy;
 }
 
-static gchar *maybe_amend_save_buffer (gchar *inbuf, int fmt)
-{
-    const gchar *cset;
-    gchar *outbuf = inbuf;
-
-    if (!g_get_charset(&cset)) {
-	/* not native UTF-8 */ 
-	strip_unicode_minus(inbuf);
-	outbuf = my_locale_from_utf8(inbuf);
-	free(inbuf);
-	inbuf = outbuf;
-    }     
-    
-    if (fmt == GRETL_FORMAT_RTF_TXT) {
-	outbuf = dosify_buffer(inbuf, fmt);
-	free(inbuf);
-    }
-
-    return outbuf;
-}
-
 int multiple_formats_ok (windata_t *vwin)
 {
     int r = vwin->role;
@@ -575,6 +575,33 @@ int multiple_formats_ok (windata_t *vwin)
     }
 }
 
+static PRN *make_prn_for_buf (gchar *buf, int fmt, int action,
+			      int *err)
+{
+    char *modbuf = NULL;
+    PRN *prn = NULL;
+
+    if (action == W_SAVE) {
+	*err = maybe_post_process_buffer(buf, fmt, action, &modbuf);
+    }
+
+    if (*err) {
+	g_free(buf);
+    } else {
+	if (modbuf != NULL) {
+	    prn = gretl_print_new_with_buffer(modbuf);
+	    g_free(buf);
+	} else {
+	    prn = gretl_print_new_with_buffer(buf);
+	}
+	if (prn == NULL) {
+	    *err = E_ALLOC;
+	}
+    }
+
+    return prn;
+}
+
 /* copying text from gretl windows */
 
 #define SPECIAL_FORMAT(f) ((f & GRETL_FORMAT_TEX) || \
@@ -582,7 +609,7 @@ int multiple_formats_ok (windata_t *vwin)
 
 static void window_copy_or_save (windata_t *vwin, guint fmt, int action) 
 {
-    gchar *cpybuf = NULL;
+    gchar *buf = NULL;
 
     if (vwin->role == VIEW_MODEL && fmt == GRETL_FORMAT_CSV) {
 	special_text_handler(vwin, fmt, action);
@@ -592,34 +619,32 @@ static void window_copy_or_save (windata_t *vwin, guint fmt, int action)
 	       fmt == GRETL_FORMAT_RTF) {
 	copy_vars_formatted(vwin, fmt, action);
     } else if (fmt == GRETL_FORMAT_TXT || fmt == GRETL_FORMAT_RTF_TXT) {
-	cpybuf = text_window_get_copy_buf(vwin, 0);
+	buf = text_window_get_copy_buf(vwin, 0);
     } else if (fmt == GRETL_FORMAT_SELECTION) {
-	cpybuf = text_window_get_copy_buf(vwin, 1);
+	buf = text_window_get_copy_buf(vwin, 1);
 	fmt = GRETL_FORMAT_TXT;
     }
 
-    if (cpybuf != NULL) {
-	PRN *textprn;
+    if (buf != NULL) {
+	/* handle the last two cases above */
+	PRN *prn;
+	int err = 0;
 
-	if (action == W_SAVE) {
-	    cpybuf = maybe_amend_save_buffer(cpybuf, fmt);
-	    if (cpybuf == NULL) {
-		return;
+	prn = make_prn_for_buf(buf, fmt, action, &err);
+
+	if (!err) {
+	    if (action == W_COPY) {
+		prn_to_clipboard(prn, fmt);
+	    } else {
+		/* saving to file */
+		int fcode = (fmt == GRETL_FORMAT_RTF_TXT)? 
+		    SAVE_RTF : SAVE_OUTPUT;
+
+		file_selector_with_parent(fcode, FSEL_DATA_PRN, prn, 
+					  vwin_toplevel(vwin));
 	    }
+	    gretl_print_destroy(prn);
 	}
-
-	textprn = gretl_print_new_with_buffer(cpybuf);
-
-	if (action == W_COPY) {
-	    prn_to_clipboard(textprn, fmt);
-	} else {
-	    int fcode = (fmt == GRETL_FORMAT_RTF_TXT)? 
-		SAVE_RTF : SAVE_OUTPUT;
-
-	    file_selector_with_parent(fcode, FSEL_DATA_PRN, textprn, 
-				      vwin_toplevel(vwin));
-	}
-	gretl_print_destroy(textprn);
     }	
 }
 
@@ -749,34 +774,37 @@ void system_print_buf (const gchar *buf, FILE *fp)
 
 /* Convert a buffer to DOS/Windows text format, optionally
    adding minimal RTF formatting. This function does not
-   take charge of text re-encoding, it just handles line
+   take charge of text re-encoding, it just handles CR + LF
    endings and (optional) RTF monospaced font spec.
 */
 
-char *dosify_buffer (const char *buf, int format)
+static char *dosify_buffer (const char *buf, int format)
 {
+    const char *rtf_preamble = "{\\rtf1\r\n"
+	"{\\fonttbl{\\f0\\fnil\\fprq1\\fcharset1 Courier New;}}\r\n"
+	"\\f0\\fs18\r\n";
     int extra = 0, nlines = 0;
-    int rtf = (format == GRETL_FORMAT_RTF_TXT);
+    int add_rtf = 0;
     char *targ, *q;
     const char *p;
-    const char *rtf_preamble = "{\\rtf1\n"
-	"{\\fonttbl{\\f0\\fnil\\fprq1\\fcharset1 Courier New;}}\n"
-	"\\f0\\fs18\n";
-    int rtf_add_bytes = strlen(rtf_preamble) + 4;
 
     if (buf == NULL || *buf == '\0') {
 	return NULL;
     }
 
+    if (format == GRETL_FORMAT_RTF_TXT) {
+	add_rtf = 1;
+    }
+
     p = buf;
     while (*p) {
 	if (*p++ == '\n') nlines++;
     }
     extra = nlines + 1;
 
-    if (rtf) {
+    if (add_rtf) {
 	extra *= 5;
-	extra += rtf_add_bytes;
+	extra += strlen(rtf_preamble) + 5;
     }
 
     targ = malloc(strlen(buf) + extra);
@@ -784,7 +812,7 @@ char *dosify_buffer (const char *buf, int format)
 	return NULL;
     }
 
-    if (rtf) {
+    if (add_rtf) {
 	strcpy(targ, rtf_preamble);
 	q = targ + strlen(targ);
     } else {
@@ -792,9 +820,19 @@ char *dosify_buffer (const char *buf, int format)
     }
 
     p = buf;
+
     while (*p) {
-	if (*p == '\n') {
-	    if (rtf) {
+	int pplus = 1;
+	int nl = 0;
+
+	if (*p == '\r' && *(p+1) == '\n') {
+	    nl = 1; pplus = 2;
+	} else if (*p == '\n') {
+	    nl = 1;
+	}
+
+	if (nl) {
+	    if (add_rtf) {
 		*q++ = '\\';
 		*q++ = 'p';
 		*q++ = 'a';
@@ -805,15 +843,126 @@ char *dosify_buffer (const char *buf, int format)
 	} else {
 	    *q++ = *p;
 	}
-	p++;
-    } 
-    *q = 0;
 
-    if (rtf) {
-	strcat(q, "}\n");
+	p += pplus;
+    }
+ 
+    *q = '\0';
+
+    if (add_rtf) {
+	strcat(q, "}\r\n");
     }
 
     return targ;
 }
 
+#ifdef G_OS_WIN32
+
+#define plain_text(f) (f & (GRETL_FORMAT_TXT | GRETL_FORMAT_CSV | GRETL_FORMAT_TAB))
+
+static int want_bom (int fmt, int action)
+{
+    /* Note sure about this, but for now if we're saving 
+       "plain text" to file in UTF-8, we'll leave it in
+       UTF-8 and prepend the UTF-8 BOM.
+    */
+    if (action == W_SAVE && fmt == GRETL_FORMAT_TXT) {
+	return 1;
+    } else {
+	return 0;
+    }
+}
+
+static char *prepend_bom (const char *orig)
+{
+    char *buf = malloc(strlen(orig) + 4);
+
+    if (buf != NULL) {
+	buf[0] = 0xEF;
+	buf[1] = 0xBB;
+	buf[2] = 0xBF;
+	buf[3] = 0;
+	strcat(buf, orig);
+    }
+ 
+    return buf;
+}
+
+#endif
+
+int maybe_post_process_buffer (const char *buf, int fmt, 
+			       int action, char **modbuf)
+{
+    int rtf_output = 0;
+    int utf8_coded = 0;
+    char *trbuf = NULL;
+    char *final = NULL;
+    int err = 0;
+
+    if (fmt & (GRETL_FORMAT_RTF | GRETL_FORMAT_RTF_TXT)) {
+	rtf_output = 1;
+    }
+
+    if (string_is_utf8((const unsigned char *) buf)) {
+	utf8_coded = 1;
+    }
+
+    if (rtf_output) {
+	/* When writing RTF, recode if required and ensure
+	   CR + LF.
+	*/
+	if (utf8_coded) {
+	    trbuf = utf8_to_rtf(buf);
+	    if (trbuf == NULL) {
+		err = E_ALLOC;
+	    }
+	}
+	if (!err) {
+	    if (trbuf != NULL) {
+		final = dosify_buffer(trbuf, fmt);
+	    } else {
+		final = dosify_buffer(buf, fmt);
+	    }
+	    if (final == NULL) {
+		err = E_ALLOC;
+	    }
+	}
+	goto finish;
+    }
+
+#ifdef G_OS_WIN32
+    if (plain_text(fmt)) {
+	if (utf8_coded && want_bom(fmt, action)) {
+	    trbuf = prepend_bom(buf);
+	    if (trbuf == NULL) {
+		err = E_ALLOC;
+	    }	    
+	}
+	if (!err && action == W_COPY) {
+	    if (trbuf != NULL) {
+		final = dosify_buffer(trbuf, fmt);
+	    } else {
+		final = dosify_buffer(buf, fmt);
+	    }
+	    if (final == NULL) {
+		err = E_ALLOC;
+	    }
+	} else {
+	    final = trbuf;
+	}
+    }
+#endif
+
+ finish:
+
+    if (trbuf != NULL && trbuf != final) {
+	free(trbuf);
+    }
+
+    *modbuf = final;
+	
+    return err;
+}
+
+
 
