@@ -106,7 +106,8 @@ enum {
     NOTES_ITEM,
     NEW_ITEM,
     CLOSE_ITEM,
-    BUNDLE_ITEM
+    BUNDLE_ITEM,
+    FIND_ITEM
 } viewbar_flags;
 
 struct stock_maker {
@@ -292,32 +293,9 @@ static void script_index (GtkWidget *w, windata_t *vwin)
     display_files(PS_FILES, NULL);
 }
 
-static void cmd_log_refresh (GtkWidget *w, windata_t *vwin)
-{
-    gchar *newtext;
-    int err = 0;
-
-    newtext = get_logfile_content(&err);
-
-    if (err) {
-	gui_errmsg(err);
-    } else {
-	GtkTextBuffer *buf;
-
-	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text));
-	gtk_text_buffer_set_text(buf, "", -1);
-	if (newtext != NULL) {
-	    textview_set_text(vwin->text, newtext);
-	    g_free(newtext);
-	}
-    }
-}
-
 static void toolbar_refresh (GtkWidget *w, windata_t *vwin)
 {
-    if (vwin->role == VIEW_LOG) {
-	cmd_log_refresh(w, vwin);
-    } else if (vwin->role == VIEW_SERIES) {
+    if (vwin->role == VIEW_SERIES) {
 	series_view_refresh(w, vwin);
     }
 }
@@ -670,7 +648,7 @@ static GretlToolItem viewbar_items[] = {
     { N_("Cut"), GTK_STOCK_CUT, G_CALLBACK(vwin_cut_callback), EDIT_ITEM }, 
     { N_("Copy"), GTK_STOCK_COPY, G_CALLBACK(vwin_copy_callback), COPY_ITEM }, 
     { N_("Paste"), GTK_STOCK_PASTE, G_CALLBACK(text_paste), EDIT_ITEM },
-    { N_("Find..."), GTK_STOCK_FIND, G_CALLBACK(text_find), 0 },
+    { N_("Find..."), GTK_STOCK_FIND, G_CALLBACK(text_find), FIND_ITEM },
     { N_("Replace..."), GTK_STOCK_FIND_AND_REPLACE, G_CALLBACK(text_replace), EDIT_ITEM },
     { N_("Undo"), GTK_STOCK_UNDO, G_CALLBACK(text_undo), EDIT_ITEM },
     { N_("Redo"), GTK_STOCK_REDO, G_CALLBACK(text_redo), EDIT_ITEM },
@@ -802,7 +780,7 @@ static GCallback tool_item_get_callback (GretlToolItem *item, windata_t *vwin,
 	return NULL;
     } else if (r != COEFFINT && f == ALPHA_ITEM) {
 	return NULL;
-    } else if (r != VIEW_LOG && r != VIEW_SERIES && f == REFRESH_ITEM) {
+    } else if (r != VIEW_SERIES && f == REFRESH_ITEM) {
 	return NULL;
     } else if (r != EDIT_GP && f == GP_HELP_ITEM) {
 	return NULL;
@@ -896,6 +874,31 @@ void vwin_toolbar_insert_winlist (windata_t *vwin)
     gtk_toolbar_insert(GTK_TOOLBAR(vwin->mbar), item, -1);
 }
 
+static void gretl_tool_item_set_tip (GtkWidget *item,
+				     GretlToolItem *tool)
+{
+    const char *accel = NULL;
+
+    if (tool->flag == EXEC_ITEM) {
+	accel = "Ctrl-R";
+    } else if (tool->flag == COPY_ITEM) {
+	accel = "Ctrl-C";
+    } else if (tool->flag == SAVE_ITEM) {
+	accel = "Ctrl-S";
+    } else if (tool->flag == FIND_ITEM) {
+	accel = "Ctrl-F";
+    }
+
+    if (accel != NULL) {
+	gchar *s = g_strdup_printf("%s (%s)", _(tool->tip), accel);
+
+	gtk_widget_set_tooltip_text(item, s);
+	g_free(s);
+    } else {
+	gtk_widget_set_tooltip_text(item, _(tool->tip));
+    }    
+}
+
 GtkWidget *gretl_toolbar_insert (GtkWidget *tbar,
 				 GretlToolItem *tool,
 				 GCallback func,
@@ -905,7 +908,7 @@ GtkWidget *gretl_toolbar_insert (GtkWidget *tbar,
     GtkToolItem *item;
 
     item = gtk_tool_button_new_from_stock(tool->icon);
-    gtk_widget_set_tooltip_text(GTK_WIDGET(item), _(tool->tip));
+    gretl_tool_item_set_tip(GTK_WIDGET(item), tool);
     g_signal_connect(G_OBJECT(item), "clicked", func, data);
     gtk_toolbar_insert(GTK_TOOLBAR(tbar), item, pos);
 
@@ -959,7 +962,7 @@ static GtkWidget *vwin_toolbar_insert (GretlToolItem *tool,
 	if (tool->flag == NEW_ITEM && window_is_tab(vwin)) {
 	    gtk_widget_set_tooltip_text(GTK_WIDGET(item), _("New tab"));
 	} else {
-	    gtk_widget_set_tooltip_text(GTK_WIDGET(item), _(tool->tip));
+	    gretl_tool_item_set_tip(GTK_WIDGET(item), tool);
 	}
     }
 
