@@ -25,8 +25,8 @@
 #include "tabwin.h"
 #include "winstack.h"
 
-#if defined(MAC_INTEGRATION) && defined(PKGBUILD)
-# define MAC_HIDE_UNHIDE
+#if GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 20
+# include "spinner.h"
 #endif
 
 #include "uservar.h"
@@ -81,6 +81,17 @@ static GtkWidget *window_from_action (GtkAction *action)
     return w;
 }
 
+static windata_t *vwin_from_action (GtkAction *action)
+{
+    GtkWidget *w = window_from_action(action);
+    
+    if (w != NULL) {
+	return g_object_get_data(G_OBJECT(w), "vwin");
+    }
+
+    return NULL;
+}
+
 /* callback to bring a selected window to the top */
 
 static void gretl_window_raise (GtkAction *action, gpointer data)
@@ -244,7 +255,7 @@ static gint catch_winlist_key (GtkWidget *w, GdkEventKey *event,
 	window_list_popup(w, (GdkEvent *) event, data);
 	return TRUE;
     }
-# ifdef MAC_HIDE_UNHIDE
+# ifdef GRETL_MACINT
     if (cmd_key(event) && mac_hide_unhide(event)) {
 	return TRUE;
     }
@@ -599,13 +610,20 @@ gboolean window_list_exit_check (void)
 			     r == VIEW_LOG ||		\
 			     r == VIEW_SCRIPT ||	\
 	                     r == TEXTBOOK_DATA ||	\
+			     r == PS_FILES ||		\
+			     r == NATIVE_DB ||		\
+			     r == REMOTE_DB ||		\
+			     r == FUNC_FILES ||		\
+			     r == REMOTE_FUNC_FILES ||	\
 			     r == CONSOLE)
 
 
-static int keep_window_open (GtkWidget *w)
+static int keep_window_open (GtkWidget *w, gretlopt opt)
 {
     const gchar *wname = gtk_widget_get_name(w);
 
+    /* FIXME maybe keep plot windows open if opt & OPT_P? */
+
     if (wname != NULL && strcmp(wname, "pkg-editor") == 0) {
 	return 1;
     } else {
@@ -618,7 +636,7 @@ static int keep_window_open (GtkWidget *w)
    close ones that need to stay open!
 */
 
-void close_session_windows (void)
+void close_session_windows (gretlopt opt)
 {
     if (n_listed_windows > 1) {
 	GList *list = gtk_action_group_list_actions(window_group);
@@ -639,7 +657,7 @@ void close_session_windows (void)
 		    /* tabbed script editor stays open, but tabbed model
 		       viewer should be closed */
 		    tabwin_close_models_viewer(w);
-		} else if (w != NULL && !keep_window_open(w)) {
+		} else if (w != NULL && !keep_window_open(w, opt)) {
 		    gtk_widget_destroy(w);
 		}
 	    }
@@ -767,17 +785,13 @@ real_get_browser_for_database (const char *filename, int code)
     if (n_listed_windows > 1) {
 	GList *list = gtk_action_group_list_actions(window_group);
 	windata_t *vwin;
-	GtkWidget *w;
 
 	while (list != NULL && ret == NULL) {
-	    w = window_from_action((GtkAction *) list->data);
-	    if (w != NULL) {
-		vwin = g_object_get_data(G_OBJECT(w), "vwin");
-		if (vwin != NULL && db_role_matches(vwin, code)) {
-		    if (!strncmp(filename, vwin->fname, 
-				 strlen(vwin->fname))) {
-			ret = vwin;
-		    }
+	    vwin = vwin_from_action((GtkAction *) list->data);
+	    if (vwin != NULL && db_role_matches(vwin, code)) {
+		if (!strncmp(filename, vwin->fname, 
+			     strlen(vwin->fname))) {
+		    ret = vwin;
 		}
 	    }
 	    list = list->next;
@@ -805,15 +819,11 @@ windata_t *get_viewer_for_data (const gpointer data)
     if (n_listed_windows > 1) {
 	GList *list = gtk_action_group_list_actions(window_group);
 	windata_t *vwin;
-	GtkWidget *w;
 
 	while (list != NULL && ret == NULL) {
-	    w = window_from_action((GtkAction *) list->data);
-	    if (w != NULL) {
-		vwin = g_object_get_data(G_OBJECT(w), "vwin");
-		if (vwin != NULL && vwin->data == data) {
-		    ret = vwin;
-		}
+	    vwin = vwin_from_action((GtkAction *) list->data);
+	    if (vwin != NULL && vwin->data == data) {
+		ret = vwin;
 	    }
 	    list = list->next;
 	}
@@ -830,15 +840,32 @@ windata_t *get_browser_for_role (int role)
     if (n_listed_windows > 1) {
 	GList *list = gtk_action_group_list_actions(window_group);
 	windata_t *vwin;
-	GtkWidget *w;
 
 	while (list != NULL && ret == NULL) {
-	    w = window_from_action((GtkAction *) list->data);
-	    if (w != NULL) {
-		vwin = g_object_get_data(G_OBJECT(w), "vwin");
-		if (vwin != NULL && vwin->role == role) {
-		    ret = vwin;
-		}
+	    vwin = vwin_from_action((GtkAction *) list->data);
+	    if (vwin != NULL && vwin->role == role) {
+		ret = vwin;
+	    }
+	    list = list->next;
+	}
+	g_list_free(list);
+    }
+
+    return ret;
+}
+
+int get_script_output_number (void)
+{
+    int ret = 0;
+
+    if (n_listed_windows > 1) {
+	GList *list = gtk_action_group_list_actions(window_group);
+	windata_t *vwin;
+
+	while (list != NULL) {
+	    vwin = vwin_from_action((GtkAction *) list->data);
+	    if (vwin != NULL && vwin->role == SCRIPT_OUT) {
+		ret++;
 	    }
 	    list = list->next;
 	}
@@ -1076,6 +1103,14 @@ static void menu_bar_add_winlist (windata_t *vwin)
     gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
 }
 
+static void destroy_hbox_child (GtkWidget *w, gpointer p)
+{
+    if (GTK_IS_SPINNER(w)) {
+	gtk_spinner_stop(GTK_SPINNER(w));
+    }
+    gtk_widget_destroy(w);
+}
+
 void vwin_pack_toolbar (windata_t *vwin)
 {
     if (vwin->topmain != NULL) {
@@ -1085,22 +1120,32 @@ void vwin_pack_toolbar (windata_t *vwin)
 	    menu_bar_add_winlist(vwin);
 	}
     } else {
-	GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
+	GtkWidget *hbox;
 
-	gtk_box_pack_start(GTK_BOX(vwin->vbox), hbox, FALSE, FALSE, 0);
-	if (vwin->role == VIEW_MODEL || vwin->role == VAR ||
-	    vwin->role == VECM) {
-	    /* model viewer: the menubar extends full-length */
-	    gtk_box_pack_start(GTK_BOX(hbox), vwin->mbar, TRUE, TRUE, 0);
-	    menu_bar_add_winlist(vwin);
-	} else {
+	hbox = g_object_get_data(G_OBJECT(vwin->main), "top-hbox");
+
+	if (hbox != NULL) {
+	    gtk_container_foreach(GTK_CONTAINER(hbox), destroy_hbox_child, NULL);
 	    gtk_box_pack_start(GTK_BOX(hbox), vwin->mbar, FALSE, FALSE, 0);
+	    gtk_widget_show_all(hbox);
+	} else {
+	    hbox = gtk_hbox_new(FALSE, 0);
+	    gtk_box_pack_start(GTK_BOX(vwin->vbox), hbox, FALSE, FALSE, 0);
+
+	    if (vwin->role == VIEW_MODEL || vwin->role == VAR ||
+		vwin->role == VECM) {
+		/* model viewer: the menubar extends full-length */
+		gtk_box_pack_start(GTK_BOX(hbox), vwin->mbar, TRUE, TRUE, 0);
+		menu_bar_add_winlist(vwin);
+	    } else {
+		gtk_box_pack_start(GTK_BOX(hbox), vwin->mbar, FALSE, FALSE, 0);
+	    }
+	    if (window_is_tab(vwin)) {
+		/* here we're re-packing vwin->mbar: move it up top */
+		gtk_box_reorder_child(GTK_BOX(vwin->vbox), hbox, 0);
+	    }
+	    gtk_widget_show_all(hbox);
 	}
-	if (window_is_tab(vwin)) {
-	    /* here we're re-packing vwin->mbar: move it up top */
-	    gtk_box_reorder_child(GTK_BOX(vwin->vbox), hbox, 0);
-	}
-	gtk_widget_show_all(hbox);
     }
 }
 
@@ -1156,3 +1201,36 @@ void gretl_viewer_set_title (windata_t *vwin, const char *title)
 	gtk_window_set_title(GTK_WINDOW(vwin->main), title);
     }
 }
+
+/* When we add popup menus as callbacks for buttons on @vwin's
+   toolbar, we want to record pointers to them so we're
+   able to destroy them when @vwin is closed, otherwise
+   we'd be leaking memory.
+*/
+
+void vwin_record_toolbar_popup (windata_t *vwin, GtkWidget *menu)
+{
+    GList *plist;
+
+    plist = g_object_get_data(G_OBJECT(vwin->mbar), "toolbar-popups");
+    plist = g_list_append(plist, menu);
+    g_object_set_data(G_OBJECT(vwin->mbar), "toolbar-popups", plist);
+}
+
+static void trash_toolbar_popup (gpointer data, gpointer p)
+{
+    gtk_widget_destroy(GTK_WIDGET(data));
+}
+
+void vwin_free_toolbar_popups (windata_t *vwin)
+{
+    if (vwin->mbar != NULL) {
+	GList *plist;
+
+	plist = g_object_get_data(G_OBJECT(vwin->mbar), "toolbar-popups");
+	if (plist != NULL) {
+	    g_list_foreach(plist, trash_toolbar_popup, NULL);
+	}
+	g_list_free(plist);
+    }
+}
