@@ -57,7 +57,7 @@ struct kpss_info_ {
     double pval;
 };
 
-enum {
+typedef enum {
     UR_NO_CONST = 1,
     UR_CONST,
     UR_TREND,
@@ -65,11 +65,11 @@ enum {
     UR_MAX
 } AdfCode;
 
-enum {
+typedef enum {
     ADF_EG_TEST   = 1 << 0,
     ADF_EG_RESIDS = 1 << 1,
     ADF_PANEL     = 1 << 2
-} adf_flags;
+} AdfFlags;
 
 enum {
     AUTO_MAIC = 1,
@@ -79,7 +79,7 @@ enum {
 
 /* replace y with demeaned or detrended y */
 
-static int GLS_demean_detrend (double *y, int T, int test)
+static int GLS_demean_detrend (double *y, int T, AdfCode test)
 {
     gretl_matrix *ya = NULL;
     gretl_matrix *Za = NULL;
@@ -226,7 +226,7 @@ adf_prepare_vars (int order, int varno, int nseas, int *d0,
 
     if (opt & OPT_G) {
 	/* GLS adjustment is wanted */
-	int test = (opt & OPT_T)? UR_TREND : UR_CONST;
+	AdfCode test = (opt & OPT_T)? UR_TREND : UR_CONST;
 	int t, v = dset->v;
 
 	*err = dataset_add_series(dset, 1);
@@ -309,7 +309,7 @@ static void show_lags_test (MODEL *pmod, int order, PRN *prn)
 	F = wald_omit_F(llist, pmod);
 
 	if (!na(F)) {
-	    pprintf(prn, "   %s: F(%d, %d) = %.3f [%.4f]\n",
+	    pprintf(prn, "  %s: F(%d, %d) = %.3f [%.4f]\n",
 		    _("lagged differences"), order, pmod->dfd, F, 
 		    snedecor_cdf_comp(order, pmod->dfd, F));
 	}
@@ -415,7 +415,7 @@ static const char *DF_test_string (int i)
 static void 
 print_adf_results (int order, int pmax, double DFt, double pv, 
 		   MODEL *dfmod, int dfnum, const char *vname, 
-		   int *blurb_done, unsigned char flags, int i, 
+		   int *blurb_done, AdfFlags flags, int i, 
 		   int niv, int nseas, gretlopt opt, 
 		   int auto_order, PRN *prn)
 {
@@ -454,20 +454,20 @@ print_adf_results (int order, int pmax, double DFt, double pv,
 
     if (flags & ADF_EG_RESIDS) {
 	/* last step of Engle-Granger test */
-	pprintf(prn, "   %s: %s\n", _("model"), 
+	pprintf(prn, "  %s: %s\n", _("model"), 
 		(order > 0)? ADF_model_string(0) : DF_model_string(0));
     } else {
-	pprintf(prn, "   %s ", _(DF_test_string(i)));
+	pprintf(prn, "  %s ", _(DF_test_string(i)));
 	if (nseas > 0 && i > 0) {
 	    pputs(prn, _("plus seasonal dummies"));
 	}
 	pputc(prn, '\n');
-	pprintf(prn, "   %s: %s\n", _("model"), 
+	pprintf(prn, "  %s: %s\n", _("model"), 
 		(order > 0)? ADF_model_string(i) : DF_model_string(i));
     }
 
     if (!na(dfmod->rho)) {
-	pprintf(prn, "   %s: %.3f\n", _("1st-order autocorrelation coeff. for e"), 
+	pprintf(prn, "  %s: %.3f\n", _("1st-order autocorrelation coeff. for e"), 
 		dfmod->rho);
     }
 
@@ -481,20 +481,20 @@ print_adf_results (int order, int pmax, double DFt, double pv,
 	sprintf(taustr, "tau_%s(%d)", urcstrs[i], niv);
     }
 
-    pprintf(prn, "   %s: %g\n"
-	    "   %s: %s = %g\n",
+    pprintf(prn, "  %s: %g\n"
+	    "  %s: %s = %g\n",
 	    _("estimated value of (a - 1)"), dfmod->coeff[dfnum],
 	    _("test statistic"), taustr, DFt);
 
     if ((opt & OPT_G) && i+1 == UR_TREND) {
 	const double *c = df_gls_ct_cval(dfmod->nobs);
 
-	pprintf(prn, "\n   %*s    ", TRANSLATED_WIDTH(_("Critical values")), " ");
+	pprintf(prn, "\n  %*s    ", TRANSLATED_WIDTH(_("Critical values")), " ");
 	pprintf(prn, "%g%%     %g%%     %g%%     %g%%\n", 10.0, 5.0, 2.5, 1.0);
-	pprintf(prn, "   %s: %.2f   %.2f   %.2f   %.2f\n", 
+	pprintf(prn, "  %s: %.2f   %.2f   %.2f   %.2f\n", 
 		_("Critical values"), c[0], c[1], c[2], c[3]);
     } else {
-	pprintf(prn, "   %s\n", pvstr);
+	pprintf(prn, "  %s\n", pvstr);
     } 
 }
 
@@ -595,7 +595,7 @@ static double get_sum_y2 (MODEL *pmod, int ylagno, const DATASET *dset)
 
 static int ic_adjust_order (int *list, int kmax, int which,
 			    DATASET *dset, gretlopt opt,
-			    int *err, PRN *prn)
+			    int test_num, int *err, PRN *prn)
 {
     MODEL kmod;
     gretlopt kmod_opt = (OPT_A | OPT_Z);
@@ -644,15 +644,21 @@ static int ic_adjust_order (int *list, int kmax, int which,
 	printmodel(&kmod, dset, OPT_NONE, prn);
 #endif
 	if (opt & OPT_V) {
-	    if (k == kmax) {
+	    const char *tag = (which == AUTO_MBIC) ? "MBIC" : "MAIC";
+
+	    if (k == kmax && test_num == 1) {
 		pputc(prn, '\n');
 	    }
-	    pprintf(prn, "MIC = %#g for k = %02d\n", MIC, k);
+	    pprintf(prn, "  k = %02d: %s = %#g\n", k, tag, MIC);
 	}	    
 	clear_model(&kmod);
 	gretl_list_delete_at_pos(tmplist, k + 2);
     }
 
+    if ((opt & OPT_V) && test_num > 1) {
+	pputc(prn, '\n');
+    }
+
     free(tmplist);
 
     if (kstar >= 0) {
@@ -701,7 +707,6 @@ double get_urc_pvalue (double tau, int n, int niv, int itv,
 		       gretlopt opt)
 {
     char datapath[FILENAME_MAX];
-    void *handle;
     double (*mackinnon_pvalue)(double, int, int, int, char *);
     double pval = NADBL;
     static int nodata;
@@ -710,7 +715,7 @@ double get_urc_pvalue (double tau, int n, int niv, int itv,
 	return pval;
     }
 
-    mackinnon_pvalue = get_plugin_function("mackinnon_pvalue", &handle);
+    mackinnon_pvalue = get_plugin_function("mackinnon_pvalue");
     if (mackinnon_pvalue == NULL) {
 	nodata = 1;
         return pval;
@@ -732,8 +737,6 @@ double get_urc_pvalue (double tau, int n, int niv, int itv,
 	    tau, n, niv, itv, pval);
 #endif
 
-    close_plugin(handle);
-
     if (*datapath == '\0') {
 	nodata = 1;
     } 
@@ -744,7 +747,7 @@ double get_urc_pvalue (double tau, int n, int niv, int itv,
 #define test_opt_not_set(o) (!(o & OPT_N) && !(o & OPT_C) && \
                              !(o & OPT_T) && !(o & OPT_R))
 
-static int test_wanted (int test, gretlopt opt)
+static int test_wanted (AdfCode test, gretlopt opt)
 {
     int ret = 0;
 
@@ -768,9 +771,9 @@ static int test_wanted (int test, gretlopt opt)
     return ret;
 }
 
-static int engle_granger_itv (gretlopt opt)
+static AdfCode engle_granger_itv (gretlopt opt)
 {
-    int itv = UR_CONST;
+    AdfCode itv = UR_CONST;
 
     if (opt & OPT_N) {
 	itv = UR_NO_CONST;
@@ -814,9 +817,15 @@ static int gettrend (DATASET *dset, int square)
     return idx;
 }
 
-static int get_auto_order_method (int *err)
+static int get_auto_order_method (AdfFlags flags, int *err)
 {
-    const char *s = get_optval_string(ADF, OPT_E);
+    const char *s;
+
+    if (flags & (ADF_EG_TEST | ADF_EG_RESIDS)) {
+	s = get_optval_string(COINT, OPT_E);
+    } else {
+	s = get_optval_string(ADF, OPT_E);
+    }
 
     if (s == NULL || *s == '\0') {
 	/* the default */
@@ -836,8 +845,8 @@ static int get_auto_order_method (int *err)
 
 static int real_adf_test (int varno, int order, int niv,
 			  DATASET *dset, gretlopt opt, 
-			  unsigned char flags,
-			  adf_info *ainfo, PRN *prn)
+			  AdfFlags flags, adf_info *ainfo, 
+			  PRN *prn)
 {
     MODEL dfmod;
     gretlopt eg_opt = OPT_NONE;
@@ -846,6 +855,7 @@ static int real_adf_test (int varno, int order, int niv,
     int blurb_done = 0;
     int auto_order = 0;
     int order_max = 0;
+    int test_num = 0;
     int *list = NULL;
     int *biglist = NULL;
     double DFt = NADBL;
@@ -865,7 +875,7 @@ static int real_adf_test (int varno, int order, int niv,
 
     if (opt & OPT_E) {
 	/* testing down */
-	auto_order = get_auto_order_method(&err);
+	auto_order = get_auto_order_method(flags, &err);
 	if (err) {
 	    return err;
 	}
@@ -889,10 +899,13 @@ static int real_adf_test (int varno, int order, int niv,
 	   regression, represented by "eg_opt".
 	*/
 	int verbose = (opt & OPT_V);
+	int silent = (opt & OPT_I);
 
 	eg_opt = opt;
 	opt = OPT_N;
-	if (verbose) {
+	if (silent) {
+	    opt |= OPT_I;
+	} else if (verbose) {
 	    opt |= OPT_V;
 	}
     }
@@ -996,12 +1009,14 @@ static int real_adf_test (int varno, int order, int niv,
 
     skipdet:
 
+	test_num++;
+
 	if (auto_order) {
 	    if (auto_order == AUTO_TSTAT) {
 		order = t_adjust_order(list, order_max, dset, &err, prn);
 	    } else {
 		order = ic_adjust_order(list, order_max, auto_order,
-					dset, opt, &err, prn);
+					dset, opt, test_num, &err, prn);
 	    }	    
 	    if (err) {
 		clear_model(&dfmod);
@@ -1058,7 +1073,7 @@ static int real_adf_test (int varno, int order, int niv,
 	    ainfo->pval = pv;
 	}
 
-	if (!(opt & OPT_Q) && !(flags & ADF_PANEL)) {
+	if (!(opt & (OPT_Q | OPT_I)) && !(flags & ADF_PANEL)) {
 	    print_adf_results(order, order_max, DFt, pv, &dfmod, dfnum, 
 			      dset->varname[varno], &blurb_done, flags,
 			      itv, niv, nseas, opt, auto_order, prn);
@@ -1071,6 +1086,9 @@ static int real_adf_test (int varno, int order, int niv,
 		gretl_model_set_int(&dfmod, "dfnum", dfnum);
 		gretl_model_set_double(&dfmod, "dfpval", pv);
 	    }
+	    if (flags & ADF_EG_RESIDS) {
+		gretl_model_set_int(&dfmod, "eg-resids", 1);
+	    }
 	    printmodel(&dfmod, dset, OPT_NONE, prn);
 	} else if (!(opt & OPT_Q) && !(flags & (ADF_EG_RESIDS | ADF_PANEL))) {
 	    pputc(prn, '\n');
@@ -1156,7 +1174,6 @@ static int do_IPS_test (double tbar, int n, const int *Ti,
     int (*get_IPS_critvals) (int, int, int, double *);
     int (*IPS_tbar_moments) (int, double *, double *);
     int (*IPS_tbar_rho_moments) (int, int, int, double *, double *);
-    void *handle = NULL;
     int Tmin = Ti[1], Tmax = Ti[1];
     int i, T, err = 0;
 
@@ -1174,7 +1191,7 @@ static int do_IPS_test (double tbar, int n, const int *Ti,
 	double E, V, Wtbar, Etbar = 0, Vtbar = 0;
 	int order_i;
 
-	IPS_tbar_rho_moments = get_plugin_function("IPS_tbar_rho_moments", &handle);
+	IPS_tbar_rho_moments = get_plugin_function("IPS_tbar_rho_moments");
 
 	if (IPS_tbar_rho_moments != NULL) {
 	    for (i=0; i<n && !err; i++) {
@@ -1198,7 +1215,7 @@ static int do_IPS_test (double tbar, int n, const int *Ti,
 	/* sample sizes differ: use IPS's Z_{tbar} */
 	double E, V, Ztbar, Etbar = 0, Vtbar = 0;
 
-	IPS_tbar_moments = get_plugin_function("IPS_tbar_moments", &handle);
+	IPS_tbar_moments = get_plugin_function("IPS_tbar_moments");
 
 	if (IPS_tbar_moments != NULL) {
 	    for (i=0; i<n && !err; i++) {
@@ -1222,7 +1239,7 @@ static int do_IPS_test (double tbar, int n, const int *Ti,
 	pprintf(prn, "N,T = (%d,%d)\n", n, Tmax);
 	pprintf(prn, "Im-Pesaran-Shin t-bar = %g\n", tbar);
 
-	get_IPS_critvals = get_plugin_function("get_IPS_critvals", &handle);
+	get_IPS_critvals = get_plugin_function("get_IPS_critvals");
 
 	if (get_IPS_critvals != NULL) {
 	    double a[] = { 0.1, 0.05, 0.01 };
@@ -1235,10 +1252,6 @@ static int do_IPS_test (double tbar, int n, const int *Ti,
 	}
     }
 
-    if (handle != NULL) {
-	close_plugin(handle);
-    }
-
     return err;
 }
 
@@ -1415,7 +1428,6 @@ int levin_lin_test (int vnum, const int *plist,
 {
     int (*real_levin_lin) (int, const int *, DATASET *, 
 			   gretlopt, PRN *);
-    void *handle;
     int panelmode;
     int err = 0;
 
@@ -1425,7 +1437,7 @@ int levin_lin_test (int vnum, const int *plist,
 	return E_BADOPT;
     }
 
-    real_levin_lin = get_plugin_function("real_levin_lin", &handle);
+    real_levin_lin = get_plugin_function("real_levin_lin");
 
     if (real_levin_lin == NULL) {
 	fputs(I_("Couldn't load plugin function\n"), stderr);
@@ -1435,7 +1447,6 @@ int levin_lin_test (int vnum, const int *plist,
 	int save_t2 = dset->t2;
  
 	err = (*real_levin_lin) (vnum, plist, dset, opt, prn);
-	close_plugin(handle);
 
 	dset->t1 = save_t1;
 	dset->t2 = save_t2;
@@ -1559,9 +1570,30 @@ static double kpss_critval (double alpha, int T, int trend)
     double b[2];
 
     kpss_parms(alpha, trend, b);
+
     return b[0] + b[1]/T;
 }
 
+gretl_matrix *kpss_critvals (int T, int trend, int *err)
+{
+    gretl_matrix *m = NULL;
+
+    if (T < 5) {
+	*err = E_TOOFEW;
+    } else {
+	m = gretl_matrix_alloc(1, 3);
+	if (m == NULL) {
+	    *err = E_ALLOC;
+	} else {
+	    m->val[0] = kpss_critval(0.10, T, trend);
+	    m->val[1] = kpss_critval(0.05, T, trend);
+	    m->val[2] = kpss_critval(0.01, T, trend);
+	}
+    }
+
+    return m;
+}
+
 #define PV_GT10 1.1
 #define PV_LT01 -1.0
 
@@ -1735,9 +1767,6 @@ real_kpss_test (int order, int varno, DATASET *dset,
 	kinfo->pval = pval;
     } else {
 	/* testing individual time series */
-	if (pval == PV_GT10 || pval == PV_LT01) {
-	    pval = NADBL;
-	}
 	if (opt & OPT_V) {
 	    pprintf(prn, "  %s: %g\n", _("Robust estimate of variance"), s2);
 	    pprintf(prn, "  %s: %g\n", _("Sum of squares of cumulated residuals"), 
@@ -1774,10 +1803,8 @@ real_kpss_test (int order, int varno, DATASET *dset,
 	print_critical_values(a, cv, KPSS, prn);
 	if (pval == PV_GT10) {
 	    pprintf(prn, "%s > .10\n", _("P-value"));
-	    pval = NADBL; /* invalidate for record_test_result */
 	} else if (pval == PV_LT01) {
 	    pprintf(prn, "%s < .01\n", _("P-value"));
-	    pval = NADBL;
 	} else if (!xna(pval)) {
 	    pprintf(prn, "%s %.3f\n", _("Interpolated p-value"), pval);
 	}
@@ -1785,6 +1812,10 @@ real_kpss_test (int order, int varno, DATASET *dset,
     }
 
     if (kinfo == NULL) {
+	if (pval == PV_GT10 || pval == PV_LT01) {
+	    /* invalidate for record_test_result */
+	    pval = NADBL;
+	}
 	record_test_result(teststat, pval, "KPSS");
     }
 
@@ -2082,6 +2113,8 @@ static int coint_set_sample (const int *list, int nv, int order,
     return 0;
 }
 
+#define EG_MIN_SAMPLE 0
+
 /**
  * engle_granger_test:
  * @order: lag order for the test.
@@ -2106,14 +2139,18 @@ static int coint_set_sample (const int *list, int nv, int order,
 int engle_granger_test (int order, const int *list, DATASET *dset, 
 			gretlopt opt, PRN *prn)
 {
+#if EG_MIN_SAMPLE
+    int test_t1, test_t2;
+#endif
     int orig_t1 = dset->t1;
     int orig_t2 = dset->t2;
-    int test_t1, test_t2;
     gretlopt adf_opt = OPT_C;
     MODEL cmod;
     int detcode = UR_CONST;
     int i, nv, k = 0;
     int step = 1;
+    int skip = 0;
+    int silent = 0;
     int *clist = NULL;
     int err = 0;
 
@@ -2145,10 +2182,18 @@ int engle_granger_test (int order, const int *list, DATASET *dset,
 	adf_opt |= OPT_V;
     }
 
+    /* or silence? */
+    if (opt & OPT_I) {
+	adf_opt |= OPT_I;
+	silent = skip = 1;
+    } else if (opt & OPT_S) {
+	skip = 1;
+    }
+
     gretl_model_init(&cmod, dset);
 
-    if (!(opt & OPT_S)) {
-	/* test all candidate vars for unit root */
+    if (!skip) {
+	/* start by testing all candidate vars for unit root */
 	coint_set_sample(clist, nv, order, dset);
 	for (i=1; i<=nv; i++) {
 	    if (step == 1) {
@@ -2161,13 +2206,17 @@ int engle_granger_test (int order, const int *list, DATASET *dset,
 	}
     }
 
-    if (step == 1) {
-	pputc(prn, '\n');
+    if (!silent) {
+	if (step == 1) {
+	    pputc(prn, '\n');
+	}
+	pprintf(prn, _("Step %d: cointegrating regression\n"), step++);
     }
-    pprintf(prn, _("Step %d: cointegrating regression\n"), step++);
 
+#if EG_MIN_SAMPLE
     test_t1 = dset->t1;
     test_t2 = dset->t2;
+#endif
 
     dset->t1 = orig_t1;
     dset->t2 = orig_t2;
@@ -2178,8 +2227,10 @@ int engle_granger_test (int order, const int *list, DATASET *dset,
 	goto bailout;
     }
 
-    cmod.aux = AUX_COINT;
-    printmodel(&cmod, dset, OPT_NONE, prn);
+    if (!silent) {
+	cmod.aux = AUX_COINT;
+	printmodel(&cmod, dset, OPT_NONE, prn);
+    }
 
     /* add residuals from cointegrating regression to data set */
     err = dataset_add_allocated_series(dset, cmod.uhat);
@@ -2191,21 +2242,27 @@ int engle_granger_test (int order, const int *list, DATASET *dset,
     strcpy(dset->varname[k], "uhat");
     cmod.uhat = NULL;
 
-    pprintf(prn, _("Step %d: testing for a unit root in %s\n"),
-	    step, dset->varname[k]);
+    if (!silent) {
+	pprintf(prn, _("Step %d: testing for a unit root in %s\n"),
+		step, dset->varname[k]);
+    }
 
+#if EG_MIN_SAMPLE
     dset->t1 = test_t1;
     dset->t2 = test_t2;
+#endif
 
     /* Run (A)DF test on the residuals */
     real_adf_test(k, order, nv, dset, adf_opt, 
 		  ADF_EG_TEST | ADF_EG_RESIDS, NULL, prn); 
 
-    pputs(prn, _("\nThere is evidence for a cointegrating relationship if:\n"
-		 "(a) The unit-root hypothesis is not rejected for the individual"
-		 " variables.\n(b) The unit-root hypothesis is rejected for the "
-		 "residuals (uhat) from the \n    cointegrating regression.\n"));
-    pputc(prn, '\n');
+    if (!silent) {
+	pputs(prn, _("\nThere is evidence for a cointegrating relationship if:\n"
+		     "(a) The unit-root hypothesis is not rejected for the individual"
+		     " variables, and\n(b) the unit-root hypothesis is rejected for the "
+		     "residuals (uhat) from the \n    cointegrating regression.\n"));
+	pputc(prn, '\n');
+    }
 
  bailout:
     
