@@ -568,10 +568,9 @@ static void bs_print_result (boot *bs, double *xi, int tail, PRN *prn)
 
     if (bs->flags & BOOT_GRAPH) {
 	int (*kdfunc) (const double *, int, const char *);
-	void *handle;
 	char label[48];
 
-	kdfunc = get_plugin_function("array_kernel_density", &handle);
+	kdfunc = get_plugin_function("array_kernel_density");
 	if (kdfunc == NULL) {
 	    return;
 	}
@@ -585,7 +584,6 @@ static void bs_print_result (boot *bs, double *xi, int tail, PRN *prn)
 	} 
 
 	(*kdfunc)(xi, bs->B, label);
-	close_plugin(handle);
     }
 }
 
@@ -662,7 +660,7 @@ static double bs_F_test (const gretl_matrix *b,
     return test;
 }
 
-#if 0
+#if 0 /* mot used */
 
 static char *squarable_cols_mask (const gretl_matrix *X, int *n)
 {
@@ -791,7 +789,7 @@ static int hsk_transform_data (boot *bs, gretl_matrix *b,
     return err;
 }
 
-#endif
+#endif /* not used */
 
 static void print_test_round (boot *bs, int i, double test, PRN *prn)
 {
@@ -1194,7 +1192,7 @@ int bootstrap_save_data (const char *fname)
 	return err;
     }
 
-    err = gretl_write_matrix_as_gdt(fname, bs_data, (const char **) S, 
+    err = gretl_matrix_write_as_gdt(fname, bs_data, (const char **) S, 
 				    NULL);
 
     gretl_matrix_free(bs_data);
