@@ -99,15 +99,20 @@ double get_date_x (int pd, const char *obs)
 int check_varname (const char *varname)
 {
     int testchar = 'a';
-    int ret = 0;
+    int firstbad = 0;
+    int err = 0;
 
     gretl_error_clear();
 
-    if (gretl_reserved_word(varname)) {
-	ret = VARNAME_RESERVED;
+    if (strlen(varname) >= VNAMELEN) {
+	gretl_errmsg_set(_("Varname exceeds the maximum of 31 characters"));
+	err = E_DATA;
+    } else if (gretl_reserved_word(varname)) {
+	err = E_DATA;
     } else if (!(isalpha((unsigned char) *varname))) {
+	firstbad = 1;
 	testchar = *varname;
-        ret = VARNAME_FIRSTCHAR;
+        err = E_DATA;
     } else {
 	const char *p = varname;
 
@@ -116,7 +121,7 @@ int check_varname (const char *varname)
 		&& !(isdigit((unsigned char) *p))
 		&& *p != '_') {
 		testchar = *p;
-		ret = VARNAME_BADCHAR;
+		err = E_DATA;
 	    }
 	    p++;
 	}
@@ -124,7 +129,7 @@ int check_varname (const char *varname)
 
     if (testchar != 'a') {
 	if (isprint((unsigned char) testchar)) {
-	    if (ret == VARNAME_FIRSTCHAR) {
+	    if (firstbad) {
 		gretl_errmsg_sprintf(_("First char of varname '%s' is bad\n"
 				       "(first must be alphabetical)"), 
 				     varname);
@@ -134,7 +139,7 @@ int check_varname (const char *varname)
 				     varname, (unsigned char) testchar);
 	    }
 	} else {
-	    if (ret == VARNAME_FIRSTCHAR) {
+	    if (firstbad) {
 		gretl_errmsg_sprintf(_("First char of varname (0x%x) is bad\n"
 				       "(first must be alphabetical)"), 
 				     (unsigned) testchar);
@@ -146,7 +151,7 @@ int check_varname (const char *varname)
 	}
     }
 
-    return ret;
+    return err;
 }   
 
 static int bad_date_string (const char *s)
@@ -1672,7 +1677,8 @@ static int merge_lengthen_series (DATASET *dset,
  * @dset: dataset struct.
  * @addset: dataset to be merged in.
  * @opt: may include OPT_T to force a time-series interpretation
- * when appending to a panel dataset.
+ * when appending to a panel dataset; may include OPT_U to update
+ * values of overlapping observations.
  * @prn: print struct to accept messages.
  * 
  * Attempt to merge the content of a newly opened data file into
@@ -1684,6 +1690,8 @@ static int merge_lengthen_series (DATASET *dset,
 static int merge_data (DATASET *dset, DATASET *addset,
 		       gretlopt opt, PRN *prn)
 {
+    int update_overlap = (opt & OPT_U);
+    int orig_n = dset->n;
     int dayspecial = 0;
     int yrspecial = 0;
     int addsimple = 0;
@@ -1765,7 +1773,7 @@ static int merge_data (DATASET *dset, DATASET *addset,
     if (!err && addpanel) {
 	err = panel_append_special(addvars, dset, addset, 
 				   opt, prn);
-    } else if (!err) { 
+    } else if (!err) {
 	int k = dset->v;
 	int i, t;
 
@@ -1776,12 +1784,16 @@ static int merge_data (DATASET *dset, DATASET *addset,
 
 	for (i=1; i<addset->v && !err; i++) {
 	    int v = series_index(dset, addset->varname[i]);
-	    int newvar = 0;
+	    int tmin, newvar = v >= k;
 
-	    if (v >= k) {
-		/* a new variable */
+	    if (!newvar && !update_overlap) {
+		tmin = orig_n;
+	    } else {
+		tmin = 0;
+	    }
+
+	    if (newvar) {
 		v = k++;
-		newvar = 1;
 		strcpy(dset->varname[v], addset->varname[i]);
 		copy_varinfo(dset->varinfo[v], addset->varinfo[i]);
 		if (is_string_valued(addset, i) &&
@@ -1793,13 +1805,13 @@ static int merge_data (DATASET *dset, DATASET *addset,
 		    series_attach_string_table(dset, v, st);
 		    series_attach_string_table(addset, i, NULL);
 		}
-	    } 
+	    }
 
 	    if (dayspecial) {
 		char obs[OBSLEN];
 		int s;
 
-		for (t=0; t<dset->n; t++) {
+		for (t=tmin; t<dset->n; t++) {
 		    ntodate(obs, t, dset);
 		    s = dateton(obs, addset);
 		    if (s >= 0 && s < addset->n) {
@@ -1818,12 +1830,12 @@ static int merge_data (DATASET *dset, DATASET *addset,
 		}
 		for (s=0; s<addset->n; s++) {
 		    t = dateton(addset->S[s], dset);
-		    if (t >= 0 && t < dset->n) {
+		    if (t >= tmin && t < dset->n) {
 			dset->Z[v][t] = addset->Z[i][s];
 		    }
 		}		
 	    } else {
-		for (t=0; t<dset->n; t++) {
+		for (t=tmin; t<dset->n; t++) {
 		    if (t >= offset && t - offset < addset->n) {
 			dset->Z[v][t] = addset->Z[i][t - offset];
 		    } else if (newvar) {
@@ -1865,8 +1877,7 @@ static void maybe_fix_calendar_dates (DATASET *dset)
  * merge_or_replace_data:
  * @dset0: original dataset struct.
  * @pdset1: new dataset struct.
- * @opt: may include OPT_T when appending to a panel dataset,
- * to force a time-series interpretation of the added data.
+ * @opt: zero or more option flags.
  * @prn: print struct to accept messages.
  *
  * Given a newly-created dataset, pointed to by @pdset1, either 
@@ -1888,7 +1899,15 @@ int merge_or_replace_data (DATASET *dset0, DATASET **pdset1,
     if (dset0->Z != NULL) {
 	/* we have an existing dataset into which the new data
 	   should be merged */
-	err = merge_data(dset0, *pdset1, opt, prn);
+	gretlopt merge_opt = OPT_NONE;
+
+	if (opt & OPT_T) {
+	    merge_opt |= OPT_T;
+	}
+	if (opt & OPT_U) {
+	    merge_opt |= OPT_U;
+	}
+	err = merge_data(dset0, *pdset1, merge_opt, prn);
 	destroy_dataset(*pdset1);
     } else {
 	/* starting from scratch */
