@@ -1489,7 +1489,7 @@ int dataset_rename_series (DATASET *dset, int v, const char *name)
 {
     int err = 0;
 
-    if (v < 0 || v >= dset->v || name == NULL) {
+    if (v <= 0 || v >= dset->v || name == NULL) {
 	err = E_DATA;
     } else if (object_is_const(dset->varname[v]) ||
 	series_is_parent(dset, v)) {
@@ -2503,7 +2503,7 @@ int dataset_stack_variables (const char *vname, const char *line,
     double *bigx = NULL;
     int i, v1 = 0, v2 = 0, nv = 0;
     int done = 0;
-    int maxok, offset;
+    int maxok, offset = 0;
     int oldn, bign, genv;
     int err = 0;
 
@@ -2918,6 +2918,10 @@ int dataset_op_from_string (const char *s)
 {
     int op = DS_NONE;
 
+    if (s == NULL || *s == '\0') {
+	return DS_NONE;
+    }
+
     if (!strcmp(s, "addobs")) {
 	op = DS_ADDOBS;
     } else if (!strcmp(s, "compact")) {
@@ -3018,18 +3022,19 @@ static int compact_data_set_wrapper (const char *s, DATASET *dset,
 {
     CompactMethod method = COMPACT_AVG;
 
-    s += strspn(s, " ");
-
-    if (!strcmp(s, "sum")) {
-	method = COMPACT_SUM;
-    } else if (!strcmp(s, "first") || !strcmp(s, "sop")) {
-	method = COMPACT_SOP;
-    } else if (!strcmp(s, "last") || !strcmp(s, "eop")) {
-	method = COMPACT_EOP;
-    } else if (!strcmp(s, "avg") || !strcmp(s, "average")) {
-	method = COMPACT_AVG;
-    } else if (*s != '\0') {
-	return E_PARSE;
+    if (s != NULL) {
+	s += strspn(s, " ");
+	if (!strcmp(s, "sum")) {
+	    method = COMPACT_SUM;
+	} else if (!strcmp(s, "first") || !strcmp(s, "sop")) {
+	    method = COMPACT_SOP;
+	} else if (!strcmp(s, "last") || !strcmp(s, "eop")) {
+	    method = COMPACT_EOP;
+	} else if (!strcmp(s, "avg") || !strcmp(s, "average")) {
+	    method = COMPACT_AVG;
+	} else if (*s != '\0') {
+	    return E_PARSE;
+	}
     }
 
     return compact_data_set(dset, k, method, 0, 0);
@@ -3196,7 +3201,7 @@ int renumber_series_with_checks (const char *s, int fixmax,
 
 /* alternate forms:
 
-           op         list   s
+           @op        @list  @param
    dataset addobs            24
    dataset compact           1
    dataset compact           4 last
@@ -3212,7 +3217,7 @@ int renumber_series_with_checks (const char *s, int fixmax,
 */
 
 int modify_dataset (DATASET *dset, int op, const int *list, 
-		    const char *s, PRN *prn)
+		    const char *param, PRN *prn)
 {
     static int resampled;
     int k = 0, err = 0;
@@ -3222,7 +3227,7 @@ int modify_dataset (DATASET *dset, int op, const int *list,
     }
 
 #if 0
-    fprintf(stderr, "modify_dataset: op=%d, s='%s'\n", op, s);
+    fprintf(stderr, "modify_dataset: op=%d, param='%s'\n", op, param);
     printlist(list, "list");
 #endif
 
@@ -3259,7 +3264,11 @@ int modify_dataset (DATASET *dset, int op, const int *list,
     if (op == DS_ADDOBS || op == DS_INSOBS || 
 	op == DS_COMPACT || op == DS_RESAMPLE ||
 	op == DS_PAD_DAILY) {
-	k = dataset_int_param(&s, op, dset, &err);
+	if (param == NULL) {
+	    err = E_ARGS;
+	} else {
+	    k = dataset_int_param(&param, op, dset, &err);
+	}
 	if (err) {
 	    return err;
 	}
@@ -3278,11 +3287,12 @@ int modify_dataset (DATASET *dset, int op, const int *list,
     } else if (op == DS_INSOBS) {
 	err = insert_obs(k, dset, prn);
     } else if (op == DS_COMPACT) {
-	err = compact_data_set_wrapper(s, dset, k);
+	err = compact_data_set_wrapper(param, dset, k);
     } else if (op == DS_EXPAND) {
-	int interp = 0, n = strlen(s);
+	int n = (param == NULL)? 0 : strlen(param);
+	int interp = 0;
 
-	if (n > 0 && !strncmp(s, "interpolate", n)) {
+	if (n > 0 && !strncmp(param, "interpolate", n)) {
 	    interp = 1;
 	}
 	err = expand_data_set(dset, k, interp);
@@ -4292,7 +4302,7 @@ static int pad_daily_data (DATASET *dset, int pd, PRN *prn)
 {
     DATASET *bigset = NULL;
     char datestr[OBSLEN];
-    long ed, ed0, edbak = 0;
+    long ed, ed0 = 0, edbak = 0;
     int wd, skip, totskip = 0;
     int t, err = 0;
 
