@@ -1981,10 +1981,10 @@ get_word_and_advance (char *s, char *word, size_t maxlen)
     return (*word != '\0')? s : NULL;
 }
 
-static char *
-get_compact_method_and_advance (char *s, CompactMethod *method)
+static const char *
+get_compact_method_and_advance (const char *s, CompactMethod *method)
 {
-    char *p;
+    const char *p;
 
     *method = COMPACT_NONE;
 
@@ -2022,6 +2022,29 @@ get_compact_method_and_advance (char *s, CompactMethod *method)
     return p;
 }
 
+static CompactMethod compact_method_from_option (int *err)
+{
+    const char *s = get_optval_string(DATA, OPT_C);
+    CompactMethod method = COMPACT_NONE;
+
+    if (s == NULL || *s == '\0') {
+	*err = E_PARSE;
+    } else if (!strcmp(s, "average")) {
+	method = COMPACT_AVG;
+    } else if (!strcmp(s, "sum")) {
+	method = COMPACT_SUM;
+    } else if (!strcmp(s, "first")) {
+	method = COMPACT_SOP;
+    } else if (!strcmp(s, "last")) {
+	method = COMPACT_EOP;
+    } else {
+	gretl_errmsg_sprintf(_("field '%s' in command is invalid"), s);
+	*err = E_PARSE;
+    }
+
+    return method;
+}
+
 static double **new_dbZ (int n)
 {
     double **Z;
@@ -2149,7 +2172,7 @@ static int parse_odbc_format (char *fmt)
     return err;
 }
 
-static char *odbc_get_query (char *s, int *err)
+static char *odbc_get_query (const char *s, int *err)
 {
     char *query = NULL;
     const char *p;
@@ -2181,11 +2204,11 @@ static char *odbc_get_query (char *s, int *err)
    otherwise we're restricted to one.
 */
 
-static char **odbc_get_varnames (char **line, int *err)
+static char **odbc_get_varnames (const char **line, int *err)
 {
     char **vnames = NULL;
     char vname[VNAMELEN];
-    char *s = *line;
+    const char *s = *line;
     int len, loop_ok = 0, nv = 0;
 
     if (strstr(s, "query=")) {
@@ -2322,7 +2345,7 @@ static int odbc_count_new_vars (char **vnames, int nv,
 
 /* data series [obs-format=format-string] [query=]query-string */
 
-static int odbc_get_series (char *line, DATASET *dset, 
+static int odbc_get_series (const char *line, DATASET *dset, 
 			    PRN *prn)
 {
     int (*get_data) (ODBC_info *);
@@ -2340,10 +2363,6 @@ static int odbc_get_series (char *line, DATASET *dset,
 	return 1;
     }
 
-    /* skip "data" plus following space */
-    line += strcspn(line, " ");
-    line += strspn(line, " ");
-
     /* get "series" field */
     vnames = odbc_get_varnames(&line, &err);
     if (err) {
@@ -2584,7 +2603,7 @@ static int is_glob (const char *s)
    command-line client or in script or console mode
 */
 
-int db_get_series (char *line, DATASET *dset, 
+int db_get_series (const char *line, DATASET *dset, 
 		   gretlopt opt, PRN *prn)
 {
     const gretl_matrix *rowmask = NULL;
@@ -2595,6 +2614,12 @@ int db_get_series (char *line, DATASET *dset,
     int from_scratch = 0;
     int err = 0;
 
+    if (!strncmp(line, "data ", 5)) {
+	/* old-style parser */
+	line += 5;
+	line += strspn(line, " ");
+    }
+
     if (opt & OPT_O) {
 	return odbc_get_series(line, dset, prn);
     }
@@ -2619,25 +2644,34 @@ int db_get_series (char *line, DATASET *dset,
 
     from_scratch = (dset->n == 0);
 
-    line = get_compact_method_and_advance(line, &method);
-
-    if (string_is_blank(line)) {
-	err = E_DATA;
+    if (opt & OPT_C) {
+	/* new-style: compaction method supplied as option */
+	method = compact_method_from_option(&err);
     } else {
-	/* get the variable names on the line */
-	vnames = gretl_string_split(line, &nnames, NULL);
-	if (vnames == NULL) {
-	    err = E_ALLOC;
+	line = get_compact_method_and_advance(line, &method);
+    }
+
+    if (!err) {
+	if (string_is_blank(line)) {
+	    err = E_DATA;
+	} else {
+	    /* get the variable names on the line */
+	    vnames = gretl_string_split(line, &nnames, NULL);
+	    if (vnames == NULL) {
+		err = E_ALLOC;
+	    }
 	}
     }
 
-    if (saved_db_type == GRETL_NATIVE_DB) {
-	idxname = native_db_index_name();
-    } else if (saved_db_type == GRETL_NATIVE_DB_WWW) {
+    if (!err) {
+	if (saved_db_type == GRETL_NATIVE_DB) {
+	    idxname = native_db_index_name();
+	} else if (saved_db_type == GRETL_NATIVE_DB_WWW) {
 #ifdef USE_CURL
-	idxname = g_strdup_printf("%sdbtmp.idx", gretl_dotdir());
-	err = remote_db_index_to_file(idxname);
+	    idxname = g_strdup_printf("%sdbtmp.idx", gretl_dotdir());
+	    err = remote_db_index_to_file(idxname);
 #endif
+	}
     }
 
     /* and process them individually */
