@@ -21,6 +21,7 @@
 #include "libset.h"
 #include "missing_private.h"
 #include "gretl_bfgs.h"
+#include "gretl_normal.h"
 
 #include <errno.h>
 
@@ -2228,13 +2229,12 @@ MODEL biprobit_model (int *list, DATASET *dset,
 		      gretlopt opt, PRN *prn)
 {
     MODEL bpmod;
-    void *handle;
     MODEL (* biprobit_estimate) (const int *, DATASET *, 
 				 gretlopt, PRN *);
 
     gretl_error_clear();
 
-    biprobit_estimate = get_plugin_function("biprobit_estimate", &handle);
+    biprobit_estimate = get_plugin_function("biprobit_estimate");
 
     if (biprobit_estimate == NULL) {
 	gretl_model_init(&bpmod, dset);
@@ -2243,9 +2243,6 @@ MODEL biprobit_model (int *list, DATASET *dset,
     }
 
     bpmod = (*biprobit_estimate) (list, dset, opt, prn);
-
-    close_plugin(handle);
-
     set_model_id(&bpmod);
 
     return bpmod;
@@ -3287,7 +3284,7 @@ MODEL logistic_model (const int *list, double lmax,
     return lmod;
 }
 
-static double choose (double n, double k)
+static double choose (double n, double k, int *err)
 {
     double c = 1.0;
     int i;
@@ -3296,16 +3293,33 @@ static double choose (double n, double k)
 	c *= (n - i) / (k - i);
     }
 
+    if (xna(c)) {
+	*err = 1;
+    }
+
     return c;
 }
 
 static double table_prob (double a, double b, double c, double d,
-			  double n)
+			  double n, int *err)
 {
-    double p1 = choose(a+b, a);
-    double p2 = choose(c+d, c);
-    double p3 = choose(n, a+c);
-    double P = p1 * p2 / p3;
+    double p1, p2, p3, P = NADBL;
+
+    p1 = choose(a+b, a, err);
+    if (!*err) {
+	p2 = choose(c+d, c, err);
+    }
+    if (!*err) {
+	p3 = choose(n, a+c, err);
+    }
+
+    if (!*err) {
+	P = p1 * p2 / p3;
+	if (xna(P)) {
+	    *err = 1;
+	    P = NADBL;
+	}
+    }
 
 #if 0
     fprintf(stderr, "\ntable_prob: a=%g, b=%g, c=%g, d=%g, n=%g\n", 
@@ -3332,11 +3346,17 @@ int fishers_exact_test (const Xtab *tab, PRN *prn)
 {
     double a, b, c, d, n, E0;
     double P0, Pi, PL, PR, P2;
+    int err = 0;
 
     if (tab->rows != 2 || tab->cols != 2) {
 	return E_DATA;
     }
 
+    if (tab->n > 1000) {
+	/* not worth trying? */
+	return E_DATA;
+    }
+
     a = tab->f[0][0];
     b = tab->f[0][1];
     c = tab->f[1][0];
@@ -3346,44 +3366,56 @@ int fishers_exact_test (const Xtab *tab, PRN *prn)
     E0 = (tab->rtotal[0] * tab->ctotal[0]) / n;
 
     /* Probability of the observed table */
-    PL = PR = P2 = P0 = table_prob(a, b, c, d, n);
+    PL = PR = P2 = P0 = table_prob(a, b, c, d, n, &err);
 
-    while (a > 0 && d > 0) {
-	a -= 1; d -= 1;
-	c += 1; b += 1;
-	Pi = table_prob(a, b, c, d, n);
-	if (Pi <= P0 || tab->f[0][0] > E0) {
-	    PL += Pi;
-	}
-	if (Pi <= P0) {
-	    P2 += Pi;
+    if (!err) {
+	while (a > 0 && d > 0) {
+	    a -= 1; d -= 1;
+	    c += 1; b += 1;
+	    Pi = table_prob(a, b, c, d, n, &err);
+	    if (err) {
+		break;
+	    }
+	    if (Pi <= P0 || tab->f[0][0] > E0) {
+		PL += Pi;
+	    }
+	    if (Pi <= P0) {
+		P2 += Pi;
+	    }
 	}
     }
 
-    a = tab->f[0][0];
-    b = tab->f[0][1];
-    c = tab->f[1][0];
-    d = tab->f[1][1];
+    if (!err) {
+	a = tab->f[0][0];
+	b = tab->f[0][1];
+	c = tab->f[1][0];
+	d = tab->f[1][1];
 
-    while (c > 0 && b > 0) {
-	c -= 1; b -= 1;
-	a += 1; d += 1;
-	Pi = table_prob(a, b, c, d, n);
-	if (Pi <= P0 || tab->f[0][0] < E0) {
-	    PR += Pi;
-	}
-	if (Pi <= P0) {
-	    P2 += Pi;
+	while (c > 0 && b > 0) {
+	    c -= 1; b -= 1;
+	    a += 1; d += 1;
+	    Pi = table_prob(a, b, c, d, n, &err);
+	    if (err) {
+		break;
+	    }
+	    if (Pi <= P0 || tab->f[0][0] < E0) {
+		PR += Pi;
+	    }
+	    if (Pi <= P0) {
+		P2 += Pi;
+	    }
 	}
     } 
 
-    pprintf(prn, "\n%s:\n", _("Fisher's Exact Test"));
-    pprintf(prn, "  Left:   P-value = %g\n", PL);
-    pprintf(prn, "  Right:  P-value = %g\n", PR);
-    pprintf(prn, "  2-Tail: P-value = %g\n", P2);
-    pputc(prn, '\n');
+    if (!err) {
+	pprintf(prn, "\n%s:\n", _("Fisher's Exact Test"));
+	pprintf(prn, "  Left:   P-value = %g\n", PL);
+	pprintf(prn, "  Right:  P-value = %g\n", PR);
+	pprintf(prn, "  2-Tail: P-value = %g\n", P2);
+	pputc(prn, '\n');
+    }
 
-    return 0;
+    return err;
 }
 
 /**
@@ -3404,13 +3436,12 @@ MODEL interval_model (int *list, DATASET *dset,
 		      gretlopt opt, PRN *prn)
 {
     MODEL intmod;
-    void *handle;
     MODEL (* interval_estimate) (int *, DATASET *, gretlopt, 
 				 PRN *);
 
     gretl_error_clear();
 
-    interval_estimate = get_plugin_function("interval_estimate", &handle);
+    interval_estimate = get_plugin_function("interval_estimate");
 
     if (interval_estimate == NULL) {
 	gretl_model_init(&intmod, dset);
@@ -3419,9 +3450,6 @@ MODEL interval_model (int *list, DATASET *dset,
     }
 
     intmod = (*interval_estimate) (list, dset, opt, prn);
-
-    close_plugin(handle);
-
     set_model_id(&intmod);
 
     return intmod;
@@ -3450,12 +3478,11 @@ MODEL tobit_model (const int *list, double llim, double rlim,
 {
     MODEL (* tobit_estimate) (const int *, double, double,
 			      DATASET *, gretlopt, PRN *);
-    void *handle;
     MODEL tmod;    
 
     gretl_error_clear();
 
-    tobit_estimate = get_plugin_function("tobit_via_intreg", &handle);
+    tobit_estimate = get_plugin_function("tobit_via_intreg");
     if (tobit_estimate == NULL) {
 	gretl_model_init(&tmod, dset);
 	tmod.errcode = E_FOPEN;
@@ -3463,8 +3490,6 @@ MODEL tobit_model (const int *list, double llim, double rlim,
     }
 
     tmod = (*tobit_estimate) (list, llim, rlim, dset, opt, prn);
-
-    close_plugin(handle);
     set_model_id(&tmod);
 
     return tmod;
@@ -3568,7 +3593,6 @@ MODEL duration_model (const int *list, DATASET *dset,
 		      gretlopt opt, PRN *prn)
 {
     MODEL dmod;
-    void *handle;
     int censvar = 0;
     int (* duration_estimate) (MODEL *, int, const DATASET *, 
 			       gretlopt, PRN *);
@@ -3582,8 +3606,7 @@ MODEL duration_model (const int *list, DATASET *dset,
         return dmod;
     }
 
-    duration_estimate = get_plugin_function("duration_estimate", 
-					    &handle);
+    duration_estimate = get_plugin_function("duration_estimate");
 
     if (duration_estimate == NULL) {
 	dmod.errcode = E_FOPEN;
@@ -3591,9 +3614,6 @@ MODEL duration_model (const int *list, DATASET *dset,
     }
 
     (*duration_estimate) (&dmod, censvar, dset, opt, prn);
-
-    close_plugin(handle);
-
     set_model_id(&dmod);
 
     return dmod;
@@ -3630,7 +3650,6 @@ MODEL count_model (const int *list, int ci, DATASET *dset,
 		   gretlopt opt, PRN *prn)
 {
     MODEL cmod;
-    void *handle;
     int *listcpy;
     int offvar;
     int (* count_data_estimate) (MODEL *, int, int,
@@ -3668,18 +3687,13 @@ MODEL count_model (const int *list, int ci, DATASET *dset,
         return cmod;
     }
 
-    count_data_estimate = get_plugin_function("count_data_estimate", 
-					      &handle);
-
+    count_data_estimate = get_plugin_function("count_data_estimate"); 
     if (count_data_estimate == NULL) {
 	cmod.errcode = E_FOPEN;
 	return cmod;
     }
 
     (*count_data_estimate) (&cmod, ci, offvar, dset, opt, prn);
-
-    close_plugin(handle);
-
     set_model_id(&cmod);
 
     return cmod;
@@ -3704,13 +3718,12 @@ MODEL heckit_model (const int *list, DATASET *dset,
 		    gretlopt opt, PRN *prn)
 {
     MODEL model;
-    void *handle;
     MODEL (* heckit_estimate) (const int *, DATASET *, 
 			       gretlopt, PRN *);
 
     gretl_error_clear();
 
-    heckit_estimate = get_plugin_function("heckit_estimate", &handle);
+    heckit_estimate = get_plugin_function("heckit_estimate");
     if (heckit_estimate == NULL) {
 	gretl_model_init(&model, dset);
 	model.errcode = E_FOPEN;
@@ -3718,9 +3731,6 @@ MODEL heckit_model (const int *list, DATASET *dset,
     }
 
     model = (*heckit_estimate) (list, dset, opt, prn);
-
-    close_plugin(handle);
-
     set_model_id(&model);
 
     return model;
@@ -3743,7 +3753,6 @@ MODEL reprobit_model (const int *list, DATASET *dset,
 		      gretlopt opt, PRN *prn)
 {
     MODEL model;
-    void *handle;
     MODEL (* reprobit_estimate) (const int *, DATASET *,
 				 gretlopt, PRN *);
     int err = 0;
@@ -3753,7 +3762,7 @@ MODEL reprobit_model (const int *list, DATASET *dset,
     if (!dataset_is_panel(dset)) {
 	err = E_PDWRONG;
     } else {	
-	reprobit_estimate = get_plugin_function("reprobit_estimate", &handle);
+	reprobit_estimate = get_plugin_function("reprobit_estimate");
 	if (reprobit_estimate == NULL) {
 	    err = E_FOPEN;
 	}
@@ -3766,9 +3775,6 @@ MODEL reprobit_model (const int *list, DATASET *dset,
     }
 
     model = (*reprobit_estimate) (list, dset, opt, prn);
-
-    close_plugin(handle);
-
     set_model_id(&model);
 
     return model;
