@@ -1298,7 +1298,7 @@ static int want_x_beta_prep (const MODEL *pmod, const int *xlist)
     if (xlist != NULL) {
 	int aflags = gretl_model_get_int(pmod, "arma_flags");
 
-	if ((aflags & ARMA_EXACT) || (aflags & ARMA_X12A)) {
+	if (aflags & (ARMA_EXACT | ARMA_X12A)) {
 	    ret = 1;
 	}
     }
@@ -2587,13 +2587,6 @@ FITRESID *get_forecast (MODEL *pmod, int t1, int t2, int pre_n,
 {
     FITRESID *fr;
 
-    /* Reject in case model was estimated using repacked daily
-       data: this case should be handled more elegantly */
-    if (gretl_model_get_int(pmod, "daily_repack")) {
-	*err = E_DATA;
-	return NULL;
-    }
-
     fr = fit_resid_new_for_model(pmod, dset, t1, t2, pre_n, err);
 
     if (!*err) {
@@ -2954,12 +2947,6 @@ static int model_do_forecast (const char *str, MODEL *pmod,
 	return E_NOTIMP;
     }    
 
-     /* Reject in case model was estimated using repacked daily
-       data: this case should be handled more elegantly */
-    if (gretl_model_get_int(pmod, "daily_repack")) {
-	return E_DATA;
-    }
-
     /* OPT_I for integrate: reject for non-OLS, or if the dependent 
        variable is not recognized as a first difference */
     if (opt & OPT_I) {
@@ -3544,11 +3531,6 @@ rolling_OLS_k_step_fcast (MODEL *pmod, DATASET *dset,
 	return NULL;
     }    
 
-    if (gretl_model_get_int(pmod, "daily_repack")) {
-	*err = E_DATA;
-	return NULL;
-    }
-
     /* check feasibility of forecast range */
     *err = rolling_fcast_adjust_obs(pmod, &t1, t2, k);
     if (*err) {
