@@ -303,10 +303,10 @@ struct str_table funcs[] = {
     { F_BFGSMAX,  "BFGSmax" },
     { F_NRMAX,    "NRmax" },
     { F_OBSNUM,   "obsnum" },
-    { F_ISSERIES, "isseries" },
-    { F_ISLIST,   "islist" },
-    { F_ISSTRING, "isstring" },
+    { F_ISDISCR,  "isdiscrete" },
+    { F_ISDUMMY,  "isdummy"},
     { F_ISNULL,   "isnull" },
+    { F_ISSTRING, "isstring" },
     { F_NELEM,    "nelem" },
     { F_PDF,      "pdf" },
     { F_CDF,      "cdf" },
@@ -1122,6 +1122,9 @@ static void word_check_next_char (parser *p)
 	    }
 	    p->upsym = USERIES;
 	    p->sym = LAG;
+	} else if (p->sym == ULIST) {
+	    p->upsym = ULIST;
+	    p->sym = LAG;
 	} else if (p->sym == MVAR && model_data_matrix(p->idnum)) {
 	    /* old-style "$coeff(x1)" etc. */
 	    p->sym = DMSTR;
