@@ -219,10 +219,10 @@ enum {
     F_RCOND,
     F_RANK, 
     F_OBSNUM,
-    F_ISSERIES,
-    F_ISLIST,	 
-    F_ISSTRING,
+    F_ISDISCR,
+    F_ISDUMMY,
     F_ISNULL,
+    F_ISSTRING,
     F_NELEM,
     F_PDF,
     F_PVAL,
@@ -348,6 +348,7 @@ enum {
     F_ISODATE,
     F_READFILE,
     F_PRINTF,
+    F_SPRINTF,
     F_MPI_SEND,
     F_BCAST,
     F_ALLREDUCE,
@@ -391,7 +392,6 @@ enum {
     F_AGGRBY,
     F_IWISHART,
     F_SSCANF,
-    F_SPRINTF,
     F_SUBSTR,
     F_REDUCE,
     F_SCATTER,
@@ -442,12 +442,11 @@ enum {
 #define funcn_symb(s) (s > F3_MAX && s < FN_MAX)
 
 /* function with single string argument */
-#define string_arg_func(s) (s == F_ISSERIES || s == F_ISNULL || \
-			    s == F_ISLIST   || s == F_ISSTRING || \
+#define string_arg_func(s) (s == F_ISNULL || s == F_ISDISCR || \
 			    s == F_OBSNUM || s == F_BACKTICK || \
 			    s == F_VARNUM || s == F_ARGNAME || \
                             s == F_REMOVE || s == F_TYPEOF || \
-			    s == F_ATOF)
+			    s == F_ATOF   || s == F_ISSTRING)
 
 /* function with multiple args, string for first arg */
 #define str0_func(s) (s == F_PVAL || s == F_CDF || s == F_INVCDF || \
