@@ -256,6 +256,7 @@ static void expected_symbol_error (int c, parser *p)
 
 static void unmatched_symbol_error (int c, parser *p)
 {
+    fprintf(stderr, "*** gensyntax: unmatched_symbol_error (%c)\n", c);
     parser_print_input(p);
     pprintf(p->prn, _("Unmatched '%c'\n"), c);
     p->err = E_PARSE;
@@ -562,7 +563,9 @@ static NODE *get_final_string_arg (parser *p, NODE *t, int sym,
     if (!varargs_func(sym)) {
 	if (wrapped) {
 	    unwrap_string_arg(p);
-	} else if (sym != F_ISSTRING && sym != F_ISNULL) {
+	} else if (sym != F_ISDISCR && 
+		   sym != F_ISNULL &&
+		   sym != F_ISSTRING) {
 	    /* not quoted: give priority to string variables
 	       unless we need the _names_ of string variables 
 	       rather then their content
@@ -833,7 +836,7 @@ static void get_matrix_def (NODE *t, parser *p, int *sub)
 	cexp = '{';
     }
 
-    if (cexp == 0) {
+    if (!p->err && cexp == 0) {
 	if (p->sym == G_RCB) {
 	    lex(p);
 	} else {
