@@ -23,9 +23,11 @@
 #include "gretl_func.h"
 #include "libset.h"
 #include "uservar.h"
+#include "gretl_array.h"
 #include "gretl_xml.h"
-
-#include <glib.h>
+#include "gretl_foreign.h"
+#include "gretl_typemap.h"
+#include "gretl_bundle.h"
 
 #define BDEBUG 0
 
@@ -54,9 +56,12 @@ struct bundled_item_ {
     char *note;
 };
 
+static gretl_bundle *sysinfo_bundle;
+
 static int real_bundle_set_data (gretl_bundle *b, const char *key,
 				 void *ptr, GretlType type,
-				 int size, const char *note);
+				 int size, int copy,
+				 const char *note);
 
 int gretl_bundle_set_name (gretl_bundle *b, const char *name)
 {
@@ -100,14 +105,16 @@ int type_can_be_bundled (GretlType type)
 	    type == GRETL_TYPE_MATRIX ||
 	    type == GRETL_TYPE_MATRIX_REF ||
 	    type == GRETL_TYPE_SERIES ||
-	    type == GRETL_TYPE_BUNDLE);
+	    type == GRETL_TYPE_BUNDLE ||
+	    type == GRETL_TYPE_ARRAY);
 }
 
 /* allocate and fill out a 'value' (type plus data pointer) that will
    be inserted into a bundle's hash table */
 
 static bundled_item *bundled_item_new (GretlType type, void *ptr, 
-				       int size, const char *note,
+				       int size, int copy,
+				       const char *note,
 				       int *err)
 {
     bundled_item *item = malloc(sizeof *item);
@@ -129,20 +136,43 @@ static bundled_item *bundled_item_new (GretlType type, void *ptr,
 	    }
 	    break;
 	case GRETL_TYPE_STRING:	
-	    item->data = gretl_strdup((char *) ptr);
+	    if (copy) {
+		item->data = gretl_strdup((char *) ptr);
+	    } else {
+		item->data = ptr;
+	    }
 	    break;
 	case GRETL_TYPE_MATRIX:
-	    item->data = gretl_matrix_copy((gretl_matrix *) ptr);
+	    if (copy) {
+		item->data = gretl_matrix_copy((gretl_matrix *) ptr);
+	    } else {
+		item->data = ptr;
+	    }
 	    break;
 	case GRETL_TYPE_MATRIX_REF:
 	    item->data = ptr;
 	    break;
 	case GRETL_TYPE_SERIES:
-	    item->data = copyvec((const double *) ptr, size);
+	    if (copy) {
+		item->data = copyvec((const double *) ptr, size);
+	    } else {
+		item->data = ptr;
+	    }
 	    item->size = size;
 	    break;
 	case GRETL_TYPE_BUNDLE:
-	    item->data = gretl_bundle_copy((gretl_bundle *) ptr, err);
+	    if (copy) {
+		item->data = gretl_bundle_copy((gretl_bundle *) ptr, err);
+	    } else {
+		item->data = ptr;
+	    }
+	    break;
+	case GRETL_TYPE_ARRAY:
+	    if (copy) {
+		item->data = gretl_array_copy((gretl_array *) ptr, err);
+	    } else {
+		item->data = ptr;
+	    }
 	    break;
 	default:
 	    *err = E_TYPES;
@@ -163,6 +193,88 @@ static bundled_item *bundled_item_new (GretlType type, void *ptr,
     return item;
 }
 
+static void release_matrix_pointer (gretl_matrix **pm)
+{
+    const void *data = *pm;
+
+    if (get_user_var_by_data(data) == NULL) {
+	/* the bundle now has the only pointer to
+	   this matrix */
+	gretl_matrix_free(*pm);
+    }
+
+    *pm = NULL;
+}
+
+static int bundled_item_replace_data (bundled_item *item,
+				      GretlType type, void *ptr, 
+				      int size, int copy)
+{
+    int err = 0;
+
+    if (ptr == item->data) {
+	return 0;
+    }
+
+    if (item->type == GRETL_TYPE_DOUBLE) {
+	double *dp = item->data;
+
+	*dp = *(double *) ptr;
+    } else if (item->type == GRETL_TYPE_STRING) {
+	free(item->data);
+	if (copy) {
+	    item->data = gretl_strdup((char *) ptr);
+	} else {
+	    item->data = ptr;
+	}
+    } else if (item->type == GRETL_TYPE_MATRIX) {
+	gretl_matrix_free(item->data);
+	if (copy) {
+	    item->data = gretl_matrix_copy((gretl_matrix *) ptr);
+	} else {
+	    item->data = ptr;
+	}
+    } else if (item->type == GRETL_TYPE_MATRIX_REF) {
+	release_matrix_pointer((gretl_matrix **) &item->data);
+	item->data = ptr;
+    } else if (item->type == GRETL_TYPE_SERIES) {
+	free(item->data);
+	if (copy) {
+	    item->data = copyvec((const double *) ptr, size);
+	} else {
+	    item->data = ptr;
+	}
+	item->size = size;
+    } else if (item->type == GRETL_TYPE_BUNDLE) {
+	gretl_bundle_destroy((gretl_bundle *) item->data);
+	if (copy) {
+	    item->data = gretl_bundle_copy((gretl_bundle *) ptr, &err);
+	} else {
+	    item->data = ptr;
+	}
+    } else if (item->type == GRETL_TYPE_ARRAY) {
+	gretl_array_destroy((gretl_array*) item->data);
+	if (copy) {
+	    item->data = gretl_array_copy((gretl_array *) ptr, &err);
+	} else {
+	    item->data = ptr;
+	}	
+    } else {
+	return E_DATA;
+    }
+
+    if (!err && item->data == NULL) {
+	err = E_ALLOC;
+    }
+
+    if (item->note != NULL) {
+	free(item->note);
+	item->note = NULL;
+    }
+
+    return err;
+}
+
 /* callback invoked when a bundle's hash table is destroyed */
 
 static void bundled_item_destroy (gpointer data)
@@ -186,11 +298,14 @@ static void bundled_item_destroy (gpointer data)
 	gretl_matrix_free((gretl_matrix *) item->data);
 	break;
     case GRETL_TYPE_MATRIX_REF:
-	item->data = NULL;
+	release_matrix_pointer((gretl_matrix **) &item->data);
 	break;
     case GRETL_TYPE_BUNDLE:
 	gretl_bundle_destroy((gretl_bundle *) item->data);
 	break;
+    case GRETL_TYPE_ARRAY:
+	gretl_array_destroy((gretl_array*) item->data);
+	break;
     default:
 	break;
     }
@@ -561,7 +676,7 @@ int gretl_bundle_set_payload_matrix (gretl_bundle *bundle,
 				     const gretl_matrix *m)
 {
     return real_bundle_set_data(bundle, "payload", (void *) m,
-				GRETL_TYPE_MATRIX, 0, NULL);
+				GRETL_TYPE_MATRIX, 0, 1, NULL);
 }
 
 /**
@@ -681,7 +796,10 @@ void *bundled_item_get_data (bundled_item *item, GretlType *type,
 			     int *size)
 {
     *type = item->type;
-    *size = item->size;
+
+    if (size != NULL) {
+	*size = item->size;
+    }
 
     return item->data;
 }
@@ -713,23 +831,32 @@ void *gretl_bundle_get_content (gretl_bundle *bundle)
 
 static int real_bundle_set_data (gretl_bundle *b, const char *key,
 				 void *ptr, GretlType type,
-				 int size, const char *note)
+				 int size, int copy, 
+				 const char *note)
 {
     bundled_item *item = NULL;
-    int err;
+    int err, replace = 0;
 
     err = strlen(key) >= VNAMELEN ? E_DATA : 0;
 
     if (err) {
 	gretl_errmsg_sprintf("'%s': invalid key string", key);
     } else {
-	item = bundled_item_new(type, ptr, size, note, &err);
+	item = g_hash_table_lookup(b->ht, key);
+	if (item != NULL) {
+	    replace = 1;
+	    if (item->type == type) {
+		return bundled_item_replace_data(item, type, ptr, 
+						 size, copy);
+	    }
+	}
+	item = bundled_item_new(type, ptr, size, copy, note, &err);
     }
 
     if (!err) {
 	gchar *k = g_strdup(key);
 
-	if (g_hash_table_lookup(b->ht, key) != NULL) {
+	if (replace) {
 	    g_hash_table_replace(b->ht, k, item);
 	} else {
 	    g_hash_table_insert(b->ht, k, item);
@@ -739,6 +866,31 @@ static int real_bundle_set_data (gretl_bundle *b, const char *key,
     return err;
 }
 
+/**
+ * gretl_bundle_donate_data:
+ * @bundle: target bundle.
+ * @key: name of key to create or replace.
+ * @ptr: data pointer.
+ * @type: type of data.
+ * @size: if @type == GRETL_TYPE_SERIES, the length of
+ * the series, otherwise 0.
+ * 
+ * Sets the data type and pointer to be associated with @key in 
+ * the bundle given by @name. If @key is already present in
+ * the bundle's hash table the original value is replaced
+ * and destroyed. The value of @ptr is transcribed into the
+ * bundle, which therefore "takes ownership" of the data;
+ * compare gretl_bundle_set_data;
+ *
+ * Returns: 0 on success, error code on error.
+ */
+
+int gretl_bundle_donate_data (gretl_bundle *bundle, const char *key,
+			      void *ptr, GretlType type, int size)
+{
+    return real_bundle_set_data(bundle, key, ptr, type, size, 0, NULL);
+}
+
 /**
  * gretl_bundle_set_data:
  * @bundle: target bundle.
@@ -751,7 +903,8 @@ static int real_bundle_set_data (gretl_bundle *b, const char *key,
  * Sets the data type and pointer to be associated with @key in 
  * the bundle given by @name. If @key is already present in
  * the bundle's hash table the original value is replaced
- * and destroyed.
+ * and destroyed. The content of @ptr is copied into the
+ * bundle; compare gretl_bundle_donate_data().
  *
  * Returns: 0 on success, error code on error.
  */
@@ -764,7 +917,8 @@ int gretl_bundle_set_data (gretl_bundle *bundle, const char *key,
     if (bundle == NULL) {
 	err = E_UNKVAR;
     } else {
-	err = real_bundle_set_data(bundle, key, ptr, type, size, NULL);
+	err = real_bundle_set_data(bundle, key, ptr, type, 
+				   size, 1, NULL);
     }
 
     return err;
@@ -958,7 +1112,7 @@ static void copy_new_bundled_item (gpointer key, gpointer value, gpointer p)
     if (!gretl_bundle_has_data(targ, (const char *) key)) {
 	real_bundle_set_data(targ, (const char *) key,
 			     item->data, item->type,
-			     item->size, item->note);
+			     item->size, 1, item->note);
     }
 }
 
@@ -972,7 +1126,7 @@ static void copy_bundled_item (gpointer key, gpointer value, gpointer p)
 
     real_bundle_set_data(targ, (const char *) key,
 			 item->data, item->type,
-			 item->size, item->note);
+			 item->size, 1, item->note);
 }
 
 /* Create a new bundle as the union of two existing bundles:
@@ -1006,7 +1160,7 @@ gretl_bundle *gretl_bundle_union (const gretl_bundle *bundle1,
  * Look for a saved bundle specified by @name, and if found,
  * make a full copy and save it under @copyname. This is
  * called from geneval.c on completion of assignment to a
- * bundle named @cpyname, where the returned value on the
+ * bundle named @copyname, where the returned value on the
  * right-hand side is a pre-existing saved bundle.
  *
  * Returns: 0 on success, non-zero code on error.
@@ -1018,14 +1172,17 @@ int gretl_bundle_copy_as (const char *name, const char *copyname)
     user_var *u;
     int err = 0;
 
-    u = get_user_var_of_type_by_name(name, GRETL_TYPE_BUNDLE);
-
-    if (u == NULL) {
-	return E_DATA;
+    if (!strcmp(name, "$sysinfo")) {
+	b0 = sysinfo_bundle;
+    } else {
+	u = get_user_var_of_type_by_name(name, GRETL_TYPE_BUNDLE);
+	if (u == NULL) {
+	    return E_DATA;
+	} else {
+	    b0 = user_var_get_value(u);
+	}
     }
 
-    b0 = user_var_get_value(u);
-
     u = get_user_var_of_type_by_name(copyname, GRETL_TYPE_BUNDLE);
     if (u != NULL) {
 	b1 = user_var_get_value(u);
@@ -1084,6 +1241,7 @@ static void print_bundled_item (gpointer key, gpointer value, gpointer p)
 {
     bundled_item *item = value;
     const gchar *kstr = key;
+    gretl_array *a;
     gretl_matrix *m;
     double x;
     char *s;
@@ -1104,12 +1262,12 @@ static void print_bundled_item (gpointer key, gpointer value, gpointer p)
 	    pprintf(prn, " %s = %s", kstr, s);
 	} else {
 	    pprintf(prn, " %s (%s)", kstr, 
-		    gretl_arg_type_name(item->type));
+		    gretl_type_get_name(item->type));
 	}	
 	break;
     case GRETL_TYPE_BUNDLE:
 	pprintf(prn, " %s (%s)", kstr, 
-		gretl_arg_type_name(item->type));
+		gretl_type_get_name(item->type));
 	break;
     case GRETL_TYPE_MATRIX:
     case GRETL_TYPE_MATRIX_REF:
@@ -1118,13 +1276,23 @@ static void print_bundled_item (gpointer key, gpointer value, gpointer p)
 	    pprintf(prn, " %s = %g", kstr, m->val[0]);
 	} else {
 	    pprintf(prn, " %s (%s: %d x %d)", kstr, 
-		    gretl_arg_type_name(item->type), 
+		    gretl_type_get_name(item->type), 
 		    m->rows, m->cols);
 	}
 	break;
     case GRETL_TYPE_SERIES:
 	pprintf(prn, " %s (%s: length %d)", kstr, 
-		gretl_arg_type_name(item->type), item->size);
+		gretl_type_get_name(item->type), item->size);
+	break;
+    case GRETL_TYPE_ARRAY:
+	a = item->data;
+	{
+	    GretlType t = gretl_array_get_type(a);
+	    int n = gretl_array_get_length(a);
+
+	    pprintf(prn, " %s = array of %s, length %d", kstr, 
+		    gretl_type_get_name(t), n);
+	}
 	break;
     default:
 	break;
@@ -1153,9 +1321,15 @@ int gretl_bundle_print (gretl_bundle *bundle, PRN *prn)
     if (bundle == NULL) {
 	return E_DATA;
     } else {
-	user_var *u = get_user_var_by_data(bundle);
-	const char *name = user_var_get_name(u);
 	int n_items = g_hash_table_size(bundle->ht);
+	user_var *u = get_user_var_by_data(bundle);
+	const char *name = NULL;
+
+	if (u != NULL) {
+	    name = user_var_get_name(u);
+	} else {
+	    name = "anonymous";
+	}
 
 	if (n_items == 0) {
 	    pprintf(prn, "bundle %s: empty\n", name);
@@ -1229,7 +1403,7 @@ static void xml_put_bundled_item (gpointer keyp, gpointer value, gpointer p)
     }
 
     fprintf(fp, "<bundled-item key=\"%s\" type=\"%s\"", key,
-	    gretl_arg_type_name(item->type));
+	    gretl_type_get_name(item->type));
 
     if (item->note != NULL) {
 	fprintf(fp, " note=\"%s\"", item->note);
@@ -1249,7 +1423,7 @@ static void xml_put_bundled_item (gpointer keyp, gpointer value, gpointer p)
 	if (na(x)) {
 	    fputs("NA", fp);
 	} else {
-	    fprintf(fp, "%.15g", x);
+	    fprintf(fp, "%.16g", x);
 	}
     } else if (item->type == GRETL_TYPE_SERIES) {
 	double *vals = (double *) item->data;
@@ -1258,15 +1432,23 @@ static void xml_put_bundled_item (gpointer keyp, gpointer value, gpointer p)
 	    if (na(vals[j])) {
 		fputs("NA ", fp);
 	    } else {
-		fprintf(fp, "%.15g ", vals[j]);
+		fprintf(fp, "%.16g ", vals[j]);
 	    }
 	}	    
     } else if (item->type == GRETL_TYPE_STRING) {
-	fputs((char *) item->data, fp);
+	gretl_xml_put_string((char *) item->data, fp);
     } else if (item->type == GRETL_TYPE_MATRIX) {
 	gretl_matrix *m = (gretl_matrix *) item->data;
 
-	gretl_xml_put_matrix(m, NULL, fp);
+	gretl_matrix_serialize(m, NULL, fp);
+    } else if (item->type == GRETL_TYPE_BUNDLE) {
+	gretl_bundle *b = (gretl_bundle *) item->data;
+
+	gretl_bundle_serialize(b, NULL, fp);
+    } else if (item->type == GRETL_TYPE_ARRAY) {
+	gretl_array *a = (gretl_array *) item->data;
+
+	gretl_array_serialize(a, fp);
     } else {
 	fprintf(stderr, "bundle -> XML: skipping %s\n", key);
     }
@@ -1274,54 +1456,38 @@ static void xml_put_bundled_item (gpointer keyp, gpointer value, gpointer p)
     fputs("</bundled-item>\n", fp);    
 }
 
-void xml_put_bundle (gretl_bundle *b, const char *name, FILE *fp)
+void gretl_bundle_serialize (gretl_bundle *b, const char *name, 
+			     FILE *fp)
 {
-    if (b->creator != NULL && *b->creator != '\0') {
+    if (name == NULL) {
+	fputs("<gretl-bundle>\n", fp);
+    } else if (b->creator != NULL && *b->creator != '\0') {
 	fprintf(fp, "<gretl-bundle name=\"%s\" creator=\"%s\">\n", 
 		name, b->creator);
     } else {
 	fprintf(fp, "<gretl-bundle name=\"%s\">\n", name);
     }
-    g_hash_table_foreach(b->ht, xml_put_bundled_item, fp);
+
+    if (b->ht != NULL) {
+	g_hash_table_foreach(b->ht, xml_put_bundled_item, fp);
+    }
+
     fputs("</gretl-bundle>\n", fp); 
 }
 
-/* For internal use only, called from uservar.c: @p1 should be of
-   type xmlNodePtr and @p2 should be an xmlDocPtr. We suppress the
-   actual pointer types in the prototype so that it's possible for a
-   module to include gretl_bundle.h without including the full libxml
-   headers.  
-*/
-
-int load_bundle_from_xml (void *p1, void *p2, const char *name,
-			  const char *creator)
+static int load_bundled_items (gretl_bundle *b, xmlNodePtr cur, xmlDocPtr doc)
 {
-    xmlNodePtr node = (xmlNodePtr) p1;
-    xmlDocPtr doc = (xmlDocPtr) p2;
-    xmlNodePtr cur = node->xmlChildrenNode;
-    gretl_bundle *b;
-    char *key, *typename;
+    GretlType type;
+    char *key;
     int err = 0;
 
-    b = gretl_bundle_new();
-    if (b == NULL) {
-	return E_ALLOC;
-    }
-
-    fprintf(stderr, "load_bundle_from_xml: '%s'\n", name);
-
-    if (creator != NULL && *creator != '\0') {
-	b->creator = gretl_strdup(creator);
-    }
-
     while (cur != NULL && !err) {
         if (!xmlStrcmp(cur->name, (XUC) "bundled-item")) {
 	    key = (char *) xmlGetProp(cur, (XUC) "key");
-	    typename = (char *) xmlGetProp(cur, (XUC) "type");
-	    if (key == NULL || typename == NULL) {
+	    type = gretl_xml_get_type_property(cur);
+	    if (key == NULL || type == 0) {
 		err = E_DATA;
 	    } else {
-		int type = gretl_type_from_string(typename);
 		int size = 0;
 
 		if (type == GRETL_TYPE_DOUBLE) {
@@ -1338,15 +1504,13 @@ int load_bundle_from_xml (void *p1, void *p2, const char *name,
 		    if (!gretl_xml_node_get_trimmed_string(cur, doc, &s)) {
 			err = E_DATA;
 		    } else {
-			err = gretl_bundle_set_data(b, key, s, type, size);
-			free(s);
+			err = gretl_bundle_donate_data(b, key, s, type, size);
 		    }
 		} else if (type == GRETL_TYPE_SERIES) {
 		    double *xvec = gretl_xml_get_double_array(cur, doc, &size, &err);
 
 		    if (!err) {
-			err = gretl_bundle_set_data(b, key, xvec, type, size);
-			free(xvec);
+			err = gretl_bundle_donate_data(b, key, xvec, type, size);
 		    }
 		} else if (type == GRETL_TYPE_MATRIX) {
 		    xmlNodePtr child = cur->xmlChildrenNode;
@@ -1357,8 +1521,31 @@ int load_bundle_from_xml (void *p1, void *p2, const char *name,
 		    } else {
 			m = gretl_xml_get_matrix(child, doc, &err);
 			if (!err) {
-			    err = gretl_bundle_set_data(b, key, m, type, size);
-			    gretl_matrix_free(m);
+			    err = gretl_bundle_donate_data(b, key, m, type, size);
+			}
+		    }
+		} else if (type == GRETL_TYPE_BUNDLE) {
+		    xmlNodePtr child = cur->xmlChildrenNode;
+		    gretl_bundle *baby;
+
+		    if (child == NULL) {
+			err = E_DATA;
+		    } else {
+			baby = gretl_bundle_deserialize(child, doc, &err);
+			if (!err) {
+			    err = gretl_bundle_donate_data(b, key, baby, type, size);
+			}
+		    }
+		} else if (type == GRETL_TYPE_ARRAY) {
+		    xmlNodePtr child = cur->xmlChildrenNode;
+		    gretl_array *a;
+
+		    if (child == NULL) {
+			err = E_DATA;
+		    } else {
+			a = gretl_array_deserialize(child, doc, &err);
+			if (!err) {
+			    err = gretl_bundle_donate_data(b, key, a, type, size);
 			}
 		    }
 		} else {
@@ -1375,20 +1562,199 @@ int load_bundle_from_xml (void *p1, void *p2, const char *name,
 		}
 
 		xmlFree(key);
-		xmlFree(typename);
 	    }
 	}
 	cur = cur->next;
     }
 
-    if (!err) {
-	err = user_var_add(name, GRETL_TYPE_BUNDLE, b);
-	fprintf(stderr, "gretl_bundle_push: err = %d\n", err);
+    return err;
+}
+
+/* For internal use only: @p1 should be of type xmlNodePtr and @p2
+   should be an xmlDocPtr. We suppress the actual pointer types in the
+   prototype so that it's possible for a module to include
+   gretl_bundle.h without including the full libxml headers.
+*/
+
+gretl_bundle *gretl_bundle_deserialize (void *p1, void *p2, 
+					int *err)
+{
+    xmlNodePtr node = p1;
+    xmlDocPtr doc = p2;
+    xmlNodePtr cur = node->xmlChildrenNode;
+    gretl_bundle *b = gretl_bundle_new();
+
+    if (b == NULL) {
+	*err = E_ALLOC;
     } else {
-	gretl_bundle_destroy(b);
-	fprintf(stderr, "bundle is broken (err = %d)\n", err);
+	*err = load_bundled_items(b, cur, doc);
+	if (*err) {
+	    fprintf(stderr, "deserialize bundle: "
+		    "bundle is broken (err = %d)\n", *err);
+	    gretl_bundle_destroy(b);
+	    b = NULL;
+	}
+    }
+
+    return b;
+}
+
+int gretl_bundle_write_to_file (gretl_bundle *b, 
+				const char *fname,
+				int to_dotdir)
+{
+    char fullname[FILENAME_MAX];
+    FILE *fp;
+    int err = 0;
+
+    if (to_dotdir) {
+	build_path(fullname, gretl_dotdir(), fname, NULL);
+    } else {
+	strcpy(fullname, fname);
+	gretl_maybe_switch_dir(fullname);
+    }
+
+    fp = gretl_fopen(fullname, "wb");
+
+    if (fp == NULL) {
+	err = E_FOPEN;
+    } else {
+	gretl_xml_header(fp);
+	gretl_bundle_serialize(b, "temp", fp);
     }
 
     return err;
 }
 
+gretl_bundle *gretl_bundle_read_from_file (const char *fname, 
+					   int from_dotdir,
+					   int *err)
+{
+    xmlDocPtr doc = NULL;
+    xmlNodePtr cur = NULL;
+    char fullname[FILENAME_MAX];
+    gretl_bundle *b;
+
+    b = gretl_bundle_new();
+    if (b == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }    
+
+    if (from_dotdir) {
+	build_path(fullname, gretl_dotdir(), fname, NULL);
+    } else {
+	strcpy(fullname, fname);
+    }
+
+    *err = gretl_xml_open_doc_root(fullname, "gretl-bundle", &doc, &cur);
+
+    if (!*err) {
+	gretl_push_c_numeric_locale();
+	cur = cur->xmlChildrenNode;
+	*err = load_bundled_items(b, cur, doc);
+	gretl_pop_c_numeric_locale();
+	xmlFreeDoc(doc);
+    }
+
+    if (*err) {
+	gretl_bundle_destroy(b);
+	b = NULL;
+    }
+
+    return b;
+}
+
+gretl_bundle *gretl_bundle_read_from_buffer (const char *buf, int len,
+					     int *err)
+{
+    xmlDocPtr doc = NULL;
+    gretl_bundle *b;
+
+    b = gretl_bundle_new();
+    if (b == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }    
+
+    xmlKeepBlanksDefault(0);
+    doc = xmlParseMemory(buf, len);
+
+    if (doc == NULL) {
+	gretl_errmsg_set(_("xmlParseMemory failed"));
+	*err = 1;
+    } else {
+	xmlNodePtr cur = xmlDocGetRootElement(doc);
+
+	if (cur == NULL) {
+	    gretl_errmsg_set(_("xmlDocGetRootElement failed"));
+	    *err = 1;
+	} else {
+	    gretl_push_c_numeric_locale();
+	    cur = cur->xmlChildrenNode;
+	    *err = load_bundled_items(b, cur, doc);
+	    gretl_pop_c_numeric_locale();
+	}
+	xmlFreeDoc(doc);
+    }
+
+    if (*err) {
+	gretl_bundle_destroy(b);
+	b = NULL;
+    }
+
+    return b;
+}
+
+gretl_bundle *get_sysinfo_bundle (int *err)
+{
+    if (sysinfo_bundle == NULL) {
+	gretl_bundle *b = gretl_bundle_new();
+
+	if (b == NULL) {
+	    *err = E_ALLOC;
+	} else {
+	    int ival = 0;
+
+#if HAVE_MPI
+	    ival = check_for_mpiexec();
+#endif	    
+	    gretl_bundle_set_scalar(b, "mpi", (double) ival);
+	    ival = gretl_max_mpi_processes();
+	    gretl_bundle_set_scalar(b, "mpimax", (double) ival);
+	    ival = gretl_n_processors();
+	    gretl_bundle_set_scalar(b, "nproc", (double) ival);
+	    ival = 0;
+#ifdef _OPENMP
+	    ival = 1;
+#endif
+	    gretl_bundle_set_scalar(b, "omp", (double) ival);
+	    ival = sizeof(void*) == 8 ? 64 : 32;
+	    gretl_bundle_set_scalar(b, "wordlen", (double) ival);
+	    gretl_bundle_set_scalar(b, "omp_num_threads", get_omp_n_threads());
+#if defined(G_OS_WIN32)
+	    gretl_bundle_set_string(b, "os", "windows");
+#elif defined(OS_OSX) 
+	    gretl_bundle_set_string(b, "os", "osx");
+#elif defined(linux)
+	    gretl_bundle_set_string(b, "os", "linux");
+#else
+	    gretl_bundle_set_string(b, "os", "other");
+#endif
+	    gretl_bundle_set_string(b, "hostname", g_get_host_name());
+	    gretl_bundle_set_string(b, "blas", blas_variant_string());
+	}
+	sysinfo_bundle = b;
+    }
+
+    return sysinfo_bundle;
+}
+
+void gretl_bundle_cleanup (void)
+{
+    if (sysinfo_bundle != NULL) {
+	gretl_bundle_destroy(sysinfo_bundle);
+	sysinfo_bundle = NULL;
+    }
+}
+
