@@ -22,6 +22,10 @@
 
 #include <stdarg.h>
 
+#ifdef  __cplusplus
+extern "C" {
+#endif
+
 #ifdef NAN
 # define M_NA NAN
 #else
@@ -224,6 +228,11 @@ void gretl_matrix_block_zero (gretl_matrix_block *B);
 
 gretl_matrix_block *gretl_matrix_block_new (gretl_matrix **pm, ...);
 
+int gretl_matrix_block_n_matrices (gretl_matrix_block *B);
+
+gretl_matrix *gretl_matrix_block_get_matrix (gretl_matrix_block *B,
+					     int i);
+
 gretl_matrix *gretl_identity_matrix_new (int n);
 
 gretl_matrix *gretl_DW_matrix_new (int n);
@@ -664,6 +673,7 @@ double gretl_scalar_qform (const gretl_vector *b,
 
 int gretl_matrix_columnwise_product (const gretl_matrix *A,
 				     const gretl_matrix *B,
+				     const gretl_matrix *S,
 				     gretl_matrix *C);
 
 int
@@ -763,4 +773,16 @@ void set_blas_mnk_min (int mnk);
 
 int get_blas_mnk_min (void);
 
+void set_simd_k_max (int k);
+
+int get_simd_k_max (void);
+
+void set_simd_mn_min (int mn);
+
+int get_simd_mn_min (void);
+
+#ifdef  __cplusplus
+}
+#endif
+
 #endif /* GRETL_MATRIX_H */
