@@ -19,12 +19,16 @@
 
 #include "libgretl.h"
 #include "libset.h"
+#include "gretl_func.h"
 #include "gretl_string_table.h"
-#include "gretl_www.h"
 #include "texprint.h"
+
 #ifdef USE_RLIB
 # include "gretl_foreign.h"
 #endif
+#ifdef USE_CURL
+# include "gretl_www.h"
+#endif
 
 #include <unistd.h>
 
@@ -71,6 +75,8 @@ struct INTERNAL_PATHS {
     char octpath[MAXLEN];
     char statapath[MAXLEN];
     char pypath[MAXLEN];
+    char mpiexec[MAXLEN];
+    char mpi_hosts[MAXLEN];
     char dbhost[32];
     char pngfont[128];
     unsigned char status;
@@ -93,7 +99,7 @@ const char *helpfile_path (int id)
     }
 }
 
-static int add_suffix (char *fname, const char *sfx)
+static int maybe_add_suffix (char *fname, const char *sfx)
 {
     if (strrchr(fname, '.') == NULL) {
 	strcat(fname, sfx);
@@ -156,15 +162,35 @@ int get_stdio_use_utf8 (void)
 
 #define FDEBUG 0
 
-/* Try to handle both possible 'cases of need': we should be using 
-   UTF-8 but the path is not in UTF-8, or the path is in UTF-8
-   but should be in locale encoding.
-*/
+/**
+ * maybe_recode_path:
+ * @path: original path.
+ * @pconv: location to receive recoded path.
+ * @want_utf8: should be 1 if a UTF-8 path is wanted
+ * unconditionally; 0 if a path in the (non-UTF-8)
+ * locale is wanted unconditionally, or -1 if the
+ * target codeset should be determined by libgretl's
+ * reckoning of what is needed for use with the C library's
+ * stdio functions that take path arguments.
+ *
+ * Note that if it turns out that @path is already valid,
+ * then nothing is assigned to @pconv; in this case the
+ * function returns 0.
+ *
+ * Returns: 0 on success (either @path did not need to be
+ * recoded, or it was successfully recoded into @pconv), or
+ * a non-zero code on error.
+ */
 
-static int maybe_recode_path (const char *path, int want_utf8, char **pconv)
+int maybe_recode_path (const char *path, char **pconv, int want_utf8)
 {
     int err = 0;
 
+    if (want_utf8 == -1) {
+	/* automatic */
+	want_utf8 = stdio_use_utf8;
+    }
+
 #if FDEBUG
     fprintf(stderr, "maybe_recode_path: want_utf8 = %d\n", want_utf8);
 #endif
@@ -226,7 +252,7 @@ FILE *gretl_fopen (const char *fname, const char *mode)
     fprintf(stderr, "gretl_fopen: got '%s'\n", fname);
 #endif
 
-    err = maybe_recode_path(fname, stdio_use_utf8, &fconv);
+    err = maybe_recode_path(fname, &fconv, -1);
 
     if (!err) {
 	if (fconv != NULL) {
@@ -331,7 +357,7 @@ int gretl_test_fopen (const char *fname, const char *mode)
     int err;
 
     gretl_error_clear();
-    err = maybe_recode_path(fname, stdio_use_utf8, &fconv);
+    err = maybe_recode_path(fname, &fconv, -1);
 
     if (err) {
 	gretl_error_clear();
@@ -392,7 +418,7 @@ FILE *gretl_fopen_with_recode (const char *fname, const char *mode,
     fprintf(stderr, "gretl_fopen_with_recode: got '%s'\n", fname);
 #endif
 
-    err = maybe_recode_path(fname, stdio_use_utf8, &fconv);
+    err = maybe_recode_path(fname, &fconv, -1);
 
     if (!err) {
 	if (fconv != NULL) {
@@ -440,7 +466,7 @@ int gretl_open (const char *pathname, int flags)
 
     gretl_error_clear();
 
-    err = maybe_recode_path(pathname, stdio_use_utf8, &pconv);
+    err = maybe_recode_path(pathname, &pconv, -1);
 
     if (!err) {
 	if (pconv != NULL) {
@@ -477,7 +503,7 @@ int gretl_stat (const char *fname, struct stat *buf)
 
     gretl_error_clear();
 
-    err = maybe_recode_path(fname, stdio_use_utf8, &pconv);
+    err = maybe_recode_path(fname, &pconv, -1);
     
     if (err) {
 	/* emulate 'stat' */
@@ -513,10 +539,10 @@ int gretl_rename (const char *oldpath, const char *newpath)
 
     gretl_error_clear();
 
-    err = maybe_recode_path(oldpath, stdio_use_utf8, &oldconv);
+    err = maybe_recode_path(oldpath, &oldconv, -1);
 
     if (!err) {
-	err = maybe_recode_path(newpath, stdio_use_utf8, &newconv);
+	err = maybe_recode_path(newpath, &newconv, -1);
     }
 
     if (!err) {
@@ -565,7 +591,7 @@ int gretl_remove (const char *path)
     int ret = -1;
     int err;
 
-    err = maybe_recode_path(path, stdio_use_utf8, &pconv);
+    err = maybe_recode_path(path, &pconv, -1);
 
     if (!err) {
 	if (pconv != NULL) {
@@ -580,7 +606,7 @@ int gretl_remove (const char *path)
     /* allow for the possibility that we're trying to remove a
        directory on win32 -> use g_remove */
     if (ret == -1) {
-	err = maybe_recode_path(path, 1, &pconv);
+	err = maybe_recode_path(path, &pconv, 1);
 	if (!err) {
 	    if (pconv != NULL) {
 		ret = g_remove(pconv);
@@ -615,7 +641,7 @@ gzFile gretl_gzopen (const char *fname, const char *mode)
 
     gretl_error_clear();
 
-    err = maybe_recode_path(fname, stdio_use_utf8, &fconv);
+    err = maybe_recode_path(fname, &fconv, -1);
 
     if (!err) {
 	if (fconv != NULL) {
@@ -641,7 +667,7 @@ static gzFile gretl_try_gzopen (const char *fname, const char *mode)
 
     gretl_error_clear();
 
-    err = maybe_recode_path(fname, stdio_use_utf8, &fconv);
+    err = maybe_recode_path(fname, &fconv, -1);
 
     if (!err) {
 	if (fconv != NULL) {
@@ -685,7 +711,7 @@ int gretl_chdir (const char *path)
     }
 #endif
 
-    err = maybe_recode_path(path, stdio_use_utf8, &pconv);
+    err = maybe_recode_path(path, &pconv, -1);
 
     if (!err) {
 	if (pconv != NULL) {
@@ -800,46 +826,33 @@ static const char *gretl_readd (DIR *d)
     return (e == NULL)? NULL : e->d_name;
 }
 
-/* recursive deletion of directory tree: must be located
-   in the directory above the one to be deleted at the
-   outset */
-
-/**
- * gretl_deltree:
- * @path: name of directory to be deleted.
- *
- * Carries out recursive deletion of the specified directory.
- * Note: the current working directory should be set to one
- * level above @path when this function is called. FIXME.
- *
- * Returns: 0 on success, non-zero on error.
- */
-
-int gretl_deltree (const char *path)
+static int real_deltree (const char *path)
 {
-    const char *fname;
     DIR *dir;
     int err = 0;
 
     errno = 0;
-
-    dir = opendir(path);
+    dir = gretl_opendir(path);
 
     if (dir == NULL) {
 	err = 1;
     } else {
+	const char *fname;
+	
 	err = chdir(path);
 	while ((fname = gretl_readd(dir)) != NULL && !err) {
+	    /* recursively delete dir's contents */
 	    if (strcmp(fname, ".") && strcmp(fname, "..")) {
 		if (gretl_isdir(fname)) {
-		    err = gretl_deltree(fname);
+		    err = real_deltree(fname);
 		} else {
-		    err = gretl_remove(fname);
+		    err = remove(fname);
 		}
 	    }
 	}
 	if (!err) {
 	    closedir(dir);
+	    /* delete the directory itself */
 	    if (chdir("..") == 0) {
 		err = gretl_remove(path);
 	    }
@@ -856,6 +869,36 @@ int gretl_deltree (const char *path)
 
 #endif /* WIN32 or not */
 
+/**
+ * gretl_deltree:
+ * @path: name of directory to be deleted.
+ *
+ * Carries out recursive deletion of the specified directory.
+ *
+ * Returns: 0 on success, non-zero on error.
+ */
+
+int gretl_deltree (const char *path)
+{
+#ifdef WIN32
+    return win32_delete_dir(path);
+#else
+    char tmp[FILENAME_MAX];
+    char *savedir = NULL;
+    int err;
+
+    savedir = getcwd(tmp, FILENAME_MAX - 1);
+
+    err = real_deltree(path);
+
+    if (savedir != NULL) {
+	gretl_chdir(savedir);
+    }
+
+    return err;
+#endif
+}
+
 DIR *gretl_opendir (const char *name)
 {
 #ifdef WIN32
@@ -931,7 +974,7 @@ int gretl_write_access (char *fname)
 
     gretl_error_clear();
 
-    err = maybe_recode_path(fname, stdio_use_utf8, &fconv);
+    err = maybe_recode_path(fname, &fconv, -1);
     if (err) {
 	return err;
     }
@@ -1022,10 +1065,16 @@ int gretl_path_prepend (char *file, const char *path)
     return 0;
 }
 
+enum {
+    ADD_GDT = 1 << 0,
+    ADD_GFN = 1 << 1,
+    SUBDIRS = 1 << 2
+};
+
 #ifdef WIN32
 
 static int try_open_file (char *targ, const char *finddir, 
-			  WIN32_FIND_DATA *fdata, int code)
+			  WIN32_FIND_DATA *fdata, int flags)
 {
     char tmp[MAXLEN];
     int n = strlen(finddir);
@@ -1038,9 +1087,15 @@ static int try_open_file (char *targ, const char *finddir,
     strcat(tmp, targ);
 
     err = gretl_test_fopen(tmp, "r");
-    if (err && code == DATA_SEARCH) {
-	if (add_suffix(tmp, ".gdt")) {
-	    err = gretl_test_fopen(tmp, "r");
+
+    if (err && (flags & ADD_GDT)) {
+	if (maybe_add_suffix(tmp, ".gdt")) {
+	    err = gretl_test_fopen(tmp, "rb");
+	    if (err) {
+		/* try .gdtb also */
+		strncat(tmp, "b", 1);
+		err = gretl_test_fopen(tmp, "rb");
+	    }
 	}
     }
 
@@ -1088,7 +1143,7 @@ static int got_subdir (WIN32_FIND_DATA *fdata)
     return ret;
 }
 
-static int find_in_subdir (const char *topdir, char *fname, int code)
+static int find_in_subdir (const char *topdir, char *fname, int flags)
 {
     HANDLE handle;
     WIN32_FIND_DATA fdata;
@@ -1101,11 +1156,11 @@ static int find_in_subdir (const char *topdir, char *fname, int code)
     handle = FindFirstFile(finddir, &fdata); 
     if (handle != INVALID_HANDLE_VALUE) {
 	if (got_subdir(&fdata)) {
-	    found = try_open_file(fname, finddir, &fdata, code);
+	    found = try_open_file(fname, finddir, &fdata, flags);
 	} 
 	while (!found && FindNextFile(handle, &fdata)) {
 	    if (got_subdir(&fdata)) {
-		found = try_open_file(fname, finddir, &fdata, code);
+		found = try_open_file(fname, finddir, &fdata, flags);
 	    }
 	} 
 	FindClose(handle);
@@ -1117,7 +1172,7 @@ static int find_in_subdir (const char *topdir, char *fname, int code)
 #else /* end of win32 file-finding, on to posix */
 
 static int try_open_file (char *targ, const char *finddir, 
-			  struct dirent *dirent, int code)
+			  struct dirent *dirent, int flags)
 {
     char tmp[MAXLEN];
     int err, found = 0;
@@ -1128,9 +1183,15 @@ static int try_open_file (char *targ, const char *finddir,
     strcat(tmp, targ);
 
     err = gretl_test_fopen(tmp, "r");
-    if (err && code == DATA_SEARCH) {
-	if (add_suffix(tmp, ".gdt")) {
+
+    if (err && (flags & ADD_GDT)) {
+	if (maybe_add_suffix(tmp, ".gdt")) {
 	    err = gretl_test_fopen(tmp, "r");
+	    if (err) {
+		/* try .gdtb also */
+		strncat(tmp, "b", 1);
+		err = gretl_test_fopen(tmp, "r");
+	    }
 	}
     }
 
@@ -1173,7 +1234,7 @@ static int got_subdir (const char *topdir, struct dirent *dirent)
     return ret;
 }
 
-static int find_in_subdir (const char *topdir, char *fname, int code)
+static int find_in_subdir (const char *topdir, char *fname, int flags)
 {
     DIR *dir;
     struct dirent *dirent;
@@ -1187,7 +1248,7 @@ static int find_in_subdir (const char *topdir, char *fname, int code)
     if (dir != NULL) {
 	while (!found && (dirent = readdir(dir))) {
 	    if (got_subdir(finddir, dirent)) {
-		found = try_open_file(fname, finddir, dirent, code);
+		found = try_open_file(fname, finddir, dirent, flags);
 	    }
 	}
 	closedir(dir);
@@ -1200,7 +1261,7 @@ static int find_in_subdir (const char *topdir, char *fname, int code)
 
 #define SEARCH_DEBUG 0
 
-static char *search_dir (char *fname, const char *topdir, int code)
+char *search_dir (char *fname, const char *topdir, int flags)
 {
     char orig[MAXLEN];
     int err;
@@ -1217,20 +1278,31 @@ static char *search_dir (char *fname, const char *topdir, int code)
 	if (!err) {
 	    return fname;
 	}
-	if (code == DATA_SEARCH && add_suffix(fname, ".gdt")) {
-	    err = gretl_test_fopen(fname, "r");
-	    if (!err) {
-		return fname;
-	    }
-	} else if (code == FUNCS_SEARCH && add_suffix(fname, ".gfn")) {
-	    err = gretl_test_fopen(fname, "r");
-	    if (!err) {
-		return fname;
+	if (flags & ADD_GDT) {
+	    if (maybe_add_suffix(fname, ".gdt")) {
+		err = gretl_test_fopen(fname, "r");
+		if (!err) {
+		    return fname;
+		}
+	    } else if (maybe_add_suffix(fname, ".gdtb")) {
+		err = gretl_test_fopen(fname, "r");
+		if (!err) {
+		    return fname;
+		}
+	    }		
+	} else if (flags & ADD_GFN) {
+	    if (maybe_add_suffix(fname, ".gfn")) {
+		err = gretl_test_fopen(fname, "r");
+		if (!err) {
+		    return fname;
+		}
 	    }
 	}	    
 	strcpy(fname, orig);
-	if (code != CURRENT_DIR && find_in_subdir(topdir, fname, code)) {
-	    return fname;
+	if (flags & SUBDIRS) {
+	    if (find_in_subdir(topdir, fname, flags)) {
+		return fname;
+	    }
 	}
     }
 
@@ -1329,9 +1401,9 @@ static int shelldir_open_dotfile (char *fname, char *orig)
 
 /**
  * get_plausible_search_dirs:
- * @type: should be DATA_SEARCH for data file packages,
- * DB_SEARCH for gretl databases, or FUNCS_SEARCH for
- * function packages.
+ * @stype: DATA_SEARCH for data file packages, DB_SEARCH for
+ * gretl databases, FUNCS_SEARCH for function packages, or
+ * SCRIPT_SEARCH for hansl scripts.
  * @n_dirs: location to receive the number of directories.
  * 
  * Returns: an array of plausible search paths, depending on the 
@@ -1339,7 +1411,7 @@ static int shelldir_open_dotfile (char *fname, char *orig)
  * with it, using strings_array_free().
  */
 
-char **get_plausible_search_dirs (int type, int *n_dirs)
+char **get_plausible_search_dirs (SearchType stype, int *n_dirs)
 {
     char **dirs = NULL;
     const char *subdir;
@@ -1348,13 +1420,13 @@ char **get_plausible_search_dirs (int type, int *n_dirs)
 
     *n_dirs = 0;
 
-    if (type == DATA_SEARCH) {
+    if (stype == DATA_SEARCH) {
 	subdir = "data";
-    } else if (type == DB_SEARCH) {
+    } else if (stype == DB_SEARCH) {
 	subdir = "db";
-    } else if (type == FUNCS_SEARCH) {
+    } else if (stype == FUNCS_SEARCH) {
 	subdir = "functions";
-    } else if (type == SCRIPT_SEARCH) {
+    } else if (stype == SCRIPT_SEARCH) {
 	subdir = "scripts";
     } else {
 	fprintf(stderr, "get_plausible_search_dir: no type specified\n");
@@ -1397,7 +1469,7 @@ char **get_plausible_search_dirs (int type, int *n_dirs)
 	if (wd != NULL) {
 	    build_path(dirname, wd, subdir, NULL);
 	    err = strings_array_add(&dirs, n_dirs, dirname);
-	    if (!err && type != FUNCS_SEARCH) {
+	    if (!err && stype != FUNCS_SEARCH) {
 		strcpy(dirname, wd);
 		err = strings_array_add(&dirs, n_dirs, dirname);
 	    }
@@ -1412,6 +1484,66 @@ char **get_plausible_search_dirs (int type, int *n_dirs)
 # define NAME_MAX 255
 #endif
 
+static char *gretl_addon_get_path (const char *name)
+{
+    char *ret = NULL;
+    char path[FILENAME_MAX];
+    double version, maxver = 0;
+    char **dirs;
+    int i, n_dirs, err;
+
+    *path = '\0';
+
+    dirs = get_plausible_search_dirs(FUNCS_SEARCH, &n_dirs);
+
+    for (i=0; i<n_dirs; i++) {
+	const char *fndir = dirs[i];
+	struct dirent *dirent;
+	const char *dname;
+	DIR *dir;
+	int found = 0;
+	
+	if ((dir = gretl_opendir(fndir)) == NULL) {
+	    continue;
+	}
+
+	while ((dirent = readdir(dir)) != NULL && !found) {
+	    dname = dirent->d_name;
+	    if (!strcmp(dname, name)) {
+		sprintf(path, "%s%c%s%c%s.gfn", fndir, SLASH, 
+			dname, SLASH, dname);
+		err = gretl_test_fopen(path, "r");
+		if (!err) {
+		    version = function_package_get_version(path);
+		    if (!na(version) && version > maxver) {
+			maxver = version;
+			free(ret);
+			ret = gretl_strdup(path);
+		    }
+		}
+		found = 1;
+	    }
+	}
+
+	closedir(dir);
+    }
+
+    strings_array_free(dirs, n_dirs);
+
+    return ret;
+}
+
+static int is_addon (const char *name)
+{
+    if (!strcmp(name, "gig") ||
+	!strcmp(name, "SVAR") ||
+	!strcmp(name, "ivpanel")) {
+	return 1;
+    } else {
+	return 0;
+    }
+}
+
 /**
  * gretl_function_package_get_path:
  * @name: the name of the package to find, without the .gfn extension.
@@ -1437,6 +1569,14 @@ char *gretl_function_package_get_path (const char *name,
     int err, found = 0;
     int i, n_dirs;
 
+    if (type == PKG_ALL && is_addon(name)) {
+	type = PKG_SUBDIR;
+    }
+
+    if (type == PKG_SUBDIR) {
+	return gretl_addon_get_path(name);
+    }
+
     *path = '\0';
 
     dirs = get_plausible_search_dirs(FUNCS_SEARCH, &n_dirs);
@@ -1502,11 +1642,14 @@ char *gretl_function_package_get_path (const char *name,
 
 /**
  * gretl_addpath:
- * @fname: initially given file name.
- * @script: if non-zero, suppose the file is a command script.
+ * @fname: the initially given file name.
+ * @script: if non-zero, suppose the file we're looking for
+ * is a hansl script.
  * 
  * Elementary path-searching: try adding various paths to the given
- * @fname and see if it can be opened.  Usually called by getopenfile().
+ * @fname and see if it can be opened. Usually called by getopenfile().
+ * If @fname does not have a dot-extension we may also try adding
+ * an appropriate gretl extension in case no file is found.
  *
  * Returns: the full name of the file that was found, or NULL if no
  * file could be found.
@@ -1516,6 +1659,7 @@ char *gretl_addpath (char *fname, int script)
 {
     char orig[MAXLEN];
     char *tmp = fname;
+    char *test;
     int err;
 
 #if SEARCH_DEBUG
@@ -1539,7 +1683,13 @@ char *gretl_addpath (char *fname, int script)
 	    make_path_absolute(fname, orig);
 	}
 	return fname;
-    } else if (g_path_is_absolute(fname)) {  
+    } else if (g_path_is_absolute(fname)) {
+	if (!script && maybe_add_suffix(fname, ".gdt")) {
+	    err = gretl_test_fopen(fname, "r");
+	    if (!err) {
+		return fname;
+	    }
+	}
 	/* unable to open file: if the path was absolute, fail */
 	return NULL;
     } else {
@@ -1548,7 +1698,10 @@ char *gretl_addpath (char *fname, int script)
 
 	if (*gpath != '\0') {
 	    /* try looking where the last-opened script was found */
-	    if (search_dir(fname, gpath, CURRENT_DIR) != NULL) {
+	    int flags = script ? 0 : ADD_GDT;
+
+	    test = search_dir(fname, gpath, flags);
+	    if (test != NULL) {
 		return fname;
 	    }
 	}
@@ -1561,20 +1714,23 @@ char *gretl_addpath (char *fname, int script)
 	    /* try searching some standard gretl paths */
 	    if (script) {
 		sprintf(trydir, "%sscripts", gpath);
-		if (search_dir(fname, trydir, SCRIPT_SEARCH) != NULL) { 
+		test = search_dir(fname, trydir, SUBDIRS);
+		if (test != NULL) { 
 		    return fname;
 		} else {
 		    fname = tmp;
 		    strcpy(fname, orig);
 		    sprintf(trydir, "%sfunctions", gpath);
-		    if (search_dir(fname, trydir, FUNCS_SEARCH) != NULL) { 
+		    test = search_dir(fname, trydir, ADD_GFN | SUBDIRS);
+		    if (test != NULL) {
 			return fname;
 		    }
 		}
 	    } else {
 		/* data file */
 		sprintf(trydir, "%sdata", gpath);
-		if (search_dir(fname, trydir, DATA_SEARCH) != NULL) { 
+		test = search_dir(fname, trydir, ADD_GDT | SUBDIRS);
+		if (test != NULL) { 
 		    return fname;
 		}
 	    } 
@@ -1592,20 +1748,23 @@ char *gretl_addpath (char *fname, int script)
 	    /* try looking in ~/Library or dotdir */
 	    if (script) {
 		sprintf(trydir, "%sscripts", gpath);
-		if (search_dir(fname, trydir, SCRIPT_SEARCH) != NULL) { 
+		test = search_dir(fname, trydir, SUBDIRS);
+		if (test != NULL) { 
 		    return fname;
 		} else {
 		    fname = tmp;
 		    strcpy(fname, orig);
 		    sprintf(trydir, "%sfunctions", gpath);
-		    if (search_dir(fname, trydir, FUNCS_SEARCH) != NULL) { 
+		    test = search_dir(fname, trydir, ADD_GFN | SUBDIRS);
+		    if (test != NULL) { 
 			return fname;
 		    }
 		}
 	    } else {
 		/* data file */
 		sprintf(trydir, "%sdata", gpath);
-		if (search_dir(fname, trydir, DATA_SEARCH) != NULL) { 
+		test = search_dir(fname, trydir, ADD_GDT | SUBDIRS);
+		if (test != NULL) { 
 		    return fname;
 		}
 	    } 
@@ -1617,7 +1776,8 @@ char *gretl_addpath (char *fname, int script)
 
 	if (*gpath != '\0') {
 	    /* try looking in user's dir (and subdirs) */
-	    if (search_dir(fname, gpath, USER_SEARCH) != NULL) { 
+	    test = search_dir(fname, gpath, SUBDIRS);
+	    if (test != NULL) { 
 		return fname;
 	    }
 	}
@@ -1628,7 +1788,8 @@ char *gretl_addpath (char *fname, int script)
 
 	if (gpath != NULL && *gpath != '\0') {
 	    /* try looking in default workdir? */
-	    if (search_dir(fname, gpath, USER_SEARCH) != NULL) { 
+	    test = search_dir(fname, gpath, SUBDIRS);
+	    if (test != NULL) { 
 		return fname;
 	    }
 	}
@@ -1639,7 +1800,8 @@ char *gretl_addpath (char *fname, int script)
 
 	if (gpath != NULL && *gpath != '\0') {
 	    /* try looking in shelldir? */
-	    if (search_dir(fname, gpath, CURRENT_DIR) != NULL) { 
+	    test = search_dir(fname, gpath, 0);
+	    if (test != NULL) { 
 		return fname;
 	    }
 	}	
@@ -1649,17 +1811,16 @@ char *gretl_addpath (char *fname, int script)
     /* try looking on the desktop? */
     if (1) {
 	char *dtdir = desktop_path();
-	char *ret = NULL;
 
 	fname = tmp;
 	strcpy(fname, orig);
 
 	if (dtdir != NULL) {
-	    ret = search_dir(fname, dtdir, CURRENT_DIR);
+	    test = search_dir(fname, dtdir, 0);
 	    free(dtdir);
 	}
-	if (ret != NULL) {
-	    return ret;
+	if (test != NULL) {
+	    return fname;
 	}
     }	    
 #endif
@@ -1729,7 +1890,6 @@ static int get_gfn_special (char *fname)
 	p = strstr(pkgname, ".gfn");
 	*p = '\0';
 	pkgpath = gretl_function_package_get_path(pkgname, PKG_ALL);
-
 	if (pkgpath != NULL) {
 	    strcpy(fname, pkgpath);
 	    free(pkgpath);
@@ -1758,7 +1918,7 @@ int getopenfile (const char *line, char *fname, gretlopt opt)
 {
     int script = (opt & (OPT_S | OPT_I))? 1 : 0;
     int got_quoted;
-    char *fullname;
+    char *test;
 
     /* skip past command word */
     line += strcspn(line, " ");
@@ -1789,11 +1949,14 @@ int getopenfile (const char *line, char *fname, gretlopt opt)
 
     /* try a basic path search on this filename */
 #if SEARCH_DEBUG
-    fprintf(stderr, "getopenfile: calling addpath\n");
+    fprintf(stderr, "getopenfile: calling addpath on '%s'\n", fname);
+#endif
+    test = gretl_addpath(fname, script);
+#if SEARCH_DEBUG
+    fprintf(stderr, "getopenfile: after: '%s'\n", fname);
 #endif
-    fullname = gretl_addpath(fname, script);
 
-    if (fullname != NULL && (opt & OPT_S)) {
+    if (test != NULL && (opt & OPT_S)) {
 	int spos = gretl_slashpos(fname);
 
 	if (spos) {
@@ -1809,19 +1972,21 @@ int getopenfile (const char *line, char *fname, gretlopt opt)
     return 0;
 }
 
-int has_system_prefix (const char *fname, int locus)
+int has_system_prefix (const char *fname, SearchType stype)
 {
     const char *gretldir = gretl_home();
     int n = strlen(gretldir);
     int ret = 0;
 
-    if (strlen(fname) < n) return 0;
+    if (strlen(fname) < n) {
+	return 0;
+    }
     
     if (!strncmp(fname, gretldir, n)) {
-	if (locus == DATA_SEARCH &&
+	if (stype == DATA_SEARCH &&
 	    !strncmp(fname + n, "data", 4)) {
 	    ret = 1;
-	} else if (locus == SCRIPT_SEARCH &&
+	} else if (stype == SCRIPT_SEARCH &&
 		   !strncmp(fname + n, "scripts", 7)) { 
 	    ret = 1;
 	}
@@ -1940,8 +2105,6 @@ static void set_builtin_path_strings (int update)
     gretl_insert_builtin_string("gnuplot",  paths.gnuplot);
     gretl_insert_builtin_string("x12a",     paths.x12a);
     gretl_insert_builtin_string("tramo",    paths.tramo);
-    gretl_insert_builtin_string("Rbin",     paths.rbinpath);
-    gretl_insert_builtin_string("Rlib",     paths.rlibpath);
 
     if (!update) {
 	/* these only have to be done once */
@@ -2250,6 +2413,11 @@ const char *gretl_x12_arima_dir (void)
     return paths.x12adir;
 }
 
+int gretl_x12_is_x13 (void)
+{
+    return strstr(paths.x12a, "x13") != NULL;
+}
+
 const char *gretl_rbin_path (void)
 {
 #ifdef WIN32
@@ -2298,6 +2466,16 @@ const char *gretl_python_path (void)
     return paths.pypath;
 }
 
+const char *gretl_mpi_hosts (void)
+{
+    return paths.mpi_hosts;
+}
+
+const char *gretl_mpiexec (void)
+{
+    return paths.mpiexec;
+}
+
 const char *gretl_current_dir (void)
 {
     return current_dir;
@@ -2368,16 +2546,11 @@ static char *rightmost (char *s1, char *s2)
 
 void win32_set_gretldir (const char *progname)
 {
-    int err;
+    int done = 0;
 
     *paths.gretldir = '\0';
 
-    /* we'll try the registry first */
-    err = read_reg_val(HKEY_LOCAL_MACHINE, "gretl", "gretldir", paths.gretldir);
-    if (!err) {
-	slash_terminate(paths.gretldir);
-	return;
-    }
+    /* try using @progname (= argv[0] when gretl was started) */
 
     if (g_path_is_absolute(progname)) {
 	strncat(paths.gretldir, progname, MAXLEN - 1);
@@ -2405,8 +2578,21 @@ void win32_set_gretldir (const char *progname)
 
 	if (s != NULL) {
 	    *(s+1) = '\0';
+	    done = 1;
 	}
-    }	
+    }
+
+    if (!done) {
+	/* try the registry */
+	char tmp[MAXLEN];
+	int err;
+
+	err = read_reg_val(HKEY_LOCAL_MACHINE, "gretl", "gretldir", tmp);
+	if (!err) {
+	    strcpy(paths.gretldir, tmp);
+	    slash_terminate(paths.gretldir);
+	}
+    }
 }
 
 #else /* !WIN32 */
@@ -2635,19 +2821,29 @@ static int initialize_dotdir (void)
     return err;
 }
 
-/* when updating: transcribe the new value unless it is empty or is
-   unchanged; if it's a directory string that needs to be
-   slash-terminated, check that; return 1 if any change was made
-   to the internally recorded value, @targ, otherwise return 0.
+enum {
+    PATH_NEEDS_SLASH = 1 << 0,
+    PATH_BLANK_OK    = 1 << 1
+};
+
+/* Updating a gretl paths element: transcribe the new value unless it
+   is unchanged; if it's a directory string that needs to be
+   slash-terminated, check that; return 1 if any change was made to
+   the internally recorded value, @targ, otherwise return 0.  Note
+   that we ignore empty @src unless the PATH_BLANK_OK flag is given.
 */
 
-static int maybe_transcribe_path (char *targ, char *src,
-				  int needs_slash)
+static int maybe_transcribe_path (char *targ, char *src, int flags)
 {
     int ret = 0;
 
-    if (*src != '\0') {
-	if (needs_slash) {
+    if (*src == '\0' && (flags & PATH_BLANK_OK)) {
+	if (*targ != '\0') {
+	    *targ = '\0';
+	    ret = 1;
+	}
+    } else if (*src != '\0') {
+	if (flags & PATH_NEEDS_SLASH) {
 	    slash_terminate(src);
 	}
 	if (strcmp(src, targ)) {
@@ -2677,20 +2873,25 @@ static int maybe_transcribe_path (char *targ, char *src,
 
    * paths.pngfont is updated separately via the plot editing
      dialog
+
+   The @opt argument can include OPT_N to force use of the English-
+   language help file where this would not be the default.
 */
 
 int gretl_update_paths (ConfigPaths *cpaths, gretlopt opt)
 {
     int ndelta = 0;
 
-    if (maybe_transcribe_path(paths.gretldir, cpaths->gretldir, 1)) {
+    if (maybe_transcribe_path(paths.gretldir, cpaths->gretldir, 
+			      PATH_NEEDS_SLASH)) {
 	set_helpfile_paths(opt);
 	set_gretl_libpath(paths.gretldir);
 	ndelta++;
     }
     
     /* native databases */
-    maybe_transcribe_path(paths.dbhost, cpaths->dbhost, 0);
+    maybe_transcribe_path(paths.dbhost, cpaths->dbhost, 
+			  PATH_BLANK_OK);
 
 #ifndef WIN32
     /* gnuplot path: this is set immutably at start-up on Windows */
@@ -2706,6 +2907,12 @@ int gretl_update_paths (ConfigPaths *cpaths, gretlopt opt)
     ndelta += maybe_transcribe_path(paths.statapath, cpaths->statapath, 0);
     ndelta += maybe_transcribe_path(paths.pypath, cpaths->pypath, 0);
 
+#ifdef HAVE_MPI
+    ndelta += maybe_transcribe_path(paths.mpiexec, cpaths->mpiexec, 0);
+    ndelta += maybe_transcribe_path(paths.mpi_hosts, cpaths->mpi_hosts, 
+				    PATH_BLANK_OK);
+#endif
+
 #ifdef USE_RLIB
     if (maybe_transcribe_path(paths.rlibpath, cpaths->rlibpath, 0)) {
 	gretl_R_reset_error();
@@ -2754,7 +2961,7 @@ static void load_default_path (char *targ)
     } else if (targ == paths.dbhost) {
 	strcpy(targ, "ricardo.ecn.wfu.edu");
     } else if (targ == paths.x12a) {
-	sprintf(targ, "%s\\x12arima\\x12a.exe", progfiles);
+	sprintf(targ, "%s\\x13as\\x13as.exe", progfiles);	
     } else if (targ == paths.tramo) {
 	sprintf(targ, "%s\\tramo\\tramo.exe", progfiles);
     } else if (targ == paths.rbinpath) {
@@ -2764,11 +2971,15 @@ static void load_default_path (char *targ)
     } else if (targ == paths.oxlpath) {
 	sprintf(targ, "%s\\OxMetrics6\\Ox\\bin\\oxl.exe", progfiles);
     } else if (targ == paths.octpath) {
-	strcpy(targ, "C:\\Octave\\3.2.3_gcc-4.0.0\\bin\\octave.exe");
+	strcpy(targ, "C:\\Octave-3.6.4\\bin\\octave.exe");
     } else if (targ == paths.statapath) {
 	sprintf(targ, "%s\\Stata\\stata.exe", progfiles);
     } else if (targ == paths.pypath) {
 	strcpy(targ, "python.exe"); /* ?? */
+    } else if (targ == paths.mpiexec) {
+	strcpy(targ, "mpiexec.exe");
+    } else if (targ == paths.mpi_hosts) {
+	*targ = '\0';
     } else if (targ == paths.pngfont) {
 	strcpy(targ, "verdana 8");
     }
@@ -2776,6 +2987,8 @@ static void load_default_path (char *targ)
     free(progfiles);
 }
 
+# if CFG_DEBUG
+
 static void show_paths_on_stderr (void)
 {
     fprintf(stderr, "after gretl_set_paths:\n");
@@ -2785,6 +2998,8 @@ static void show_paths_on_stderr (void)
     fprintf(stderr, " gnuplot = '%s'\n", paths.gnuplot);
 }
 
+# endif
+
 #else /* !WIN32 */
 
 /* unix-type variants of defaults for any paths that we need 
@@ -2798,7 +3013,12 @@ static void load_default_workdir (char *targ)
     if (home != NULL) {
 	sprintf(targ, "%s/gretl/", home);
     } else {
-	sprintf(targ, "%suser/", paths.gretldir);
+	home = getenv("GRETL_WORKDIR");
+	if (home != NULL) {
+	    strcpy(targ, home);
+	} else {
+	    sprintf(targ, "%suser/", paths.gretldir);
+	}
     }
 }
 
@@ -2808,15 +3028,13 @@ static void load_default_path (char *targ)
     const char *app_paths[] = {
 	"/Applications/OxMetrics6/ox/bin/oxl",
 	"/Applications/Octave.app/Contents/Resources/bin/octave",
-	"/Applications/Stata/Stata.app/Contents/MacOS/Stata",
-	"python"
+	"/Applications/Stata/Stata.app/Contents/MacOS/Stata"
     };
 #else
     const char *app_paths[] = {
 	"oxl",
 	"octave",
-	"stata",
-	"python"
+	"stata"
     };
 #endif  
 
@@ -2853,7 +3071,11 @@ static void load_default_path (char *targ)
     } else if (targ == paths.statapath) {
 	strcpy(paths.statapath, app_paths[2]);
     } else if (targ == paths.pypath) {
-	strcpy(paths.pypath, app_paths[3]);
+	strcpy(paths.pypath, "python");
+    } else if (targ == paths.mpiexec) {
+	strcpy(paths.mpiexec, "mpiexec");
+    } else if (targ == paths.mpi_hosts) {
+	*paths.mpi_hosts = '\0';
     } else if (targ == paths.pngfont) {
 #if defined(MAC_NATIVE)
 	strcpy(targ, "Sans 13");
@@ -2924,18 +3146,25 @@ static void copy_paths_with_fallback (ConfigPaths *cpaths)
     path_init(paths.octpath, cpaths->octpath, 0);
     path_init(paths.statapath, cpaths->statapath, 0);
     path_init(paths.pypath, cpaths->pypath, 0);
+    path_init(paths.mpiexec, cpaths->mpiexec, 0);
+    path_init(paths.mpi_hosts, cpaths->mpi_hosts, 0);
 
     /* graphing font */
     path_init(paths.pngfont, cpaths->pngfont, 0);
 }
 
-/* This is called after reading the gretl config file (or reading from
-   the registry on Windows) at startup (and only then).  Subsequent
-   updates to paths via the GUI (if any) are handled by the function
-   gretl_update_paths().
+/* This is called after reading the gretl config file at startup 
+   (and only then).  Subsequent updates to paths via the GUI (if any) 
+   are handled by the function gretl_update_paths().
+
+   The no_dotdir member of cpaths is used only when gretlcli is
+   operating in "slave" mode (e.g. under a webserver). It forces gretl
+   to use paths.workdir as the "dotdir" rather than using a directory
+   under the executing user's HOME.  See
+   http://gretl.sourceforge.net/slave/
 */
 
-int gretl_set_paths (ConfigPaths *cpaths, gretlopt opt)
+int gretl_set_paths (ConfigPaths *cpaths)
 {
     int err0 = 0, err1 = 0;
     int retval = 0;
@@ -2944,18 +3173,17 @@ int gretl_set_paths (ConfigPaths *cpaths, gretlopt opt)
     *paths.workdir = '\0';
     *paths.plotfile = '\0';
 
-    initialize_gretldir(cpaths->gretldir, opt);
+    initialize_gretldir(cpaths->gretldir, OPT_NONE);
 
-    if (!(opt & OPT_D)) {
-	/* OPT_D says dotdir = workdir */
+    if (!cpaths->no_dotdir) {
 	err0 = initialize_dotdir();
     }
 
     copy_paths_with_fallback(cpaths);
 
-    if (opt & OPT_D) {
+    if (cpaths->no_dotdir) {
 	strcpy(paths.dotdir, paths.workdir);
-    }
+    }    
 
     if (strcmp(paths.dotdir, paths.workdir)) { 
 	err1 = validate_writedir(paths.workdir);
@@ -2987,10 +3215,9 @@ int gretl_set_paths (ConfigPaths *cpaths, gretlopt opt)
 
 #if CFG_DEBUG
     fprintf(stderr, "gretl_set_paths: returning %d\n", retval);
-#endif
-
-#ifdef WIN32
+# ifdef WIN32
     show_paths_on_stderr();
+# endif
 #endif
 
     return retval;
@@ -3209,8 +3436,6 @@ int slash_terminate (char *path)
     return 0;
 }
 
-#ifndef WIN32
-
 static void rc_set_gp_colors (const char *gpcolors)
 {
     char cstr[N_GP_COLORS][8];
@@ -3235,6 +3460,108 @@ static int rc_bool (const char *s)
     }	
 }
 
+static void handle_use_cwd (int use_cwd, ConfigPaths *cpaths)
+{
+    libset_set_bool(USE_CWD, use_cwd);
+
+    if (use_cwd) {
+	char *s, cwd[MAXLEN];
+
+	s = getcwd(cwd, MAXLEN);
+	if (s != NULL) {
+	    *cpaths->workdir = '\0';
+	    strncat(cpaths->workdir, s, MAXLEN - 2);
+	    slash_terminate(cpaths->workdir);
+	}
+    }
+}
+
+#define PROXLEN 64
+
+void get_gretl_config_from_file (FILE *fp, ConfigPaths *cpaths,
+				 char *dbproxy, int *use_proxy)
+{
+    char line[MAXLEN], key[32], val[MAXLEN];
+
+    while (fgets(line, sizeof line, fp) != NULL) {
+	if (*line == '#') {
+	    continue;
+	}
+	if (!strncmp(line, "recent", 6)) {
+	    /* reached the "recent files" section */
+	    break;
+	}
+	if (sscanf(line, "%s", key) != 1) {
+	    continue;
+	}
+	*val = '\0';
+	/* get the string that follows " = " */ 
+	strncat(val, line + strlen(key) + 3, MAXLEN - 1);
+	gretl_strstrip(val); 
+	if (!strcmp(key, "gretldir")) {
+	    strncat(cpaths->gretldir, val, MAXLEN - 1);
+#ifndef WIN32
+	} else if (!strcmp(key, "gnuplot")) {
+	    strncat(cpaths->gnuplot, val, MAXLEN - 1);
+#endif
+	} else if (!strcmp(key, "userdir")) {
+	    strncat(cpaths->workdir, val, MAXLEN - 1);
+	} else if (!strcmp(key, "no_dotdir")) {
+	    cpaths->no_dotdir = rc_bool(val);
+	} else if (!strcmp(key, "shellok")) {
+	    libset_set_bool(SHELL_OK, rc_bool(val));
+	} else if (!strcmp(key, "usecwd")) {
+	    handle_use_cwd(rc_bool(val), cpaths);
+	} else if (!strcmp(key, "dbhost")) {
+	    strncat(cpaths->dbhost, val, 32 - 1);
+	} else if (!strcmp(key, "dbproxy")) {
+	    strncat(dbproxy, val, PROXLEN - 1);
+	} else if (!strcmp(key, "useproxy")) {
+	    *use_proxy = rc_bool(val);
+	} else if (!strcmp(key, "x12a")) {
+	    strncat(cpaths->x12a, val, MAXLEN - 1);
+	} else if (!strcmp(key, "tramo")) {
+	    strncat(cpaths->tramo, val, MAXLEN - 1);
+	} else if (!strcmp(key, "Rbin")) {
+	    strncat(cpaths->rbinpath, val, MAXLEN - 1);
+	} else if (!strcmp(key, "Rlib")) {
+	    strncat(cpaths->rlibpath, val, MAXLEN - 1);
+	} else if (!strcmp(key, "ox")) {
+	    strncat(cpaths->oxlpath, val, MAXLEN - 1);
+	} else if (!strcmp(key, "octave")) {
+	    strncat(cpaths->octpath, val, MAXLEN - 1);
+	} else if (!strcmp(key, "stata")) {
+	    strncat(cpaths->statapath, val, MAXLEN - 1);
+	} else if (!strcmp(key, "python")) {
+	    strncat(cpaths->pypath, val, MAXLEN - 1);
+	} else if (!strcmp(key, "mpiexec")) {
+	    strncat(cpaths->mpiexec, val, MAXLEN - 1);
+	} else if (!strcmp(key, "mpi_hosts")) {
+	    strncat(cpaths->mpi_hosts, val, MAXLEN - 1);
+	} else if (!strcmp(key, "mpi_pref")) {
+#ifdef HAVE_MPI
+	    set_mpi_variant(val);
+#else
+	    ;
+#endif
+	} else if (!strcmp(key, "Png_font")) {
+	    strncat(cpaths->pngfont, val, 128 - 1);
+	} else if (!strcmp(key, "Gp_colors")) {
+	    rc_set_gp_colors(val);
+	} else if (!strcmp(key, "HC_xsect")) {
+	    set_xsect_hccme(val);
+	} else if (!strcmp(key, "HC_tseri")) {
+	    set_tseries_hccme(val);
+	} else if (!strcmp(key, "HC_panel")) {
+	    set_panel_hccme(val);
+	} else if (!strcmp(key, "HC_garch")) {
+	    set_garch_robust_vcv(val);
+	}
+    }
+}
+
+#ifndef WIN32
+
 void get_gretl_rc_path (char *rcfile)
 {
     char *path = getenv("GRETL_CONFIG_FILE");
@@ -3255,8 +3582,6 @@ void get_gretl_rc_path (char *rcfile)
     }
 }
 
-#define PROXLEN 64
-
 /* non-Windows read of the gretl configuration file on behalf
    of the CLI program, gretlcli
 */
@@ -3264,111 +3589,31 @@ void get_gretl_rc_path (char *rcfile)
 int cli_read_rc (void) 
 {
     ConfigPaths cpaths = {0};
-    FILE *fp = NULL;
     char rcfile[FILENAME_MAX];
-    char line[MAXLEN], key[32], val[MAXLEN];
     char dbproxy[PROXLEN] = {0};
-    gretlopt paths_opt = OPT_NONE;
-    int usecwd = 0;
     int use_proxy = 0;
+    FILE *fp;
     int err = 0;
 
     get_gretl_rc_path(rcfile);
-
     fp = gretl_fopen(rcfile, "r");
+
     if (fp == NULL) {
 	err = E_FOPEN;
-	goto bailout;
+    } else {
+	get_gretl_config_from_file(fp, &cpaths, dbproxy, &use_proxy);
+	fclose(fp);
     }
 
-    while (fgets(line, sizeof line, fp) != NULL) {
-	if (*line == '#') {
-	    continue;
-	}
-	if (!strncmp(line, "recent", 6)) {
-	    /* reached the "recent files" section */
-	    break;
-	}
-	if (sscanf(line, "%s", key) == 1) {
-	    *val = '\0';
-	    /* get the string that follows " = " */ 
-	    strncat(val, line + strlen(key) + 3, MAXLEN - 1);
-	    gretl_strstrip(val); 
-	    if (!strcmp(key, "gretldir")) {
-		strncat(cpaths.gretldir, val, MAXLEN - 1);
-	    } else if (!strcmp(key, "gnuplot")) {
-		strncat(cpaths.gnuplot, val, MAXLEN - 1);
-	    } else if (!strcmp(key, "userdir")) {
-		strncat(cpaths.workdir, val, MAXLEN - 1);
-	    } else if (!strcmp(key, "no_dotdir")) {
-		if (rc_bool(val)) {
-		    paths_opt = OPT_D;
-		}
-	    } else if (!strcmp(key, "shellok")) {
-		libset_set_bool(SHELL_OK, rc_bool(val));
-	    } else if (!strcmp(key, "usecwd")) {
-		usecwd = rc_bool(val);
-		libset_set_bool(USE_CWD, usecwd);
-	    } else if (!strcmp(key, "dbhost")) {
-		strncat(cpaths.dbhost, val, 32 - 1);
-	    } else if (!strcmp(key, "dbproxy")) {
-		strncat(dbproxy, val, PROXLEN - 1);
-	    } else if (!strcmp(key, "useproxy")) {
-		use_proxy = rc_bool(val);
-	    } else if (!strcmp(key, "x12a")) {
-		strncat(cpaths.x12a, val, MAXLEN - 1);
-	    } else if (!strcmp(key, "tramo")) {
-		strncat(cpaths.tramo, val, MAXLEN - 1);
-	    } else if (!strcmp(key, "Rbin")) {
-		strncat(cpaths.rbinpath, val, MAXLEN - 1);
-	    } else if (!strcmp(key, "Rlib")) {
-		strncat(cpaths.rlibpath, val, MAXLEN - 1);
-	    } else if (!strcmp(key, "ox")) {
-		strncat(cpaths.oxlpath, val, MAXLEN - 1);
-	    } else if (!strcmp(key, "octave")) {
-		strncat(cpaths.octpath, val, MAXLEN - 1);
-	    } else if (!strcmp(key, "stata")) {
-		strncat(cpaths.statapath, val, MAXLEN - 1);
-	    } else if (!strcmp(key, "python")) {
-		strncat(cpaths.pypath, val, MAXLEN - 1);
-	    } else if (!strcmp(key, "Png_font")) {
-		strncat(cpaths.pngfont, val, 128 - 1);
-	    } else if (!strcmp(key, "Gp_colors")) {
-		rc_set_gp_colors(val);
-	    } else if (!strcmp(key, "HC_xsect")) {
-		set_xsect_hccme(val);
-	    } else if (!strcmp(key, "HC_tseri")) {
-		set_tseries_hccme(val);
-	    } else if (!strcmp(key, "HC_panel")) {
-		set_panel_hccme(val);
-	    } else if (!strcmp(key, "HC_garch")) {
-		set_garch_robust_vcv(val);
-	    }
-	}
-    }
-
-    fclose(fp);
-
-    if (usecwd) {
-	char *s, cwd[MAXLEN];
-
-	s = getcwd(cwd, MAXLEN);
-	if (s != NULL) {
-	    *cpaths.workdir = '\0';
-	    strncat(cpaths.workdir, s, MAXLEN - 2);
-	    slash_terminate(cpaths.workdir);
-	}
-    }
-
- bailout:
-
     if (err) {
-	gretl_set_paths(&cpaths, paths_opt);
+	gretl_set_paths(&cpaths);
     } else {
-	err = gretl_set_paths(&cpaths, paths_opt);
+	err = gretl_set_paths(&cpaths);
     }
 
+#ifdef USE_CURL
     gretl_www_init(cpaths.dbhost, dbproxy, use_proxy);
+#endif
 
 #if 0
     show_paths();
