@@ -602,6 +602,36 @@ char *gretl_print_steal_buffer (PRN *prn)
     return buf;
 }
 
+/**
+ * gretl_print_replace_buffer:
+ * @prn: printing struct.
+ * @buf: malloced replacement buffer.
+ * 
+ * If @prn currently has a printing buffer in place,
+ * destroy the original and replace it with @buf. Note
+ * @prn "takes ownership" of @buf, which will be freed
+ * when @prn is destroyed.
+ *
+ * Returns: 0 on success, non-zero code on error.
+ */
+
+int gretl_print_replace_buffer (PRN *prn, char *buf)
+{
+    int err = 0;
+
+    if (prn == NULL || prn->buf == NULL || buf == NULL) {
+	err = E_DATA;
+    } else {
+	free(prn->buf);
+	prn->buf = buf;
+	prn->blen = strlen(buf);
+	prn->bufsize = prn->blen + 1;
+	prn->savepos = -1;
+    }
+
+    return err;
+}
+
 /**
  * gretl_print_read_tempfile:
  * @prn: printing struct.
@@ -1225,7 +1255,13 @@ int plain_format (PRN *prn)
 
 int rtf_format (PRN *prn)
 {
-    return (prn != NULL && (prn->format & GRETL_FORMAT_RTF));
+    if (prn != NULL) {
+	if (prn->format & (GRETL_FORMAT_RTF | GRETL_FORMAT_RTF_TXT)) {
+	    return 1;
+	}
+    }
+
+    return 0;
 }
 
 /**
