@@ -21,8 +21,10 @@
 #include "libset.h"
 #include "gretl_func.h"
 #include "uservar.h"
-#include "gretl_www.h"
 #include "gretl_string_table.h"
+#ifdef USE_CURL
+# include "gretl_www.h"
+#endif
 
 #include <glib.h>
 
@@ -67,7 +69,7 @@ static gretl_string_table *gretl_string_table_alloc (void)
 /**
  * gretl_string_table_new:
  * @list: list of series IDs whose values are to be 
- * given a string representation. or NULL. 
+ * given a string representation, or NULL. 
  * 
  * These values in @list should correspond to the 0-based indices 
  * of the series in question within the dataset.  For example, 
@@ -231,10 +233,14 @@ int *series_table_map (series_table *st_from, series_table *st_to)
  * should not be modified in any way.
  */
 
-const char **series_table_get_strings (series_table *st, int *n_strs)
+char **series_table_get_strings (series_table *st, int *n_strs)
 {
-    *n_strs = st->n_strs;
-    return (const char **) st->strs;
+    if (st != NULL) {
+	*n_strs = st->n_strs;
+	return st->strs;
+    } else {
+	return NULL;
+    }
 }
 
 /**
@@ -591,18 +597,10 @@ static built_in_string built_ins[] = {
     { "tramo",    NULL },
     { "tramodir", NULL },
     { "seats",    NULL },
-    { "shelldir", NULL },
-    { "Rbin",     NULL },
-    { "Rlib",     NULL },
-    { "pkgdir",   NULL }
+    { "pkgdir",   NULL },
+    { "pstmp",    NULL }
 };
 
-#ifdef WIN32
-static built_in_string dsep = { "dirsep", "\\" };
-#else
-static built_in_string dsep = { "dirsep", "/" };
-#endif
-
 void builtin_strings_cleanup (void)
 {
     int i, n = sizeof built_ins / sizeof built_ins[0];
@@ -623,16 +621,18 @@ void builtin_strings_cleanup (void)
 
 void gretl_insert_builtin_string (const char *name, const char *s)
 {
-    int n = sizeof built_ins / sizeof built_ins[0];
-    int i, m;
+    int i, n = sizeof built_ins / sizeof built_ins[0];
 
     for (i=0; i<n; i++) {
 	if (!strcmp(name, built_ins[i].name)) {
 	    free(built_ins[i].s);
 	    if (s == NULL) {
 		built_ins[i].s = NULL;
+	    } else if (!strcmp(name, "pstmp")) {
+		built_ins[i].s = gretl_strdup(s);
 	    } else {
-		m = strlen(s);
+		int m = strlen(s);
+
 		if (s[m-1] == SLASH) {
 		    /* drop trailing dir separator for paths */
 		    built_ins[i].s = gretl_strndup(s, m - 1);
@@ -647,15 +647,13 @@ void gretl_insert_builtin_string (const char *name, const char *s)
 
 char *get_built_in_string_by_name (const char *name)
 {
-    if (!strcmp(name, "dirsep")) {
-	return dsep.s;
-    } else {
-	int i, n = sizeof built_ins / sizeof built_ins[0];
+    int i, n = sizeof built_ins / sizeof built_ins[0];
 
-	for (i=0; i<n; i++) {
-	    if (!strcmp(name, built_ins[i].name)) {
-		return built_ins[i].s;
-	    }
+    for (i=0; i<n; i++) {
+	if (!strcmp(name, built_ins[i].name)) {
+	    char *s = built_ins[i].s;
+
+	    return s != NULL ? s : "";
 	}
     }
 
@@ -826,10 +824,15 @@ char *retrieve_file_content (const char *fname, const char *codeset,
     if (fname == NULL || *fname == '\0') {
 	*err = E_DATA;
     } else if (is_web_resource(fname)) {
+#ifdef USE_CURL
 	content = retrieve_public_file_as_buffer(fname, &len, err);
 	if (!*err && !g_utf8_validate(content, len, NULL)) {
 	    content = recode_content(content, codeset, err);
 	}
+#else
+	gretl_errmsg_set(_("Internet access not supported"));
+	*err = E_DATA;
+#endif
     } else {
 	char fullname[FILENAME_MAX];
 	GError *gerr = NULL;
