@@ -29,6 +29,10 @@
 #include <shlobj.h>
 #include <aclapi.h>
 
+#define REGDEBUG 0
+
+#if REGDEBUG
+
 static void win_print_last_error (void)
 {
     DWORD dw = GetLastError();
@@ -50,6 +54,8 @@ static void win_print_last_error (void)
     }
 }
 
+#endif
+
 /* returns 0 on success */
 
 int read_reg_val (HKEY tree, const char *base, 
@@ -71,8 +77,10 @@ int read_reg_val (HKEY tree, const char *base,
 		       );
 
     if (ret != ERROR_SUCCESS) {
+#if REGDEBUG
 	fprintf(stderr, "Couldn't read registry path %s\n", regpath);
 	win_print_last_error();
+#endif
         return 1;
     }
 
@@ -147,121 +155,50 @@ static FILE *cli_rcfile_open (void)
     return fp;
 }
 
-static int read_rc_string (FILE *fp, const char *key, char *value)
-{
-    char line[MAXLEN];
-    char keystr[32];
-    int n, ret = 0;
-
-    sprintf(keystr, "%s = ", key);
-    n = strlen(keystr);
-    rewind(fp);
-
-    while (fgets(line, sizeof line, fp) && !ret) {
-	gretl_strstrip(line);
-	if (!strncmp(line, keystr, n)) {
-	    strcpy(value, line + n);
-	    ret = 1;
-	}
-    }
-
-    return ret;
-}
-
-static int read_rc_var (char *key, char *val,
-			FILE *fnet, FILE *frc,
-			const char *regbase,
-			HKEY tree)
-{
-    int done = 0;
-
-    *val = '\0';
-
-    if (fnet != NULL) {
-	done = read_rc_string(fnet, key, val);
-    }
-
-    if (!done && frc != NULL) {
-	done = read_rc_string(frc, key, val);
-    }
-
-    if (!done && regbase != NULL) {
-	read_reg_val(tree, regbase, key, val);
-    }
-
-    return *val != '\0';
-}
-
-void cli_read_registry (char *callname)
+void win32_cli_read_rc (char *callname)
 {
     ConfigPaths cpaths = {0};
-    char valstr[MAXLEN];
-    char dbproxy[64];
+    char dbproxy[64] = {0};
     int use_proxy = 0;
-    FILE *fnet, *frc;
+    FILE *fp;
     
-    fnet = cli_gretlnet_open(callname);
-    frc = cli_rcfile_open();
-
-    /* gretl installation directory */
-    read_rc_var("gretldir", cpaths.gretldir, fnet, frc, 
-		"gretl", HKEY_LOCAL_MACHINE);
-
-    /* user's working directory */
-    read_rc_var("userdir", cpaths.workdir, fnet, frc, 
-		"gretl", HKEY_CURRENT_USER);
-
-    /* path to X-12-ARIMA */
-    read_rc_var("x12a", cpaths.x12a, fnet, frc, 
-		"x12arima", HKEY_LOCAL_MACHINE);
-
-    /* path to tramo */
-    read_rc_var("tramo", cpaths.tramo, fnet, frc,
-		"tramo", HKEY_LOCAL_MACHINE);
-
-    /* path to R binary (non-interactive use) */
-    read_rc_var("Rbin", cpaths.rbinpath, fnet, frc, NULL, 0);
-
-    /* path to R shared library */
-    read_rc_var("Rlib", cpaths.rlibpath, fnet, frc, NULL, 0);
-
-    /* path to oxl */
-    read_rc_var("ox", cpaths.oxlpath, fnet, frc, NULL, 0);
-
-    /* remote database host */
-    read_rc_var("dbhost", cpaths.dbhost, fnet, frc, 
-		"gretl", HKEY_CURRENT_USER);
-
-    /* www proxy for reading remote databases */
-    read_rc_var("dbproxy", dbproxy, fnet, frc, 
-		"gretl", HKEY_CURRENT_USER);
-
-    /* should a proxy be used? */
-    read_rc_var("useproxy", valstr, fnet, frc, 
-		"gretl", HKEY_CURRENT_USER);
-    if (!strcmp(valstr, "true") || !strcmp(valstr, "1")) {
-	use_proxy = 1;
-    } 
-
-    /* do we allow the shell command within gretl? */
-    read_rc_var("shellok", valstr, fnet, frc, 
-		"gretl", HKEY_CURRENT_USER);
-    if (!strcmp(valstr, "true") || !strcmp(valstr, "1")) {
-	libset_set_bool(SHELL_OK, 1);
-    } else {
-	libset_set_bool(SHELL_OK, 0);
+    /* try for a per-user rc file first */
+    fp = cli_rcfile_open();
+    if (fp != NULL) {
+	get_gretl_config_from_file(fp, &cpaths, dbproxy, &use_proxy);
+	fclose(fp);
     }
 
-    gretl_set_paths(&cpaths, OPT_NONE);
+    /* read the "gretlnet" file, if present: any settings from this
+       file will override those from the per-user rc file.
+    */ 
+    fp = cli_gretlnet_open(callname);
+    if (fp != NULL) {
+	get_gretl_config_from_file(fp, &cpaths, dbproxy, &use_proxy);
+	fclose(fp);
+    }	
+    
+    /* for a short list of items, if they're (still) missing, maybe we 
+       can get them from the registry 
+    */
+
+    if (cpaths.gretldir[0] == '\0') {
+	read_reg_val(HKEY_LOCAL_MACHINE, "gretl", "gretldir",
+		     cpaths.gretldir);
+    }
+
+    if (cpaths.x12a[0] == '\0') {
+	read_reg_val(HKEY_LOCAL_MACHINE, "x12arima", "x12a",
+		     cpaths.x12a);
+    }
+
+    if (cpaths.tramo[0] == '\0') {
+	read_reg_val(HKEY_LOCAL_MACHINE, "tramo", "tramo",
+		     cpaths.tramo);
+    }	
+
+    gretl_set_paths(&cpaths);
     gretl_www_init(cpaths.dbhost, dbproxy, use_proxy);
-
-    if (fnet != NULL) {
-	fclose(fnet);
-    }
-
-    if (frc != NULL) {
-	fclose(frc);
-    }
 }
 
 void win_show_last_error (void)
@@ -318,7 +255,6 @@ static int real_win_run_sync (char *cmdline, const char *currdir,
 
     ZeroMemory(&si, sizeof si);
     ZeroMemory(&pi, sizeof pi);  
-
     si.cb = sizeof si;
 
     if (console_app) {
@@ -344,7 +280,7 @@ static int real_win_run_sync (char *cmdline, const char *currdir,
 	WaitForSingleObject(pi.hProcess, INFINITE); 
 	if (GetExitCodeProcess(pi.hProcess, &exitcode)) {
 	    if (exitcode != 0) {
-		gretl_errmsg_sprintf("%s: exit code %d\n", cmdline, 
+		gretl_errmsg_sprintf("%s: exit code %d", cmdline, 
 				     exitcode);
 		err = 1;
 	    }
@@ -1010,6 +946,32 @@ int maybe_print_R_path_addition (FILE *fp)
     return err;
 }
 
+int win32_check_for_program (const char *prog)
+{
+    char tmp[MAXLEN];
+    WIN32_FIND_DATA find_data;
+    HANDLE hfind;
+    int ret = 1;
+
+    if (prog == NULL || *prog == '\0') {
+	return 0;
+    }
+
+    hfind = FindFirstFile(prog, &find_data);
+    if (hfind == INVALID_HANDLE_VALUE) {
+	ret = 0;
+    }
+    FindClose(hfind);
+
+    if (ret == 0) {
+	char *p;
+
+	ret = SearchPath(NULL, prog, NULL, MAXLEN, tmp, &p);
+    }
+
+    return ret != 0;
+}
+
 /* the following needed since mingw does not include strptime */
 
 /*
@@ -1499,3 +1461,87 @@ char *strptime (const char *buf, const char *format, struct tm *timeptr)
 
     return (char *) buf;
 }
+
+double win32_fscan_nonfinite (FILE *fp, int *err)
+{
+    char test[5];
+
+    fscanf(fp, "%4s", test);
+
+    if (!strncmp(test, "nan", 3) ||
+	!strncmp(test, "-nan", 4)) {
+	return M_NA;
+    } else if (!strncmp(test, "inf", 3)) {
+	return 1.0 / 0.0;
+    } else if (!strncmp(test, "-inf", 4)) {
+	return -1.0 / 0.0;
+    } else {
+	*err = E_DATA;
+	return 0;
+    }
+}
+
+double win32_sscan_nonfinite (const char *s, int *err)
+{
+    char test[5];
+
+    sscanf(s, "%4s", test);
+
+    if (!strncmp(test, "nan", 3) ||
+	!strncmp(test, "-nan", 4)) {
+	return M_NA;
+    } else if (!strncmp(test, "inf", 3)) {
+	return 1.0 / 0.0;
+    } else if (!strncmp(test, "-inf", 4)) {
+	return -1.0 / 0.0;
+    } else {
+	*err = E_DATA;
+	return 0;
+    }
+}
+
+void win32_fprint_nonfinite (double x, FILE *fp)
+{
+    if (isnan(x) || na(x)) {
+	fputs("nan ", fp);
+    } else if (x < 0) {
+	fputs("-inf ", fp);
+    } else {
+	fputs("inf ", fp);
+    }
+}
+
+void win32_pprint_nonfinite (double x, PRN *prn)
+{
+    if (isnan(x) || na(x)) {
+	pputs(prn, "nan ");
+    } else if (x < 0) {
+	pputs(prn, "-inf ");
+    } else {
+	pputs(prn, "inf ");
+    }
+}
+
+static LARGE_INTEGER timer_freq;
+static LARGE_INTEGER wt0;
+
+void win32_stopwatch_init (void)
+{
+    QueryPerformanceFrequency(&timer_freq);
+    QueryPerformanceCounter(&wt0);
+}
+
+double win32_stopwatch (void)
+{
+    LARGE_INTEGER wt1;
+    gint64 dt;
+
+    QueryPerformanceCounter(&wt1);
+    dt = wt1.QuadPart - wt0.QuadPart;
+    dt *= 1000000;
+    dt /= timer_freq.QuadPart;
+    wt0 = wt1;
+
+    return (double) dt / 1.0e6;
+} 
+
