@@ -19,21 +19,13 @@
 
 /* gretl_www.c for gretl -- uses libcurl API */
 
-/* Note: STANDALONE is defined if we're building the stand-alone
-   updater program for Windows.  This program does not depend on glib
-   or libgretl.
-*/
-
 #ifdef WIN64
 # include <winsock2.h>
 #endif
 
-#ifndef STANDALONE
-# include "libgretl.h"
-# include "libset.h"
-# include "build.h"
-#endif
-
+#include "libgretl.h"
+#include "libset.h"
+#include "build.h"
 #include "version.h"
 #include "gretl_www.h"
 
@@ -62,10 +54,8 @@ static char datapkg_list[DBHLEN] = "/addons-data/datapkgs.txt";
 static int wproxy;
 static char proxyhost[128];
 
-#ifndef STANDALONE
 static char sffiles[DBHLEN] = "downloads.sourceforge.net";
 static char sfweb[DBHLEN]   = "gretl.sourceforge.net";
-#endif
 
 typedef struct urlinfo_ urlinfo;
 
@@ -81,7 +71,6 @@ struct urlinfo_ {
     char agent[32];          /* user-agent string */
     FILE *fp;                /* for saving content locally */
     int (*progfunc)();       /* progress indicator function */
-    void *phandle;           /* plugin handle */
     int pstarted;            /* progress bar status flag */
 };
 
@@ -108,12 +97,9 @@ static void urlinfo_init (urlinfo *u,
     u->verbose = getenv("GRETL_WWW_VERBOSE") != NULL;
 
     u->progfunc = NULL;
-    u->phandle = NULL;
     u->pstarted = 0;
 
-#ifndef STANDALONE
     gretl_error_clear();
-#endif
 
 #ifdef BUILD_DATE
     sprintf(u->agent, "gretl-%s-%s", GRETL_VERSION, BUILD_DATE);
@@ -121,14 +107,12 @@ static void urlinfo_init (urlinfo *u,
     sprintf(u->agent, "gretl-%s", GRETL_VERSION);
 #endif
 
-#if defined(STANDALONE) || defined (WIN32)
-    /* the linux test updater program pretends to be Windows */
+#ifdef WIN32
     strcat(u->agent, "w");
 #endif
 }
 
-static void urlinfo_finalize (urlinfo *u, char **getbuf,
-			      int err)
+static void urlinfo_finalize (urlinfo *u, char **getbuf, int *err)
 {
     if (u->fp != NULL) {
 	fclose(u->fp); 
@@ -136,36 +120,24 @@ static void urlinfo_finalize (urlinfo *u, char **getbuf,
 	*getbuf = u->getbuf;
     }
 
-    if (err && u->localfile != NULL) {
+    if (*err && u->localfile != NULL) {
 	gretl_remove(u->localfile);
     }
-}
 
-#ifdef STANDALONE
+    if (u->saveopt == SAVE_TO_FILE || u->saveopt == SAVE_TO_BUFFER) {
+	if (u->datalen == 0) {
+	    *err = E_DATA;
+	}
+    }    
+}
 
 static void urlinfo_set_show_progress (urlinfo *u)
 {
-    return;
-}
+    int (*show_progress) (gint64, gint64, int) = NULL;
 
-static int progress_func (void *clientp, double dltotal, double dlnow, 
-			  double ultotal, double ulnow)
-{
-    return 0;
-}
-
-#else
-
-static void urlinfo_set_show_progress (urlinfo *u)
-{
-    int (*show_progress) (long, long, int) = NULL;
-    void *handle;
-
-    show_progress = get_plugin_function("show_progress", 
-					&handle);
+    show_progress = get_plugin_function("show_progress");
     if (show_progress != NULL) {
 	u->progfunc = show_progress;
-	u->phandle = handle;
     }
 }
 
@@ -187,15 +159,10 @@ static int progress_func (void *clientp, double dltotal, double dlnow,
     return (ret == SP_RETURN_CANCELED) ? 1 : 0;
 }
 
-#endif
-
 static void stop_progress_bar (urlinfo *u)
 {
     if (u->progfunc != NULL) {
 	u->progfunc(0, 1024, SP_FINISH);
-#ifndef STANDALONE
-	close_plugin(u->phandle);
-#endif
     }
 }
 
@@ -276,7 +243,6 @@ static size_t write_func (void *buf, size_t size, size_t nmemb,
 
 static int progress_bar_wanted (int opt)
 {
-#ifndef STANDALONE
     if (gretl_in_gui_mode()) {
 	return (opt == GRAB_IDX || 
 		opt == GRAB_DATA ||
@@ -287,7 +253,6 @@ static int progress_bar_wanted (int opt)
 		opt == GRAB_PKG ||
 		opt == GRAB_FOREIGN);
     }
-#endif
     return 0;
 }
 
@@ -304,8 +269,6 @@ static const char *print_option (int opt)
 	return "GRAB_NBO_DATA";
     case GRAB_FILE:
 	return "GRAB_FILE";
-    case QUERY:
-	return "QUERY";
     case LIST_FUNCS:
 	return "LIST_FUNCS";
     case GRAB_FUNC:
@@ -356,7 +319,7 @@ static void maybe_revise_www_paths (void)
     }
 }
 
-static int gretl_curl_switch (int on)
+static int gretl_curl_toggle (int on)
 {
     static int init_done;
 
@@ -384,7 +347,7 @@ static int curl_get (urlinfo *u)
     CURLcode res;
     int err = 0;
 
-    err = gretl_curl_switch(1);
+    err = gretl_curl_toggle(1);
     if (err) {
 	return err;
     }
@@ -512,7 +475,7 @@ static int retrieve_url (const char *hostname,
 
     err = curl_get(&u);
 
-    urlinfo_finalize(&u, getbuf, err);
+    urlinfo_finalize(&u, getbuf, &err);
 
     return err;
 }
@@ -542,36 +505,29 @@ int gretl_www_init (const char *host, const char *proxy, int use_proxy)
 
 void gretl_www_cleanup (void)
 {
-    gretl_curl_switch(0);
+    gretl_curl_toggle(0);
 }
 
-int get_update_info (char **saver, time_t filedate, int queryopt)
+int get_update_info (char **saver, int verbose)
 {
     urlinfo u;
-    char tmp[32];
     int err = 0;
 
     urlinfo_init(&u, gretlhost, SAVE_TO_BUFFER, NULL);
     strcat(u.url, updatecgi);
 
-    if (queryopt == QUERY_VERBOSE) {
-	strcat(u.url, "?opt=MANUAL_QUERY&date=");
+    if (verbose) {
+	strcat(u.url, "?opt=MANUAL_QUERY");
     } else {
-	strcat(u.url, "?opt=QUERY&date=");
+	strcat(u.url, "?opt=QUERY");
     } 
 
-    sprintf(tmp, "%d", (int) filedate); /* FIXME? */
-    strcat(u.url, tmp);
-
     err = curl_get(&u);
-
-    urlinfo_finalize(&u, saver, err);
+    urlinfo_finalize(&u, saver, &err);
 
     return err;
 }
 
-#ifndef STANDALONE /* functions below not needed for updater */
-
 /* The content of the function package to be uploaded is in @buf;
    the (short, pathless) filename for this package is in @fname.
    If @retbuf is non-NULL it gets a copy of the response from the
@@ -598,7 +554,7 @@ int upload_function_package (const char *login, const char *pass,
     urlinfo_init(&u, gretlhost, saveopt, NULL);
     strcat(u.url, datacgi);
 
-    err = gretl_curl_switch(1);
+    err = gretl_curl_toggle(1);
     if (err) {
 	return err;
     }
@@ -823,7 +779,7 @@ static int proto_length (const char *s)
  * of the file on the server.
  *
  * Retrieves the specified resource and writes it to
- * @localname, if possible. Only handles http requests.
+ * @localname, if possible.
  *
  * Returns: 0 on success, non-zero on failure.
  */
@@ -857,7 +813,7 @@ int retrieve_public_file (const char *uri, char *localname)
 	    urlinfo_set_show_progress(&u);
 	}
 	err = curl_get(&u);
-	urlinfo_finalize(&u, NULL, err);
+	urlinfo_finalize(&u, NULL, &err);
     }
 
     if (err) {
@@ -896,7 +852,7 @@ char *retrieve_public_file_as_buffer (const char *uri, size_t *len,
 	urlinfo_init(&u, NULL, SAVE_TO_BUFFER, NULL);
 	strcpy(u.url, uri);
 	*err = curl_get(&u);
-	urlinfo_finalize(&u, &buf, *err);
+	urlinfo_finalize(&u, &buf, err);
 	*len = (*err)? 0 : u.datalen;
     }
 
@@ -912,4 +868,127 @@ char *retrieve_public_file_as_buffer (const char *uri, size_t *len,
     return buf;
 }
 
-#endif /* !STANDALONE */
+/* below: back-end to the implementation of the hansl "curl"
+   function
+*/
+
+struct GetBuf {
+    char **pbuf;
+    size_t written;
+};
+
+static size_t curl_bufwrite (void *buf, size_t sz, size_t nmemb, void *p)
+{
+    struct GetBuf *out = (struct GetBuf *) p;
+    char *mem;
+
+    if (out == NULL || out->pbuf == NULL || nmemb == 0) {
+	return 0;
+    }    
+
+    sz *= nmemb;
+    mem = realloc(*out->pbuf, out->written + sz + 1);
+
+    if (mem != NULL) {
+	memset(mem + out->written, 0, sz + 1);
+	*out->pbuf = mem;
+	mem = memcpy(mem + out->written, buf, sz);
+	out->written += sz;
+    }
+
+    return (mem == NULL)? 0 : nmemb;
+}
+
+/**
+ * gretl_curl:
+ * @url: complete URL: protocol, host and path.
+ * @header: optional HTTP header (or NULL).
+ * @postdata: string to send as data for POST (or NULL).
+ * @include: if non-zero, include the received header with
+ * the body output.
+ * @output: location to receive the output.
+ * @errmsg: location to receive cURL error message, or NULL.
+ *
+ * Somewhat flexible URI "grabber", allowing use of the POST
+ * method with header and data to be sent to the host.
+ *
+ * Returns: 0 on success, non-zero code on error.
+ */
+
+int gretl_curl (const char *url, const char *header, 
+		const char *postdata, int include,
+		char **output, char **errmsg)
+{
+    CURL *curl;
+    struct curl_slist *hlist = NULL;
+    struct GetBuf getbuf = {
+	output, /* pointer to buffer */
+	0       /* bytes written */
+    };
+    CURLcode res;
+    int err = 0;
+
+    err = gretl_curl_toggle(1);
+    if (err) {
+	return err;
+    }
+  
+    curl = curl_easy_init();
+    if (curl == NULL) {
+	gretl_errmsg_set("curl_easy_init failed");
+	return 1;
+    }
+
+    if (header != NULL) {
+	hlist = curl_slist_append(hlist, header);
+    }
+
+    if (getenv("GRETL_WWW_VERBOSE") != NULL) {
+	curl_easy_setopt(curl, CURLOPT_VERBOSE, 1);
+    }
+
+    curl_easy_setopt(curl, CURLOPT_URL, url);
+    curl_easy_setopt(curl, CURLOPT_WRITEDATA, &getbuf);
+    curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, curl_bufwrite);
+    curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);
+
+    if (include) {
+	curl_easy_setopt(curl, CURLOPT_HEADER, 1);
+    }  
+
+    if (hlist != NULL) {
+	curl_easy_setopt(curl, CURLOPT_HTTPHEADER, hlist);
+    }
+
+    if (postdata != NULL) {
+	curl_easy_setopt(curl, CURLOPT_POSTFIELDS, (void *) postdata);
+    }
+
+    if (wproxy && *proxyhost != '\0') {
+	curl_easy_setopt(curl, CURLOPT_PROXY, proxyhost);
+    }
+
+    res = curl_easy_perform(curl);
+
+    if (res != CURLE_OK) {
+	const char *cmsg = curl_easy_strerror(res);
+
+	gretl_errmsg_sprintf("cURL error %d (%s)", res, cmsg);
+	if (*output != NULL) {
+	    free(*output);
+	    *output = NULL;
+	}
+	if (errmsg != NULL) {
+	    *errmsg = gretl_strdup(cmsg);
+	}
+	err = E_DATA;
+    }
+
+    if (hlist != NULL) {
+	curl_slist_free_all(hlist);
+    }    
+    curl_easy_cleanup(curl);
+
+    return err;
+}
+
