@@ -28,4 +28,10 @@ int gretl_make_zipfile (const char *fname, const char *path,
 int gretl_unzip_session_file (const char *fname, gchar **zdirname, 
 			      GError **gerr);
 
+int gretl_zip_datafile (const char *fname, const char *path,
+			int level, GError **gerr);
+
+int gretl_unzip_datafile (const char *fname, const char *path,
+			  GError **gerr);
+
 #endif /* GRETL_ZIP_H */
