@@ -24,9 +24,16 @@
 #include "usermat.h"
 #include "uservar.h"
 #include "matrix_extra.h"
-#include "gretl_string_table.h"
 #include "gretl_func.h"
 
+#ifdef _OPENMP
+# include <omp.h>
+#endif
+
+#ifdef HAVE_MPI
+# include "gretl_mpi.h"
+#endif
+
 #include <unistd.h>
 #include <errno.h>
 
@@ -68,7 +75,7 @@ enum {
 };    
 
 /* for values that really want a non-negative integer */
-#define UNSET_INT -1
+#define UNSET_INT -9
 #define is_unset(i) (i == UNSET_INT)
 
 typedef struct set_vars_ set_vars;
@@ -95,6 +102,7 @@ struct set_vars_ {
     double bfgs_toler;          /* convergence tolerance, BFGS */
     double bfgs_maxgrad;        /* max acceptable gradient norm, BFGS */
     int bfgs_verbskip;          /* BFGS: show one in n iterations  */
+    int optim_steplen;          /* step length algorithm (only BFGS for now) */
     int bhhh_maxiter;           /* max iterations, BHHH */          
     double bhhh_toler;          /* convergence tolerance, BHHH */
     int lbfgs_mem;              /* memory span for L-BFGS-B */
@@ -110,17 +118,22 @@ struct set_vars_ {
     char csv_write_na[8];       /* representation of NA in CSV output */
     char csv_read_na[8];        /* representation of NA in CSV input */
     double nadarwat_trim;       /* multiple of h to use in nadarwat() for trimming */
+    int fdjac_qual;             /* quality of "fdjac" function */
 };
 
 #define ECHO "echo"
 #define MESSAGES "messages"
 #define WARNINGS "warnings"
 #define GRETL_DEBUG "debug"
+#define USE_DCMT "use_dcmt"
+
 #define BLAS_MNK_MIN "blas_mnk_min"
+#define SIMD_K_MAX "simd_k_max"
+#define SIMD_MN_MIN "simd_mn_min"
 #define MP_MNK_MIN "mp_mnk_min"
-/* support dyslexia */
-#define BLAS_NMK_MIN "blas_nmk_min"
-#define MP_NMK_MIN "mp_nmk_min"
+/* but now preferred */
+#define OMP_MNK_MIN "omp_mnk_min"
+#define OMP_N_THREADS "omp_num_threads"
 
 #define libset_boolvar(s) (!strcmp(s, ECHO) || \
                            !strcmp(s, MESSAGES) || \
@@ -141,6 +154,7 @@ struct set_vars_ {
 			   !strcmp(s, R_LIB) || \
 			   !strcmp(s, BFGS_RSTEP) || \
 			   !strcmp(s, DPDSTYLE) || \
+			   !strcmp(s, USE_DCMT) || \
 			   !strcmp(s, USE_OPENMP))
 
 #define libset_double(s) (!strcmp(s, CONV_HUGE) || \
@@ -153,6 +167,7 @@ struct set_vars_ {
 
 #define libset_int(s) (!strcmp(s, BFGS_MAXITER) || \
 		       !strcmp(s, BFGS_VERBSKIP) || \
+		       !strcmp(s, OPTIM_STEPLEN) || \
 		       !strcmp(s, BHHH_MAXITER) || \
 		       !strcmp(s, GMM_MAXITER) || \
 		       !strcmp(s, LBFGS_MEM) || \
@@ -166,9 +181,12 @@ struct set_vars_ {
 		       !strcmp(s, GRETL_OPTIM) || \
 		       !strcmp(s, GRETL_DEBUG) || \
 		       !strcmp(s, BLAS_MNK_MIN) || \
+		       !strcmp(s, OMP_MNK_MIN) || \
 		       !strcmp(s, MP_MNK_MIN) || \
-		       !strcmp(s, BLAS_NMK_MIN) || \
-		       !strcmp(s, MP_NMK_MIN))
+		       !strcmp(s, OMP_N_THREADS) || \
+		       !strcmp(s, SIMD_K_MAX) || \
+		       !strcmp(s, SIMD_MN_MIN) || \
+		       !strcmp(s, FDJAC_QUAL))
 
 /* global state */
 set_vars *state;
@@ -179,7 +197,12 @@ static int R_lib = 1;
 static int csv_digits = UNSET_INT;
 static char data_delim = ',';
 static char data_export_decpoint = '.';
-static int mp_mnk_min = 89999; /* was 65535 */
+#ifdef OS_OSX
+static int omp_mnk_min = -1;
+#else
+static int omp_mnk_min = 80000; /* was 65535 */
+#endif
+static int omp_n_threads;
 
 static int boolvar_get_flag (const char *s);
 static const char *hac_lag_string (void);
@@ -255,15 +278,15 @@ static const char *optim_strs[] = {
     NULL
 };
 
-static const char *normal_rand_strs[] = {
-    "ziggurat",
-    "box-muller",
+static const char *steplen_strs[] = {
+    "power",
+    "quadratic",
     NULL
 };
 
-static const char *RNG_strs[] = {
-    "MT",
-    "SFMT",
+static const char *normal_rand_strs[] = {
+    "ziggurat",
+    "box-muller",
     NULL
 };
 
@@ -283,10 +306,10 @@ static const char **libset_option_strings (const char *s)
 	return optim_strs;
     } else if (!strcmp(s, NORMAL_RAND)) {
 	return normal_rand_strs;
-    } else if (!strcmp(s, RNG)) {
-	return RNG_strs;
     } else if (!strcmp(s, "csv_delim")) {
 	return csv_delim_args;
+    } else if (!strcmp(s, OPTIM_STEPLEN)) {
+	return steplen_strs;
     } else {
 	return NULL;
     }
@@ -335,8 +358,8 @@ static const char *libset_option_string (const char *s)
 	return optim_strs[state->optim];
     } else if (!strcmp(s, NORMAL_RAND)) {
 	return normal_rand_strs[gretl_rand_get_box_muller()];
-    } else if (!strcmp(s, RNG)) {
-	return RNG_strs[gretl_rand_get_sfmt()];
+    } else if (!strcmp(s, OPTIM_STEPLEN)) {
+	return steplen_strs[state->optim_steplen];
     } else {
 	return "?";
     }
@@ -403,6 +426,7 @@ static void state_vars_copy (set_vars *sv)
     sv->bfgs_toler = state->bfgs_toler;
     sv->bfgs_maxgrad = state->bfgs_maxgrad;
     sv->bfgs_verbskip = state->bfgs_verbskip;
+    sv->optim_steplen = state->optim_steplen;
     sv->bhhh_maxiter = state->bhhh_maxiter;
     sv->bhhh_toler = state->bhhh_toler;
     sv->lbfgs_mem = state->lbfgs_mem;
@@ -410,6 +434,7 @@ static void state_vars_copy (set_vars *sv)
     sv->arma_vcv = state->arma_vcv;
     sv->garch_robust_vcv = state->garch_robust_vcv;
     sv->nadarwat_trim = state->nadarwat_trim;
+    sv->fdjac_qual = state->fdjac_qual;
 
     sv->initvals = gretl_matrix_copy(state->initvals);
     sv->matmask = gretl_matrix_copy(state->matmask);
@@ -420,6 +445,44 @@ static void state_vars_copy (set_vars *sv)
     robust_opts_copy(&sv->ropts);
 }
 
+/* for processors count */
+#if defined(WIN32)
+# include <windows.h>
+#elif defined(OS_OSX)
+# include <sys/param.h>
+# include <sys/sysctl.h>
+#endif
+
+int gretl_n_processors (void)
+{
+    static int n_proc = -1;
+
+    if (n_proc >= 1) {
+	return n_proc;
+    }
+
+    n_proc = 1;
+
+#if defined(WIN32)
+    SYSTEM_INFO sysinfo;
+
+    GetSystemInfo(&sysinfo);
+    n_proc = sysinfo.dwNumberOfProcessors;
+#elif defined(OS_OSX)
+    int mib[2] = {CTL_HW, HW_NCPU}; /* or CTL_KERN, KERN_MAXPROC ? */
+    size_t len = sizeof n_proc;
+
+    if (sysctl(mib, 2, &n_proc, &len, NULL, 0) == -1) {
+	perror("could not determine number of CPUs available");
+	n_proc = 1;
+    }    
+#else
+    n_proc = sysconf(_SC_NPROCESSORS_ONLN);
+#endif
+
+    return n_proc;
+}
+
 #define OMP_SHOW 0
 
 int libset_use_openmp (guint64 n)
@@ -427,7 +490,7 @@ int libset_use_openmp (guint64 n)
 #if defined(_OPENMP)
     if (state == NULL || !(state->flags & STATE_OPENMP_ON)) {
 	return 0;
-    } else if (mp_mnk_min >= 0 && n > (guint64) mp_mnk_min) {
+    } else if (omp_mnk_min >= 0 && n >= (guint64) omp_mnk_min) {
 # if OMP_SHOW > 1
 	fprintf(stderr, "libset_use_openmp: yes\n");
 # endif
@@ -438,33 +501,37 @@ int libset_use_openmp (guint64 n)
     return 0;
 }
 
+static int set_omp_n_threads (int n)
+{
+#if defined(_OPENMP)
+    if (n < 1 || n > gretl_n_processors()) {
+	gretl_errmsg_sprintf("omp_num_threads: must be >= 1 and <= %d",
+			     gretl_n_processors());
+	return E_DATA;
+    } else {
+	omp_n_threads = n;
+	omp_set_num_threads(n);
+	return 0;
+    }
+#else
+    gretl_errmsg_set("OpenMP is not enabled");
+    return E_DATA;
+#endif
+}
+
+int get_omp_n_threads (void)
+{
+    return omp_n_threads;
+}
+
 #if defined(_OPENMP)
-# if defined(WIN32)
-# include <windows.h>
-# elif defined(OS_OSX)
-# include <sys/param.h>
-# include <sys/sysctl.h>
-# endif
 
 static int openmp_by_default (void)
 {
-    int num_cores = 1, ret = 0;
-# if defined(WIN32)
-    SYSTEM_INFO sysinfo;
+    static int called = 0;
+    int num_cores = gretl_n_processors();
+    int ret = num_cores > 1;
 
-    GetSystemInfo(&sysinfo);
-    num_cores = sysinfo.dwNumberOfProcessors;
-# elif defined(OS_OSX)
-    int mib[2] = {CTL_HW, HW_NCPU}; /* or CTL_KERN, KERN_MAXPROC ? */
-    size_t len = sizeof num_cores;
-
-    if (sysctl(mib, 2, &num_cores, &len, NULL, 0) == -1) {
-	perror("could not determine number of cpus available");
-    }    
-# else
-    num_cores = sysconf(_SC_NPROCESSORS_ONLN);
-# endif
-    ret = num_cores > 1;
     if (ret) {
 	/* one can use the environment to turn this off */
 	char *envstr = getenv("GRETL_USE_OPENMP");
@@ -474,17 +541,22 @@ static int openmp_by_default (void)
 	}
     }
 
+    if (!called && ret) {
+	char *s = getenv("OMP_NUM_THREADS");
+
+	if (s != NULL && *s != '\0') {
+	    omp_n_threads = atoi(s);
+	} else {
+	    omp_n_threads = num_cores;
+	}
+	called = 1;
+    }
+
 # if OMP_SHOW
     if (1) {
 	fprintf(stderr, "number of cores detected = %d\n", num_cores);
 	fprintf(stderr, "use OpenMP by default? %s\n", ret ? "yes" : "no");
-	if (ret) {
-	    char *s = getenv("OMP_NUM_THREADS");
-
-	    if (s != NULL && *s != '\0') {
-		fprintf(stderr, "OMP_NUM_THREADS set to %d\n", atoi(s));
-	    }
-	}
+	fprintf(stderr, "omp_num_threads = %d\n", omp_n_threads);
     }
 # endif	
 
@@ -518,10 +590,11 @@ static void state_vars_init (set_vars *sv)
     sv->initvals = NULL;
     sv->matmask = NULL;
 
-    sv->bfgs_maxiter = -1;
+    sv->bfgs_maxiter = UNSET_INT;
     sv->bfgs_toler = NADBL;
     sv->bfgs_maxgrad = 5.0;
     sv->bfgs_verbskip = 1;
+    sv->optim_steplen = STEPLEN_POWER;
     sv->bhhh_maxiter = 500;
     sv->bhhh_toler = NADBL;
     sv->lbfgs_mem = 8;
@@ -529,6 +602,7 @@ static void state_vars_init (set_vars *sv)
     sv->arma_vcv = ML_HESSIAN;
     sv->garch_robust_vcv = ML_UNSET;
     sv->nadarwat_trim = 4.0;
+    sv->fdjac_qual = 0;
 
     *sv->shelldir = '\0';
 
@@ -617,20 +691,21 @@ void set_mp_bits (int b)
 
 int get_mp_bits (void)
 {
-    if (user_mp_bits >= DEFAULT_MP_BITS) {
+    if (user_mp_bits > DEFAULT_MP_BITS) {
 	return user_mp_bits;
     } else {
 	char *s = getenv("GRETL_MP_BITS");
 	int b;
 
-	if (s != NULL) {
+	if (s != NULL) { 
 	    b = atoi(s);
 	    if (mp_bits_ok(b)) {
 		return b;
 	    }
 	}
-	return DEFAULT_MP_BITS;
     }
+
+    return DEFAULT_MP_BITS;
 }
 
 const gretl_matrix *get_init_vals (void)
@@ -818,10 +893,11 @@ static int negval_invalid (const char *var)
 
     if (var != NULL) {
 	if (!strcmp(var, BLAS_MNK_MIN) ||
+	    !strcmp(var, OMP_MNK_MIN) ||
 	    !strcmp(var, MP_MNK_MIN) ||
-	    !strcmp(var, BLAS_NMK_MIN) ||
-	    !strcmp(var, MP_NMK_MIN)) {
-	    /* these can be set to -1 */
+	    !strcmp(var, SIMD_K_MAX) ||
+	    !strcmp(var, SIMD_MN_MIN)) {
+	    /* these can all be set to -1 */
 	    ret = 0;
 	}
     }
@@ -952,15 +1028,15 @@ static int parse_libset_int_code (const char *key,
 		break;
 	    }
 	}
-    } else if (!g_ascii_strcasecmp(key, RNG)) {
-	for (i=0; RNG_strs[i] != NULL; i++) {
-	    if (!g_ascii_strcasecmp(val, RNG_strs[i])) {
-		gretl_rand_set_sfmt(i);
+    } else if (!g_ascii_strcasecmp(key, OPTIM_STEPLEN)) {
+	for (i=0; i<STEPLEN_MAX; i++) {
+	    if (!g_ascii_strcasecmp(val, steplen_strs[i])) {
+		state->optim_steplen = i;
 		err = 0;
 		break;
 	    }
-	}
-    }	
+	}	
+    }
 
     if (err) {
 	gretl_errmsg_sprintf(_("%s: invalid value '%s'"), key, val);
@@ -1118,8 +1194,6 @@ void shelldir_init (const char *s)
 	    *state->shelldir = '\0';
 	} 
     }
-
-    gretl_insert_builtin_string("shelldir", state->shelldir);
 }
 
 static int set_shelldir (const char *s)
@@ -1132,7 +1206,6 @@ static int set_shelldir (const char *s)
 
     if (*s == '\0') {
 	*state->shelldir = '\0';
-	gretl_insert_builtin_string("shelldir", state->shelldir);
     } else if (*s == '"') {
 	s++;
 	len = gretl_charpos('"', s);
@@ -1176,7 +1249,6 @@ static int set_shelldir (const char *s)
 
 	if (!err) {
 	    strcpy(state->shelldir, test);
-	    gretl_insert_builtin_string("shelldir", state->shelldir);
 	}
     }
 
@@ -1271,7 +1343,7 @@ static void libset_print_bool (const char *s, PRN *prn,
 			 !strcmp(s, VECM_NORM) || \
 			 !strcmp(s, GRETL_OPTIM) || \
 			 !strcmp(s, NORMAL_RAND) || \
-                         !strcmp(s, RNG))
+			 !strcmp(s, OPTIM_STEPLEN))
 
 const char *intvar_code_string (const char *s)
 {
@@ -1373,7 +1445,10 @@ static int print_settings (PRN *prn, gretlopt opt)
     libset_print_bool(WARNINGS, prn, opt);
     libset_print_int(GRETL_DEBUG, prn, opt);
     libset_print_int(BLAS_MNK_MIN, prn, opt);
-    libset_print_int(MP_MNK_MIN, prn, opt);
+    libset_print_int(OMP_MNK_MIN, prn, opt);
+    libset_print_int(OMP_N_THREADS, prn, opt);
+    libset_print_int(SIMD_K_MAX, prn, opt);
+    libset_print_int(SIMD_MN_MIN, prn, opt);
 
     if (opt & OPT_D) {
 	libset_print_bool(SHELL_OK, prn, opt);
@@ -1396,6 +1471,7 @@ static int print_settings (PRN *prn, gretlopt opt)
     libset_print_int(BFGS_MAXITER, prn, opt);
     libset_print_double(BFGS_TOLER, prn, opt);
     libset_print_double(BFGS_MAXGRAD, prn, opt);
+    libset_print_int(OPTIM_STEPLEN, prn, opt);
     libset_print_int(BHHH_MAXITER, prn, opt);
     libset_print_double(BHHH_TOLER, prn, opt);
     libset_print_int(RQ_MAXITER, prn, opt);
@@ -1409,17 +1485,19 @@ static int print_settings (PRN *prn, gretlopt opt)
     libset_print_bool(USE_FCP, prn, opt);
     libset_print_bool(DPDSTYLE, prn, opt);
     libset_print_double(NADARWAT_TRIM, prn, opt);
+    libset_print_int(FDJAC_QUAL, prn, opt);
 
     libset_header(N_("Random number generation"), prn, opt);
 
     if (opt & OPT_D) {
 	pprintf(prn, " seed = %u\n", gretl_rand_get_seed());
 	pprintf(prn, " normal_rand = %s\n", libset_option_string(NORMAL_RAND));
-	pprintf(prn, " RNG = %s\n", libset_option_string(RNG));
     } else {
 	pprintf(prn, "set seed %u\n", gretl_rand_get_seed());
 	pprintf(prn, "set normal_rand %s\n", libset_option_string(NORMAL_RAND));
-	pprintf(prn, "set RNG %s\n", libset_option_string(RNG));
+    }
+    if (gretl_mpi_initialized()) {
+	libset_print_bool(USE_DCMT, prn, opt);
     }
 
     libset_header(N_("Robust estimation"), prn, opt);
@@ -1457,7 +1535,7 @@ static int libset_query_settings (const char *s, PRN *prn)
 	coded_var_show_opts(s, prn);
     } else if (libset_int(s)) {
 	int k = libset_get_int(s);
-
+	
 	if (is_unset(k)) {
 	    pprintf(prn, "%s: positive integer, currently unset\n", s);
 	} else {
@@ -1689,6 +1767,7 @@ int execute_set_line (const char *line, DATASET *dset,
 		err = libset_set_int(setobj, k);
 	    }
 	} else {
+	    gretl_errmsg_sprintf(_("set: unknown variable '%s'"), setobj);
 	    err = E_UNKVAR;
 	}
     }
@@ -1807,6 +1886,8 @@ int libset_get_int (const char *key)
 
     if (!strcmp(key, BFGS_MAXITER)) {
 	return state->bfgs_maxiter;
+    } else if (!strcmp(key, OPTIM_STEPLEN)) {
+	return state->optim_steplen;
     } else if (!strcmp(key, BHHH_MAXITER)) {
 	return state->bhhh_maxiter;
     } else if (!strcmp(key, RQ_MAXITER)) {
@@ -1837,14 +1918,22 @@ int libset_get_int (const char *key)
 	return state->optim;
     } else if (!strcmp(key, GRETL_DEBUG)) {
 	return gretl_debug;
-    } else if (!strcmp(key, BLAS_MNK_MIN) || !strcmp(key, BLAS_NMK_MIN)) {
+    } else if (!strcmp(key, BLAS_MNK_MIN)) {
 	return get_blas_mnk_min();
-    } else if (!strcmp(key, MP_MNK_MIN) || !strcmp(key, MP_NMK_MIN)) {
-	return mp_mnk_min;
+    } else if (!strcmp(key, OMP_MNK_MIN) || !strcmp(key, MP_MNK_MIN)) {
+	return omp_mnk_min;
+    } else if (!strcmp(key, OMP_N_THREADS)) {
+	return omp_n_threads;
+    } else if (!strcmp(key, SIMD_K_MAX)) {
+	return get_simd_k_max();
+    } else if (!strcmp(key, SIMD_MN_MIN)) {
+	return get_simd_mn_min();
     } else if (!strcmp(key, BFGS_VERBSKIP)) {
 	return state->bfgs_verbskip;
     } else if (!strcmp(key, CSV_DIGITS)) {
 	return csv_digits;
+    } else if (!strcmp(key, FDJAC_QUAL)) {
+	return state->fdjac_qual;
     } else {
 	fprintf(stderr, "libset_get_int: unrecognized "
 		"variable '%s'\n", key);	
@@ -1863,6 +1952,10 @@ static int intvar_min_max (const char *s, int *min, int *max,
     } else if (!strcmp(s, BFGS_VERBSKIP)) {
 	*min = 1;
 	*var = &state->bfgs_verbskip;
+    } else if (!strcmp(s, OPTIM_STEPLEN)) {
+	*min = 0;
+	*max = STEPLEN_MAX;
+	*var = &state->optim_steplen;
     } else if (!strcmp(s, BHHH_MAXITER)) {
 	*min = 1;
 	*var = &state->bhhh_maxiter;
@@ -1891,6 +1984,7 @@ static int intvar_min_max (const char *s, int *min, int *max,
 	*var = &state->horizon;
     } else if (!strcmp(s, LOOP_MAXITER)) {
 	*min = 0;
+	*max = INT_MAX - 1;
 	*var = &state->loop_maxiter;
     } else if (!strcmp(s, VECM_NORM)) {
 	*min = 0;
@@ -1903,6 +1997,10 @@ static int intvar_min_max (const char *s, int *min, int *max,
     } else if (!strcmp(s, GRETL_DEBUG)) {
 	*min = 0;
 	*var = &gretl_debug;
+    } else if (!strcmp(s, FDJAC_QUAL)) {
+	*min = 0;
+	*max = 3;
+	*var = &state->fdjac_qual;
     } else {
 	fprintf(stderr, "libset_set_int: unrecognized "
 		"variable '%s'\n", s);	
@@ -1918,12 +2016,20 @@ int libset_set_int (const char *key, int val)
 	return 1;
     }
 
-    if (!strcmp(key, BLAS_MNK_MIN) || !strcmp(key, BLAS_NMK_MIN)) {
+    if (!strcmp(key, BLAS_MNK_MIN)) {
 	set_blas_mnk_min(val);
 	return 0;
-    } else if (!strcmp(key, MP_MNK_MIN) || !strcmp(key, MP_NMK_MIN)) {
-	mp_mnk_min = val;
+    } else if (!strcmp(key, SIMD_K_MAX)) {
+	set_simd_k_max(val);
 	return 0;
+    } else if (!strcmp(key, SIMD_MN_MIN)) {
+	set_simd_mn_min(val);
+	return 0;
+    } else if (!strcmp(key, OMP_MNK_MIN) || !strcmp(key, MP_MNK_MIN)) {
+	omp_mnk_min = val;
+	return 0;
+    } else if (!strcmp(key, OMP_N_THREADS)) {
+	return set_omp_n_threads(val);
     } else {
 	int min = 0, max = 0;
 	int *ivar = NULL;
@@ -2029,6 +2135,8 @@ int libset_get_bool (const char *key)
 	return R_functions;
     } else if (!strcmp(key, R_LIB)) {
 	return R_lib;
+    } else if (!strcmp(key, USE_DCMT)) {
+        return gretl_rand_get_dcmt();
     }
 
     if (!strcmp(key, MAX_VERBOSE) && gretl_debug > 1) {
@@ -2131,6 +2239,8 @@ int libset_set_bool (const char *key, int val)
 	return check_R_setting(&R_functions, val, key);
     } else if (!strcmp(key, R_LIB)) {
 	return check_R_setting(&R_lib, val, key);
+    } else if (!strcmp(key, USE_DCMT)) {
+	return gretl_rand_set_dcmt(val);
     }
 
     flag = boolvar_get_flag(key);
@@ -2484,7 +2594,7 @@ DEBUG_OUTPUT get_debug_output_func (void)
     return dbg_output;
 }
 
-/* support for GUI Stop button */
+/* support for GUI "Stop" button */
 
 static QUERY_STOP query_stop;
 
