@@ -306,7 +306,7 @@ gretl_covariance_matrix_from_varlist (const int *list,
     gretl_matrix *V;
     gretl_vector *xbar;
     int k = list[0];
-    int t, i, j;
+    int t, i, j, vi, vj;
     double vv, x, y;
     int n, err = 0;
     
@@ -326,12 +326,14 @@ gretl_covariance_matrix_from_varlist (const int *list,
     }
 
     for (i=0; i<k && !err; i++) {
+	vi = list[i+1];
 	for (j=i; j<k; j++) {
+	    vj = list[j+1];
 	    vv = 0.0;
 	    n = 0;
 	    for (t=dset->t1; t<=dset->t2; t++) {
-		x = dset->Z[list[i+1]][t];
-		y = dset->Z[list[j+1]][t];
+		x = dset->Z[vi][t];
+		y = dset->Z[vj][t];
 		if (na(x) || na(y)) {
 		    continue;
 		}
@@ -342,7 +344,7 @@ gretl_covariance_matrix_from_varlist (const int *list,
 		err = E_DATA;
 		vv = NADBL;
 	    } else {
-		vv /= (n - 1); /* plain n? */
+		vv /= (n - 1); /* or plain n? */
 	    }
 	    gretl_matrix_set(V, i, j, vv);
 	    gretl_matrix_set(V, j, i, vv);
@@ -444,6 +446,41 @@ static int get_mask_count (const char *mask, int n)
     return k;
 }
 
+static void add_dataset_colnames (gretl_matrix *M,
+				  const int *list,
+				  const DATASET *dset)
+{
+    int i, vi, nv = list[0];
+    char **S;
+    int err;
+
+    /* we won't treat errors here as fatal */
+
+    if (nv != M->cols) {
+	/* "can't happen" */
+	return;
+    }
+
+    S = strings_array_new(nv);
+    if (S == NULL) {
+	return;
+    }
+
+    for (i=0; i<nv; i++) {
+	vi = list[i+1];
+	S[i] = gretl_strdup(dset->varname[vi]);
+	if (S[i] == NULL) {
+	    strings_array_free(S, nv);
+	    return;
+	}
+    }
+
+    err = gretl_matrix_set_colnames(M, S);
+    if (err) {
+	strings_array_free(S, nv);
+    }
+}
+
 static gretl_matrix *
 real_gretl_matrix_data_subset (const int *list, 
 			       const DATASET *dset,
@@ -455,7 +492,7 @@ real_gretl_matrix_data_subset (const int *list,
     int T, Tmax = t2 - t1 + 1;
     int k = list[0];
     int skip;
-    int j, s, t;
+    int j, vj, s, t;
 
     if (k <= 0 || Tmax <= 0) {
 	*err = E_DATA;
@@ -463,7 +500,6 @@ real_gretl_matrix_data_subset (const int *list,
     }	
 
     *err = 0;
-
     T = Tmax;
 
     if (mask != NULL) {
@@ -478,7 +514,8 @@ real_gretl_matrix_data_subset (const int *list,
     } else if (op == M_MISSING_SKIP || op == M_MISSING_ERROR) {
 	for (t=t1; t<=t2 && !*err; t++) {
 	    for (j=0; j<k; j++) {
-		x = dset->Z[list[j+1]][t];
+		vj = list[j+1];
+		x = dset->Z[vj][t];
 		if (na(x)) {
 		    if (op == M_MISSING_SKIP) {
 			T--;
@@ -510,7 +547,8 @@ real_gretl_matrix_data_subset (const int *list,
 	    skip = mask[t - t1];
 	} else if (op == M_MISSING_SKIP) {
 	    for (j=0; j<k; j++) {
-		x = dset->Z[list[j+1]][t];
+		vj = list[j+1];
+		x = dset->Z[vj][t];
 		if (na(x)) {
 		    skip = 1;
 		    break;
@@ -519,7 +557,8 @@ real_gretl_matrix_data_subset (const int *list,
 	}
 	if (!skip) {
 	    for (j=0; j<k; j++) {
-		x = dset->Z[list[j+1]][t];
+		vj = list[j+1];
+		x = dset->Z[vj][t];
 		gretl_matrix_set(M, s, j, x);
 	    }
 	    if (s == 0) {
@@ -533,27 +572,31 @@ real_gretl_matrix_data_subset (const int *list,
     }
 
     if (!*err && (mask != NULL || op == M_MISSING_OK)) {
-	for (j=0; j<k && !*err; j++) {
-	    for (t=0; t<T && !*err; t++) {
-		x = gretl_matrix_get(M, t, j);
-		if (na(x)) {
-		    if (op == M_MISSING_OK) {
-			/* convert NA to NAN */
-			gretl_matrix_set(M, t, j, M_NA);
-		    } else {
-			*err = E_MISSDATA;
-		    }
+	int n = k * T;
+
+	for (j=0; j<n && !*err; j++) {
+	    if (na(M->val[j])) {
+		if (op == M_MISSING_OK) {
+		    /* convert NA to NAN */
+		    M->val[j] = M_NA;
+		} else {
+		    *err = E_MISSDATA;
 		}
 	    }
 	}
-    }	
+    }
 
     if (*err) {
 	gretl_matrix_free(M);
 	M = NULL;
-    } else if (T == Tmax) {
-	gretl_matrix_set_t1(M, t1);
-	gretl_matrix_set_t2(M, t2);
+    } else {
+	if (T == Tmax) {
+	    gretl_matrix_set_t1(M, t1);
+	    gretl_matrix_set_t2(M, t2);
+	}
+	if (dset->varname != NULL) {
+	    add_dataset_colnames(M, list, dset);
+	}
     }
 
     return M;
@@ -1125,7 +1168,17 @@ gretl_matrix *gretl_matrix_read_from_text (const char *fname,
 		    n = fscanf(fp, "%lf", &x);
 		}
 		if (n != 1) {
+#ifdef WIN32
+		    if (fz) {
+			x = win32_sscan_nonfinite(p, err);
+		    } else {
+			x = win32_fscan_nonfinite(fp, err);
+		    }
+#else
 		    *err = E_DATA;
+#endif
+		}
+		if (*err) {
 		    fprintf(stderr, "error reading row %d, column %d\n", i+1, j+1);
 		} else {
 		    gretl_matrix_set(A, i, j, x);
@@ -1158,6 +1211,39 @@ gretl_matrix *gretl_matrix_read_from_text (const char *fname,
     return A;
 }
 
+#ifdef WIN32
+
+static void win32_xna_out (double x, gzFile fz, FILE *fp,
+			   char pad)
+{
+    if (isnan(x) || na(x)) {
+	if (fz) {
+	    gzputs(fz, "nan");
+	} else {
+	    fputs("nan", fp);
+	}
+    } else if (x < 0) {
+	if (fz) {
+	    gzputs(fz, "-inf");
+	} else {
+	    fputs("-inf", fp);
+	}
+    } else {
+	if (fz) {
+	    gzputs(fz, "inf");
+	} else {
+	    fputs("inf", fp);
+	}
+    }
+    if (fz) {
+	gzputc(fz, pad);
+    } else {
+	fputc(pad, fp);
+    }
+}
+
+#endif
+
 /**
  * gretl_matrix_write_as_text:
  * @A: matrix.
@@ -1180,7 +1266,8 @@ int gretl_matrix_write_as_text (gretl_matrix *A, const char *fname,
     int i, j, err = 0;
     gzFile fz = Z_NULL;
     FILE *fp = NULL;
-    char d = '\t';
+    double x;
+    char pad, d = '\t';
     int gz;
 
     gz = has_suffix(fname, ".gz");
@@ -1192,14 +1279,14 @@ int gretl_matrix_write_as_text (gretl_matrix *A, const char *fname,
 	if (gz) {
 	    fz = gretl_gzopen(targ, "w");
 	} else {
-	    fp = gretl_fopen(targ, "w");
+	    fp = gretl_fopen(targ, "wb");
 	}
     } else {
 	fname = gretl_maybe_switch_dir(fname);
 	if (gz) {
 	    fz = gretl_gzopen(fname, "w");
 	} else {
-	    fp = gretl_fopen(fname, "w");
+	    fp = gretl_fopen(fname, "wb");
 	}
     }
 
@@ -1215,20 +1302,20 @@ int gretl_matrix_write_as_text (gretl_matrix *A, const char *fname,
 
     for (i=0; i<r; i++) {
 	for (j=0; j<c; j++) {
+	    pad = (j == c-1)? '\n' : d;
+	    x = gretl_matrix_get(A, i, j);
+#ifdef WIN32
+	    if (xna(x)) {
+		win32_xna_out(x, fz, fp, pad);
+		continue;
+	    }
+#endif
 	    if (fz) {
-		gzprintf(fz, "%26.18E", gretl_matrix_get(A, i, j));
-		if (j == c-1) {
-		    gzputc(fz, '\n'); 
-		} else {
-		    gzputc(fz, d);
-		}		
+		gzprintf(fz, "%26.18E", x);
+		gzputc(fz, pad);
 	    } else {
-		fprintf(fp, "%26.18E", gretl_matrix_get(A, i, j));
-		if (j == c-1) {
-		    fputc('\n', fp); 
-		} else {
-		    fputc(d, fp);
-		}
+		fprintf(fp, "%26.18E", x);
+		fputc(pad, fp);
 	    }
 	}
     }
@@ -1357,7 +1444,7 @@ real_matrix_print_to_prn (const gretl_matrix *m, const char *msg,
 	mt2 = gretl_matrix_get_t2(m);
     }
 
-    /* @plain != 0 means skip the header stuff */
+    /* @plain means skip the header stuff */
 
     if (msg != NULL && *msg != '\0' && !plain) {
 	pprintf(prn, "%s (%d x %d)", msg, m->rows, m->cols);
@@ -1511,6 +1598,62 @@ static void maybe_print_col_heads (const gretl_matrix *m,
     }
 }
 
+/* Return an array of char holding 1s for columns in
+   @m that are all integers, 0s otherwise. Except
+   that if we find that either none or all the columns
+   if @m are composed of integers we scratch the
+   array and signal the result via the @intcast pointer.
+*/
+
+static char *get_intcols (const gretl_matrix *m,
+			  int *intcast)
+{
+    char *ret = malloc(m->cols);
+
+    if (ret != NULL) {
+	double x;
+	int i, j, n = 0;
+
+	for (j=0; j<m->cols; j++) {
+	    ret[j] = 1;
+	    for (i=0; i<m->rows; i++) {
+		x = gretl_matrix_get(m, i, j);
+		if (x != floor(x)) {
+		    ret[j] = 0;
+		    break;
+		}
+	    }
+	    n += ret[j];
+	}
+	if (n == 0 || n == m->cols) {
+	    /* no, or all, integer columns */
+	    free(ret);
+	    ret = NULL;
+	    *intcast = n > 0;
+	}
+    }
+
+    return ret;
+}
+
+/* Take a format which is designed for floating-point
+   values and convert it for use with integers: preserve
+   the width but discard the precision, and change the
+   trailing 'f', 'g' or whatever to 'd'.
+*/
+
+static void revise_integer_format (char *ifmt)
+{
+    char *p = strchr(ifmt, '.');
+
+    if (p != NULL) {
+	*p = '\0';
+	strncat(p, "d", 1);
+    } else {
+	ifmt[strlen(ifmt)-1] = 'd';
+    }
+}
+
 /**
  * gretl_matrix_print_with_format:
  * @m: matrix to print.
@@ -1542,21 +1685,38 @@ void gretl_matrix_print_with_format (const gretl_matrix *m,
 	real_matrix_print_to_prn(m, NULL, 1, NULL, NULL, prn);
     } else {
 	const char **rownames = NULL;
+	char *ifmt = NULL, *xfmt = NULL;
+	char *intcols = NULL;
 	int llen = 0, intcast = 0;
 	double x;
+	int cpos = strlen(fmt) - 1;
 	int i, j, c;
 
-	c = fmt[strlen(fmt)-1];
+	c = fmt[cpos];
+
 	if (c == 'd' || c == 'u' || c == 'x' || c == 'l') {
 	    intcast = 1;
+	} else if (c == 'v') {
+	    /* "variant" column format */
+	    intcols = get_intcols(m, &intcast);
 	}
 
+	xfmt = gretl_strdup(fmt);
+
+	if (intcast || intcols) {
+	    ifmt = gretl_strdup(fmt);
+	    if (c == 'v') {
+		revise_integer_format(ifmt);
+		xfmt[cpos] = 'g';
+	    }
+	} 
+
 	rownames = gretl_matrix_get_rownames(m);
 	if (rownames != NULL) {
 	    llen = max_label_length(rownames, m->rows);
 	}
 
-	maybe_print_col_heads(m, fmt, wid, prec, intcast, llen, prn);
+	maybe_print_col_heads(m, xfmt, wid, prec, intcast, llen, prn);
 
 	for (i=0; i<m->rows; i++) {
 	    if (rownames != NULL) {
@@ -1564,28 +1724,35 @@ void gretl_matrix_print_with_format (const gretl_matrix *m,
 	    }
 	    for (j=0; j<m->cols; j++) {
 		x = gretl_matrix_get(m, i, j);
+		if (intcols != NULL) {
+		    intcast = intcols[j];
+		}
 		if (intcast) {
 		    if (wid >= 0 && prec >= 0) {
-			pprintf(prn, fmt, wid, prec, (int) x);
+			pprintf(prn, ifmt, wid, prec, (int) x);
 		    } else if (wid >= 0 || prec >= 0) {
 			c = (wid >= 0)? wid : prec;
-			pprintf(prn, fmt, c, (int) x);
+			pprintf(prn, ifmt, c, (int) x);
 		    } else {
-			pprintf(prn, fmt, (int) x);
+			pprintf(prn, ifmt, (int) x);
 		    }
 		} else {
 		    if (wid >= 0 && prec >= 0) {
-			pprintf(prn, fmt, wid, prec, x);
+			pprintf(prn, xfmt, wid, prec, x);
 		    } else if (wid >= 0 || prec >= 0) {
 			c = (wid >= 0)? wid : prec;
-			pprintf(prn, fmt, c, x);
+			pprintf(prn, xfmt, c, x);
 		    } else {
-			pprintf(prn, fmt, x);
+			pprintf(prn, xfmt, x);
 		    }
 		}
 	    }
 	    pputc(prn, '\n');
 	}
+
+	free(intcols);
+	free(ifmt);
+	free(xfmt);
     }
 }
 
@@ -2030,25 +2197,19 @@ int gretl_matrix_mp_ols (const gretl_vector *y, const gretl_matrix *X,
 			 gretl_vector *b, gretl_matrix *vcv, 
 			 gretl_vector *uhat, double *s2)
 {
-    void *handle = NULL;
     int (*matrix_mp_ols) (const gretl_vector *,
 			  const gretl_matrix *,
 			  gretl_vector *,
 			  gretl_matrix *,
 			  gretl_vector *,
 			  double *);
-    int err;
 
-    matrix_mp_ols = get_plugin_function("matrix_mp_ols", &handle);
+    matrix_mp_ols = get_plugin_function("matrix_mp_ols");
     if (matrix_mp_ols == NULL) {
 	return 1;
     }
 
-    err = (*matrix_mp_ols)(y, X, b, vcv, uhat, s2);
-
-    close_plugin(handle);
-
-    return err;
+    return (*matrix_mp_ols)(y, X, b, vcv, uhat, s2);
 }
 
 /* following: quadrature sources based on John Burkhart's GPL'd
