@@ -370,7 +370,7 @@ static int nls_genr_setup (nlspec *s)
 
     if (!err && s->hesscall != NULL) {
 	s->hgen = genr_compile(s->hesscall, s->dset, 
-			       OPT_P | OPT_U, &err);
+			       OPT_P | OPT_O, &err);
 	if (!err) {
 	    err = execute_genr(s->hgen, s->dset, s->prn);
 	} 
@@ -849,6 +849,20 @@ static int nls_make_trimmed_dataset (nlspec *spec, int t1, int t2)
 
 #endif
 
+static int nl_coeff_check (nlspec *s)
+{
+    int i;
+
+    for (i=0; i<s->ncoeff; i++) {
+	if (na(s->coeff[i])) {
+	    gretl_errmsg_set("Unitialized parameter");
+	    return E_DATA;
+	}
+    }
+
+    return 0;
+}
+
 /* Adjust starting and ending points of sample if need be, to avoid
    missing values; abort if there are missing values within the
    (possibly reduced) sample range.  For this purpose we generate the
@@ -2537,7 +2551,7 @@ static int lm_approximate (nlspec *spec, PRN *prn)
 	spec->opt = OPT_NONE;
 
 	/* call minpack again */
-	fdjac2_(nls_calc_approx, m, n, spec->coeff, spec->fvec, 
+	fdjac2_(nls_calc_approx, m, n, 0, spec->coeff, spec->fvec, 
 		spec->jac, ldjac, &iflag, epsfcn, wa4, spec);
 	spec->crit = ess;
 	spec->iters = iters;
@@ -3030,6 +3044,11 @@ static MODEL real_nl_model (nlspec *spec, DATASET *dset,
     int i, t, origv;
     int err = 0;
 
+    if (spec == NULL) {
+	/* we use the static spec composed via nl_parse_line() */
+	spec = &private_spec;
+    }
+
     gretl_model_init(&nlmod, dset);
 
     if (dset == NULL || dset->v == 0) {
@@ -3041,11 +3060,6 @@ static MODEL real_nl_model (nlspec *spec, DATASET *dset,
     origv = dset->v;
     gretl_model_smpl_init(&nlmod, dset);
 
-    if (spec == NULL) {
-	/* we use the static spec composed via nl_parse_line() */
-	spec = &private_spec;
-    }
-
     if (spec->nlfunc == NULL && spec->ci != GMM) {
 	gretl_errmsg_set(_("No function has been specified"));
 	nlmod.errcode = E_PARSE;
@@ -3075,6 +3089,11 @@ static MODEL real_nl_model (nlspec *spec, DATASET *dset,
 	goto bailout;
     } 
 
+    if (nl_coeff_check(spec)) {
+	nlmod.errcode = E_DATA;
+	goto bailout;
+    } 
+
     if (spec->ci == GMM) {
 	err = gmm_missval_check_etc(spec);
     } else {
@@ -3090,8 +3109,7 @@ static MODEL real_nl_model (nlspec *spec, DATASET *dset,
 	goto bailout;
     }
 
-    if (spec->ci == GRETL_TYPE_DOUBLE) {
-	/* FIXME what's this -- a typo?? 2013-07-07 */
+    if (spec->lhtype == GRETL_TYPE_DOUBLE) {
 	spec->fvec = NULL;
 	spec->jac = NULL;
     } else {
@@ -3855,4 +3873,3 @@ int nls_boot_calc (const MODEL *pmod, DATASET *dset,
 
     return err;
 }
-
