@@ -109,6 +109,7 @@ struct gretl_option gretl_opts[] = {
     { APPEND,   OPT_L, "cols", 2 },
     { APPEND,   OPT_S, "sheet", 2 },
     { APPEND,   OPT_V, "verbose", 0 },
+    { APPEND,   OPT_U, "update-overlap", 0 },
     { ARBOND,   OPT_A, "asymptotic", 0 },
     { ARBOND,   OPT_D, "time-dummies", 1 },
     { ARBOND,   OPT_H, "orthdev", 0 },
@@ -157,6 +158,7 @@ struct gretl_option gretl_opts[] = {
     { CORR,     OPT_V, "verbose", 0 },
     { CORRGM,   OPT_U, "plot", 2},
     { CUSUM,    OPT_R, "squares", 0 },
+    { DATA,     OPT_C, "compact", 2 },
     { DATA,     OPT_O, "odbc", 0 },
     { DATA,     OPT_M, "rowmask", 2 },
     { DATAMOD,  OPT_P, "preserve", 0 },
@@ -311,6 +313,7 @@ struct gretl_option gretl_opts[] = {
     { MPI,      OPT_Q, "quiet", 0},
     { MPI,      OPT_V, "verbose", 0},
     { MPI,      OPT_S, "single-rng", 0},
+    { MPI,      OPT_D, "debug", 0},
     { LABELS,   OPT_D, "delete", 0 },
     { LABELS,   OPT_F, "from-file", 2 },
     { LABELS,   OPT_T, "to-file", 2 },
@@ -339,7 +342,6 @@ struct gretl_option gretl_opts[] = {
     { MODPRINT, OPT_T, "tex", 0 },
     { MODELTAB, OPT_C, "complete", 0 },
     { MODELTAB, OPT_O, "output", 2 },
-    { MPOLS,    OPT_O, "vcv", 0 },
     { MPOLS,    OPT_S, "simple-print", 0 },
     { NEGBIN,   OPT_G, "opg", 0 },
     { NEGBIN,   OPT_M, "model1", 0 },
@@ -347,7 +349,6 @@ struct gretl_option gretl_opts[] = {
     { NEGBIN,   OPT_C, "cluster", 2 },
     { NEGBIN,   OPT_V, "verbose", 0 },
     { NLS,      OPT_N, "numerical", 0 },
-    { NLS,      OPT_O, "vcv", 0 },
     { NLS,      OPT_R, "robust", 0 },
     { NLS,      OPT_V, "verbose", 0 },
     { NORMTEST, OPT_A, "all", 0 },
@@ -397,7 +398,6 @@ struct gretl_option gretl_opts[] = {
     { PANEL,    OPT_I, "iterate", 0 },
     { PANEL,    OPT_M, "matrix-diff", 0 },
     { PANEL,    OPT_N, "nerlove", 0 },
-    { PANEL,    OPT_O, "vcv", 0 },
     { PANEL,    OPT_P, "pooled", 0 },
     { PANEL,    OPT_R, "robust", 0 },
     { PANEL,    OPT_S, "silent", 0 },
@@ -1186,13 +1186,27 @@ static void set_stored_options (int ci, gretlopt opt, int flags)
     fprintf(stderr, "setting stored options: ci = %d\n", ci);
 #endif
 
+    if (vcv_opt_ok(ci) && (opt & OPT_O)) {
+	real_push_option(ci, OPT_O, NULL, 1, flags);
+	opt &= ~OPT_O; /* handled */
+    }
+
+    if (quiet_opt_ok(ci) && (opt & OPT_Q)) {
+	real_push_option(ci, OPT_Q, NULL, 1, flags);
+	opt &= ~OPT_Q; /* handled */
+    } 
+
+    if (opt == 0) {
+	return;
+    }
+
     for (i=0; gretl_opts[i].o != 0; i++) {
 	if (ci == gretl_opts[i].ci) {
 	    if (opt & gretl_opts[i].o) {
 		real_push_option(ci, gretl_opts[i].o, NULL, 1, flags);
 	    }
-	    got_ci = 1;
-	} else if (got_ci) {
+	    got_ci = ci;
+	} else if (got_ci > 0 && ci != got_ci) {
 	    break;
 	}
     }
@@ -1347,7 +1361,7 @@ int set_optval_string (int ci, gretlopt opt, const char *s)
 }
 
 /* valid_long_opt: this is (semi-) public because we have need of
-   it in the experimental command tokenizer
+   it in the new command tokenizer
 */
 
 gretlopt valid_long_opt (int ci, const char *s, OptStatus *status)
