@@ -960,14 +960,15 @@ static void print_linestyle (const GPT_SPEC *spec, int i, FILE *fp)
     GPT_LINE *line;
     int done = 0;
 
-    fprintf(fp, "set style line %d ", i+1);
+    if (gnuplot_version() >= 5.0) {
+	fprintf(fp, "set linetype %d ", i+1);
+    } else {
+	fprintf(fp, "set style line %d ", i+1);
+    }
 
     if (i < spec->n_lines) {
 	line = &spec->lines[i];
-	if (*line->rgb != '\0' && line->type != LT_AUTO) {
-	    fprintf(fp, "lc rgb \"%s\" lt %d\n", line->rgb, line->type);
-	    done = 1;
-	} else if (*line->rgb != '\0') {
+	if (*line->rgb != '\0') {
 	    fprintf(fp, "lc rgb \"%s\"\n", line->rgb);
 	    done = 1;
 	}
@@ -999,33 +1000,60 @@ static void write_styles_from_plotspec (const GPT_SPEC *spec, FILE *fp)
 {
     char cstr[8];
     int i;
-    
-    if (frequency_plot_code(spec->code)) {
-	const gretlRGB *color = get_graph_color(BOXCOLOR);
 
-	print_rgb_hash(cstr, color);
-	fprintf(fp, "set style line 1 lc rgb \"%s\"\n", cstr);
-	fputs("set style line 2 lc rgb \"#000000\"\n", fp);
-    } else if (spec->code == PLOT_RQ_TAU) {
-	fputs("set style line 1 lc rgb \"#000000\"\n", fp);
-	for (i=1; i<BOXCOLOR; i++) {
-	    print_linestyle(spec, i, fp);
+    if (gnuplot_version() >= 5.0) {
+	if (frequency_plot_code(spec->code)) {
+	    const gretlRGB *color = get_graph_color(BOXCOLOR);
+
+	    print_rgb_hash(cstr, color);
+	    fprintf(fp, "set linetype 1 lc rgb \"%s\"\n", cstr);
+	    fputs("set linetype 2 lc rgb \"#000000\"\n", fp);
+	} else if (spec->code == PLOT_RQ_TAU) {
+	    fputs("set linetype 1 lc rgb \"#000000\"\n", fp);
+	    for (i=1; i<BOXCOLOR; i++) {
+		print_linestyle(spec, i, fp);
+	    }
+	} else {
+	    for (i=0; i<BOXCOLOR; i++) {
+		print_linestyle(spec, i, fp);
+	    }
+	}
+
+	if (spec->nbars > 0 || any_filledcurve(spec)) {
+	    const gretlRGB *color = get_graph_color(SHADECOLOR);
+
+	    print_rgb_hash(cstr, color);
+	    fprintf(fp, "set linetype %d lc rgb \"%s\"\n", 
+		    SHADECOLOR + 1, cstr);
 	}
     } else {
-	for (i=0; i<BOXCOLOR; i++) {
-	    print_linestyle(spec, i, fp);
+	if (frequency_plot_code(spec->code)) {
+	    const gretlRGB *color = get_graph_color(BOXCOLOR);
+
+	    print_rgb_hash(cstr, color);
+	    fprintf(fp, "set style line 1 lc rgb \"%s\"\n", cstr);
+	    fputs("set style line 2 lc rgb \"#000000\"\n", fp);
+	} else if (spec->code == PLOT_RQ_TAU) {
+	    fputs("set style line 1 lc rgb \"#000000\"\n", fp);
+	    for (i=1; i<BOXCOLOR; i++) {
+		print_linestyle(spec, i, fp);
+	    }
+	} else {
+	    for (i=0; i<BOXCOLOR; i++) {
+		print_linestyle(spec, i, fp);
+	    }
 	}
+
+	if (spec->nbars > 0 || any_filledcurve(spec)) {
+	    const gretlRGB *color = get_graph_color(SHADECOLOR);
+
+	    print_rgb_hash(cstr, color);
+	    fprintf(fp, "set style line %d lc rgb \"%s\"\n", 
+		    SHADECOLOR + 1, cstr);
+	}
+
+	fputs("set style increment user\n", fp);
     }
-
-    if (spec->nbars > 0 || any_filledcurve(spec)) {
-	const gretlRGB *color = get_graph_color(SHADECOLOR);
-
-	print_rgb_hash(cstr, color);
-	fprintf(fp, "set style line %d lc rgb \"%s\"\n", 
-		SHADECOLOR + 1, cstr);
-    }
-
-    fputs("set style increment user\n", fp);
 }
 
 static int print_point_type (GPT_LINE *line)
@@ -1307,8 +1335,14 @@ int plotspec_print (GPT_SPEC *spec, FILE *fp)
 	    }
 	}
 
+	if (spec->code == PLOT_STACKED_BAR) {
+	    /* stacked-bar histogram: special case */
+	    fprintf(fp, "'-' using 2 title \"%s\"", line->title);
+	    goto end_print_line;
+	}
+
 	if (na(line->scale)) {
-	    fprintf(fp, "%s ", line->formula); 
+	    fprintf(fp, "%s ", line->formula);
 	} else if (line->scale == 1.0) {
 	    fputs("'-' using 1", fp);
 	    if (line->ncols == 5) {
@@ -1375,6 +1409,8 @@ int plotspec_print (GPT_SPEC *spec, FILE *fp)
 	    fprintf(fp, " whiskerbars %g", line->whiskwidth);
 	}
 
+    end_print_line:
+
 	if (more_lines(spec, i, skipline)) {
 	    fputs(", \\", fp);
 	} 
