@@ -358,6 +358,31 @@ int has_suffix (const char *str, const char *sfx)
     return ret;
 }
 
+/**
+ * has_native_data_suffix:
+ * @fname: the filename to check.
+ *
+ * Returns: 1 if @fname ends with a suffix indicating it is a
+ * native gretl data file, 0 otherwise.
+ */
+
+int has_native_data_suffix (const char *fname)
+{
+    const char *p;
+
+    if (fname != NULL && (p = strrchr(fname, '.')) != NULL) {
+	p++;
+	if (!strcmp(p, "gdt") || !strcmp(p, "gdtb")) {
+	    return 1;
+	}
+	if (!strcmp(p, "GDT") || !strcmp(p, "GDTB")) {
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
 /**
  * numeric_string:
  * @str: the string to examine.
@@ -374,9 +399,15 @@ int numeric_string (const char *str)
 	return 0;
     }
 
-    if (!strcmp(str, "inf") || !strcmp(str, "nan")) {
-	/* could be variable names: they are not valid numbers */
-	return 0;
+    if (strlen(str) == 3) {
+	char screen[4];
+
+	strcpy(screen, str);
+	gretl_lower(screen);
+	if (!strcmp(screen, "inf") || !strcmp(screen, "nan")) {
+	    /* could be variable names: they are not valid numbers */
+	    return 0;
+	}
     }
 
     gretl_push_c_numeric_locale();
@@ -1027,6 +1058,30 @@ int count_fields (const char *s, const char *sep)
     return nf;
 }
 
+/**
+ * count_lines:
+ * @s: the string to process.
+ *
+ * Returns: the number of complete lines (lines ending
+ * with the newline character) in @s.
+ */
+
+int count_lines (const char *s)
+{
+    int nl = 0;
+
+    if (s != NULL) {
+	while (*s) {
+	    if (*s == '\n') {
+		nl++;
+	    }
+	    s++;
+	}
+    }
+
+    return nl;
+}
+
 /**
  * shift_string_left:
  * @str: the string to process.
@@ -2115,7 +2170,7 @@ char *make_varname_unique (char *vname, int v, DATASET *dset)
 	    }
 	}
 	if (conflict) {
-	    sprintf(tmp, "_%d", k);
+	    sprintf(tmp, "%d", k);
 	    vname[n] = '\0';
 	    strncat(vname, tmp, strlen(tmp));
 	} else {
@@ -2133,13 +2188,18 @@ char *make_varname_unique (char *vname, int v, DATASET *dset)
 
 int fix_varname_duplicates (DATASET *dset)
 {
-    int dups = 0;
+    int msg_done, dups = 0;
     int i, j;
 
     for (i=1; i<dset->v; i++) {
+	msg_done = 0;
 	for (j=i+1; j<dset->v; j++) {
-	    if (strcmp(dset->varname[i], dset->varname[j]) == 0) {
-		fprintf(stderr, "'%s' duplicated variable name\n", dset->varname[i]);
+	    if (!strcmp(dset->varname[i], dset->varname[j])) {
+		if (!msg_done) {
+		    fprintf(stderr, "'%s' duplicated variable name\n", 
+			    dset->varname[i]);
+		    msg_done = 1;
+		}
 		dups = 1;
 		make_varname_unique(dset->varname[j], j, dset);
 	    }
@@ -2526,3 +2586,57 @@ char *gretl_literal_replace (const char *orig,
 
     return mod;
 }
+
+/**
+ * gretl_substring:
+ * @str: the string to operate on.
+ * @first: 1-based index of initial character.
+ * @last: 1-based index of final character.
+ * @err: location to receive error code.
+ *
+ * Returns a substring of @str, from @first to @last.
+ */
+
+char *gretl_substring (const char *str, int first, int last, int *err)
+{
+    int len, ini, fin, sublen;
+    char *ret;
+
+    if (first <= 0 || last <= 0) {
+	gretl_errmsg_sprintf("Index value %d is out of bounds",
+			     first <= 0 ? first : last);
+	*err = E_DATA;
+    }
+
+    len = g_utf8_strlen(str, -1);
+    ini = (first < 1) ? 1 : ((first > len) ? len : first);
+    fin = (last < 1) ? 1 : ((last > len) ? len : last);
+    sublen = (fin >= ini) ? fin - ini + 1 : 0;
+
+    if (sublen == 0) {
+	ret = calloc(1, 1);
+    } else {
+	const char *s1;
+	int i;
+
+	for (i=1; i<ini; i++) {
+	    str = g_utf8_next_char(str);
+	}
+	s1 = str;
+	for (i=ini; i<=last; i++) {
+	    str = g_utf8_next_char(str);
+	}
+	len = str - s1;
+	ret = calloc(len + 1, 1);
+	if (ret != NULL) {
+	    *ret = '\0';
+	    gretl_utf8_strncat(ret, s1, len);
+	}
+    }
+
+    if (ret == NULL) {
+	*err = E_ALLOC;
+    }
+
+    return ret;
+}
