@@ -23,8 +23,11 @@
 #include "gretl_func.h"
 #include "usermat.h"
 #include "gretl_string_table.h"
+#include "matrix_extra.h"
+#include "gretl_array.h"
 #include "libset.h"
 #include "monte_carlo.h"
+#include "gretl_typemap.h"
 #include "uservar.h"
 
 #define UVDEBUG 0
@@ -78,11 +81,21 @@ static double *na_ptr (void)
     return px;
 }
 
-static user_var *user_var_new (const char *name, int type, void *value)
+static user_var *user_var_new (const char *name, int type, 
+			       void *value, int *err)
 {
-    user_var *u = malloc(sizeof *u);
+    user_var *u;
 
-    if (u != NULL) {
+    if (type == GRETL_TYPE_NONE) {
+	*err = E_DATA;
+	return NULL;
+    }
+
+    u = malloc(sizeof *u);
+
+    if (u == NULL) {
+	*err = E_ALLOC;
+    } else {
 	u->type = type;
 	u->level = gretl_function_depth();
 	u->flags = 0;
@@ -96,7 +109,6 @@ static user_var *user_var_new (const char *name, int type, void *value)
 	    if (m == NULL) {
 		u->ptr = gretl_null_matrix_new();
 	    } else if (matrix_is_saved(m)) {
-		fprintf(stderr, "*** user_var_new: got matrix_is_saved\n");
 		u->ptr = gretl_matrix_copy(m);
 	    } else {
 		u->ptr = value;
@@ -125,10 +137,22 @@ static user_var *user_var_new (const char *name, int type, void *value)
 	    } else {
 		u->ptr = value;
 	    }
+	} else if (gretl_array_type(type)) {
+	    if (value == NULL) {
+		u->ptr = gretl_array_new(type, 0, err);
+	    } else {
+		u->ptr = value;
+	    }
+	    u->type = GRETL_TYPE_ARRAY;
+	} else {
+	    *err = E_DATA;
 	}
     }
 
     if (u->ptr == NULL) {
+	if (!*err) {
+	    *err = E_ALLOC;
+	}
 	free(u);
 	u = NULL;
     }
@@ -145,6 +169,8 @@ static void uvar_free_value (user_var *u)
     } else if (u->type == GRETL_TYPE_STRING) {
 	bufgets_finalize(u->ptr);
 	free(u->ptr);
+    } else if (u->type == GRETL_TYPE_ARRAY) {
+	gretl_array_destroy(u->ptr);
     } else {
 	/* scalar, list */
 	free(u->ptr);
@@ -214,9 +240,11 @@ static int real_user_var_add (const char *name,
 			      void *value, 
 			      gretlopt opt)
 {
-    user_var *u = user_var_new(name, type, value);
+    user_var *u;
     int err = 0;
 
+    u = user_var_new(name, type, value, &err);
+
     /* We use OPT_P for a private variable, OPT_A
        when adding as a function argument, OPT_S
        when adding as a "shell" variable.
@@ -226,9 +254,7 @@ static int real_user_var_add (const char *name,
     fprintf(stderr, "real_user_var_add: '%s'\n", name);
 #endif
 
-    if (u == NULL) {
-	err = E_ALLOC;
-    } else {
+    if (!err) {
 	err = resize_uvar_stack(n_vars + 1);
 	if (!err) {
 	    if (opt & OPT_P) {
@@ -252,8 +278,6 @@ static int real_user_var_add (const char *name,
 	return (*user_var_callback)(name, type, UVAR_ADD);
     }
 
-    /* FIXME GUI scalars callback? */
-
     return err;
 }
 
@@ -555,6 +579,14 @@ int user_var_set_name (user_var *uvar, const char *name)
     }
 }
 
+static int array_ref_type (GretlType type)
+{
+    return type == GRETL_TYPE_STRINGS_REF ||
+	type == GRETL_TYPE_MATRICES_REF ||
+	type == GRETL_TYPE_BUNDLES_REF ||
+	type == GRETL_TYPE_LISTS_REF;
+}
+
 /**
  * user_var_localize:
  * @origname: name of variable at caller level.
@@ -574,12 +606,20 @@ int user_var_localize (const char *origname,
     user_var *u;
     int err = 0;
 
+    if (array_ref_type(type)) {
+	type = GRETL_TYPE_ARRAY;
+    } else {
+	type = gretl_type_get_plain_type(type);
+    }
+
     if (type == GRETL_TYPE_SCALAR_REF) {
 	type = GRETL_TYPE_DOUBLE;
     } else if (type == GRETL_TYPE_MATRIX_REF) {
 	type = GRETL_TYPE_MATRIX;
     } else if (type == GRETL_TYPE_BUNDLE_REF) {
 	type = GRETL_TYPE_BUNDLE;
+    } else if (array_ref_type(type)) {
+	type = GRETL_TYPE_ARRAY;
     }
 
     u = get_user_var_of_type_by_name(origname, type);
@@ -618,6 +658,8 @@ int user_var_unlocalize (const char *localname,
 	type = GRETL_TYPE_MATRIX;
     } else if (type == GRETL_TYPE_BUNDLE_REF) {
 	type = GRETL_TYPE_BUNDLE;
+    } else if (array_ref_type(type)) {
+	type = GRETL_TYPE_ARRAY;
     }
 
     u = get_user_var_of_type_by_name(localname, type);
@@ -724,6 +766,23 @@ char *user_string_resize (const char *name, size_t len, int *err)
     return (char *) u->ptr;
 }
 
+static int check_array_type_compat (GretlType type,
+				    user_var *u)
+{
+    int err = 0;
+
+    if (u->type != GRETL_TYPE_ARRAY) {
+	err = E_TYPES;
+    } else {
+	/* we also need a more specific check here */
+	if (type != gretl_array_get_type(u->ptr)) {
+	    err = E_TYPES;
+	}
+    }    
+
+    return err;
+}
+
 int user_var_add_or_replace (const char *name,
 			     GretlType type,
 			     void *value)
@@ -731,10 +790,15 @@ int user_var_add_or_replace (const char *name,
     user_var *u = get_user_var_by_name(name);
     int err = 0;
 
-    if (u != NULL && u->type != type) {
-	err = E_TYPES;
-    } else if (u != NULL) {
- 	err = user_var_replace_value(u, value);
+    if (u != NULL) {
+	if (gretl_array_type(type)) {
+	    err = check_array_type_compat(type, u);
+	} else if (u->type != type) {
+	    err = E_TYPES;
+	}
+	if (!err) {
+	    err = user_var_replace_value(u, value);
+	}
     } else {
 	err = real_user_var_add(name, type, value, OPT_NONE);
     }
@@ -904,6 +968,7 @@ int copy_matrix_as (const gretl_matrix *m, const char *newname,
 int copy_as_arg (const char *param_name, GretlType type, void *value)
 {
     void *copyval = NULL;
+    GretlType cpytype = type;
     int err = 0;
 
     if (type == GRETL_TYPE_MATRIX) {
@@ -940,16 +1005,22 @@ int copy_as_arg (const char *param_name, GretlType type, void *value)
 	    copyval = px;
 	}
     } else if (type == GRETL_TYPE_BUNDLE) {
-	gretl_bundle *bcpy = gretl_bundle_copy((gretl_bundle *) value,
-					       &err);
+	gretl_bundle *bcpy = gretl_bundle_copy(value, &err);
 
 	if (!err) {
 	    copyval = bcpy;
 	}
-    }	
+    } else if (gretl_array_type(type)) {
+	gretl_array *acpy = gretl_array_copy(value, &err);
+	
+	if (!err) {
+	    copyval = acpy;
+	    cpytype = gretl_array_get_type(acpy);
+	}
+    }
 
     if (!err) {
- 	err = real_user_var_add(param_name, type, copyval, OPT_A);
+ 	err = real_user_var_add(param_name, cpytype, copyval, OPT_A);
     }
 
     return err;
@@ -1112,6 +1183,7 @@ int delete_user_vars_of_type (GretlType type, PRN *prn)
 
     if (type == GRETL_TYPE_MATRIX ||
 	type == GRETL_TYPE_BUNDLE ||
+	type == GRETL_TYPE_ARRAY  ||
 	type == GRETL_TYPE_STRING ||
 	type == GRETL_TYPE_DOUBLE ||
 	type == GRETL_TYPE_LIST) {
@@ -1126,7 +1198,7 @@ int delete_user_vars_of_type (GretlType type, PRN *prn)
 
 	if (!err && gretl_messages_on()) {
 	    pprintf(prn, "Deleted all variables of type %s\n",
-		    gretl_arg_type_name(type));
+		    gretl_type_get_name(type));
 	}
     } else {
 	err = E_DATATYPE;
@@ -1166,49 +1238,9 @@ char *temp_name_for_bundle (void)
 
 static void xml_put_user_matrix (user_var *u, FILE *fp)
 {
-    gretl_matrix *M;
-    const char **S;
-    int i, j;
-
-    if (u == NULL || u->ptr == NULL) {
-	return;
+    if (u != NULL && u->ptr != NULL) {
+	gretl_matrix_serialize(u->ptr, u->name, fp);
     }
-
-    M = u->ptr;
-
-    fprintf(fp, "<gretl-matrix name=\"%s\" rows=\"%d\" cols=\"%d\"", 
-	    u->name, M->rows, M->cols);
-
-    S = gretl_matrix_get_colnames(M);
-
-    if (S != NULL) {
-	fputs(" colnames=\"", fp);
-	for (j=0; j<M->cols; j++) {
-	    fputs(S[j], fp);
-	    fputc((j < M->cols - 1)? ' ' : '"', fp);
-	}
-    } 
-
-    S = gretl_matrix_get_rownames(M);
-
-    if (S != NULL) {
-	fputs(" rownames=\"", fp);
-	for (j=0; j<M->rows; j++) {
-	    fputs(S[j], fp);
-	    fputc((j < M->rows - 1)? ' ' : '"', fp);
-	}
-    }     
-
-    fputs(">\n", fp);
-
-    for (i=0; i<M->rows; i++) {
-	for (j=0; j<M->cols; j++) {
-	    fprintf(fp, "%.16g ", gretl_matrix_get(M, i, j));
-	}
-	fputc('\n', fp);
-    }
-
-    fputs("</gretl-matrix>\n", fp); 
 }
 
 /**
@@ -1247,7 +1279,7 @@ void print_scalars (PRN *prn)
 	if (uvars[i]->type == GRETL_TYPE_DOUBLE &&
 	    uvars[i]->level == level) {
 	    x = *(double *) uvars[i]->ptr;
-	    pprintf(prn, " %*s = %.15g\n", maxlen, uvars[i]->name, x);
+	    pprintf(prn, " %*s = %.16g\n", maxlen, uvars[i]->name, x);
 	}
     }
 
@@ -1410,7 +1442,8 @@ int gretl_is_scalar (const char *name)
  *
  * Returns: the value of string variable @name, or %NULL
  * if there is no such variable. Note that this is the
- * actual string value, not a copy thereof.
+ * actual string value, not a copy thereof, compare
+ * copy_string_by_name().
  */
 
 char *get_string_by_name (const char *name)
@@ -1426,6 +1459,41 @@ char *get_string_by_name (const char *name)
     }
 }
 
+/**
+ * copy_string_by_name:
+ * @name: the name of the string variable to access.
+ * @err: location to receive error code.
+ *
+ * Returns: a copy of the value of string variable @name, 
+ * or %NULL on failure.
+ */
+
+char *copy_string_by_name (const char *name, int *err)
+{
+    user_var *u;
+    const char *s;
+    char *ret = NULL;
+
+    u = get_user_var_of_type_by_name(name, GRETL_TYPE_STRING);
+
+    if (u != NULL) {
+	s = u->ptr;
+    } else {
+	s = get_built_in_string_by_name(name);
+    }
+
+    if (s == NULL) {
+	*err = E_DATA;
+    } else {
+	ret = gretl_strdup(s);
+	if (ret == NULL) {
+	    *err = E_ALLOC;
+	}
+    }
+
+    return ret;
+}
+
 /**
  * gretl_is_string:
  * @name: name to test.
@@ -1656,7 +1724,7 @@ static void write_user_scalars (FILE *fp)
     for (i=0; i<n_vars; i++) {
 	if (uvars[i]->type == GRETL_TYPE_DOUBLE) {
 	    x = *(double *) uvars[i]->ptr;
-	    fprintf(fp, " <gretl-scalar name=\"%s\" value=\"%.15g\"/>\n", 
+	    fprintf(fp, " <gretl-scalar name=\"%s\" value=\"%.16g\"/>\n", 
 		    uvars[i]->name, x);
 	}
     }
@@ -1679,9 +1747,9 @@ static void write_user_lists (FILE *fp)
 
     for (i=0; i<n_vars; i++) {
 	if (uvars[i]->type == GRETL_TYPE_LIST) {
-	    gretl_xml_put_named_list(uvars[i]->name, 
-				     uvars[i]->ptr, 
-				     fp);
+	    gretl_list_serialize(uvars[i]->ptr,
+				 uvars[i]->name,
+				 fp);
 	}
     }
 }
@@ -1692,7 +1760,9 @@ static void write_user_bundles (FILE *fp)
 
     for (i=0; i<n_vars; i++) {
 	if (uvars[i]->type == GRETL_TYPE_BUNDLE) {
-	    xml_put_bundle(uvars[i]->ptr, uvars[i]->name, fp);
+	    gretl_bundle_serialize(uvars[i]->ptr, 
+				   uvars[i]->name, 
+				   fp);
 	}
     }
 }
@@ -1725,8 +1795,6 @@ static int read_user_scalars (xmlDocPtr doc, xmlNodePtr cur)
 static int read_user_matrices (xmlDocPtr doc, xmlNodePtr cur) 
 {
     gretl_matrix *m;
-    char *colnames;
-    char *rownames;
     char *name;
     int err = 0;
 
@@ -1738,20 +1806,10 @@ static int read_user_matrices (xmlDocPtr doc, xmlNodePtr cur)
 	    if (name == NULL) {
 		err = 1;
 	    } else {
-		colnames = rownames = NULL;
-		m = xml_get_user_matrix(cur, doc, &colnames, 
-					&rownames, &err);
+		m = gretl_xml_get_matrix(cur, doc, &err);
 		if (m != NULL) {
 		    err = user_var_add(name, GRETL_TYPE_MATRIX, m);
-		    if (!err && colnames != NULL) {
-			umatrix_set_names_from_string(m, colnames, 0);
-		    }
-		    if (!err && rownames != NULL) {
-			umatrix_set_names_from_string(m, rownames, 1);
-		    }		    
 		}
-		free(colnames);
-		free(rownames);
 		free(name);
 	    }
 	}
@@ -1774,7 +1832,7 @@ static int read_user_lists (xmlDocPtr doc, xmlNodePtr cur)
 	    if (!gretl_xml_get_prop_as_string(cur, "name", &name)) {
 		err = E_DATA;
 	    } else {
-		list = gretl_xml_node_get_list(cur, doc, &err);
+		list = gretl_xml_get_list(cur, doc, &err);
 		if (!err) {
 		    err = user_var_add(name, GRETL_TYPE_LIST, list);
 		}
@@ -1797,18 +1855,22 @@ static int read_user_bundles (xmlDocPtr doc, xmlNodePtr cur)
 
     while (cur != NULL && !err) {
         if (!xmlStrcmp(cur->name, (XUC) "gretl-bundle")) {
-	    char *name, *creator = NULL;
+	    char *name = (char *) xmlGetProp(cur, (XUC) "name");
 
-	    name = (char *) xmlGetProp(cur, (XUC) "name");
 	    if (name == NULL) {
 		err = 1;
 	    } else {
-		creator = (char *) xmlGetProp(cur, (XUC) "creator");
-		err = load_bundle_from_xml(cur, doc, name, creator);
-		free(name);
-		if (creator != NULL) {
-		    free(creator);
+		char *creator = NULL;
+		gretl_bundle *b;
+		
+		b = gretl_bundle_deserialize(cur, doc, &err);
+		if (!err) {
+		    creator = (char *) xmlGetProp(cur, (XUC) "creator");
+		    gretl_bundle_set_creator(b, creator);
+		    err = user_var_add(name, GRETL_TYPE_BUNDLE, b);
 		}
+		free(name);
+		free(creator);
 	    }
 	}
 	cur = cur->next;
@@ -1951,3 +2013,31 @@ int deserialize_user_vars (const char *dirname)
 	    
     return err;
 }
+
+int print_user_var_by_name (const char *name,
+			    const DATASET *dset,
+			    PRN *prn)
+{
+    user_var *u = get_user_var_by_name(name);
+
+    if (u == NULL || u->ptr == NULL) {
+	return E_DATA;
+    }
+
+    if (u->type == GRETL_TYPE_DOUBLE) {
+	print_scalar_by_name(name, prn);
+    } else if (u->type == GRETL_TYPE_MATRIX) {
+	gretl_matrix_print_to_prn(u->ptr, name, prn);
+    } else if (u->type == GRETL_TYPE_BUNDLE) {
+	gretl_bundle_print(u->ptr, prn);
+    } else if (u->type == GRETL_TYPE_ARRAY) {
+	gretl_array_print(u->ptr, prn);
+    } else if (u->type == GRETL_TYPE_LIST) {
+	gretl_list_print(u->ptr, dset, prn);
+    } else if (u->type == GRETL_TYPE_STRING) {
+	pputs(prn, (char *) u->ptr);
+	pputc(prn, '\n');
+    }
+
+    return 0;
+}
