@@ -2586,7 +2586,6 @@ int gretl_VECM_test (GRETL_VAR *vecm,
 		     gretlopt opt,
 		     PRN *prn)
 {
-    void *handle = NULL;
     int (*jfun) (GRETL_VAR *, gretl_restriction *,
 		 const DATASET *, gretlopt opt, PRN *);
     int err = 0;
@@ -2597,13 +2596,12 @@ int gretl_VECM_test (GRETL_VAR *vecm,
 
     gretl_error_clear();
 
-    jfun = get_plugin_function("vecm_test_restriction", &handle);
+    jfun = get_plugin_function("vecm_test_restriction");
     
     if (jfun == NULL) {
 	err = 1;
     } else {
 	err = (*jfun) (vecm, rset, dset, opt, prn);
-	close_plugin(handle);
     }
 
     return err;    
@@ -2613,19 +2611,17 @@ static int
 johansen_test_complete (GRETL_VAR *jvar, const DATASET *dset, 
 			gretlopt opt, PRN *prn)
 {
-    void *handle = NULL;
     int (*jfun) (GRETL_VAR *, const DATASET *, gretlopt, PRN *);
     int err = 0;
 
     gretl_error_clear();
     
-    jfun = get_plugin_function("johansen_coint_test", &handle);
+    jfun = get_plugin_function("johansen_coint_test");
 
     if (jfun == NULL) {
 	err = 1;
     } else {
 	err = (* jfun) (jvar, dset, opt, prn);
-	close_plugin(handle);
     }
     
     return err;
@@ -2635,20 +2631,18 @@ static int
 johansen_estimate_complete (GRETL_VAR *jvar, gretl_restriction *rset,
 			    const DATASET *dset, PRN *prn)
 {
-    void *handle = NULL;
     int (*jfun) (GRETL_VAR *, gretl_restriction *,
 		 const DATASET *, PRN *);
     int err = 0;
 
     gretl_error_clear();
 
-    jfun = get_plugin_function("johansen_estimate", &handle);
+    jfun = get_plugin_function("johansen_estimate");
 
     if (jfun == NULL) {
 	err = 1;
     } else {
 	err = (* jfun) (jvar, rset, dset, prn);
-	close_plugin(handle);
     }
 
     return err;
@@ -4176,13 +4170,13 @@ GRETL_VAR *gretl_VAR_from_XML (xmlNodePtr node, xmlDocPtr doc,
 
     while (cur != NULL && !*err) {
 	if (!xmlStrcmp(cur->name, (XUC) "lags")) {
-	    var->lags = gretl_xml_node_get_list(cur, doc, err);
+	    var->lags = gretl_xml_get_list(cur, doc, err);
 	} else if (!xmlStrcmp(cur->name, (XUC) "ylist")) {
-	    var->ylist = gretl_xml_node_get_list(cur, doc, err);
+	    var->ylist = gretl_xml_get_list(cur, doc, err);
 	} else if (!xmlStrcmp(cur->name, (XUC) "xlist")) {
-	    var->xlist = gretl_xml_node_get_list(cur, doc, err);
+	    var->xlist = gretl_xml_get_list(cur, doc, err);
 	} else if (!xmlStrcmp(cur->name, (XUC) "rlist")) {
-	    var->rlist = gretl_xml_node_get_list(cur, doc, err);
+	    var->rlist = gretl_xml_get_list(cur, doc, err);
 	} else if (!xmlStrcmp(cur->name, (XUC) "Fvals")) {
 	    var->Fvals = gretl_xml_get_double_array(cur, doc, &n, err);
 	} else if (!xmlStrcmp(cur->name, (XUC) "Ivals")) {
@@ -4275,20 +4269,20 @@ static void johansen_serialize (JohansenInfo *j, FILE *fp)
 
     fputs(">\n", fp);
 
-    gretl_xml_put_matrix(j->R0, "u", fp);
-    gretl_xml_put_matrix(j->R1, "v", fp);
-    gretl_xml_put_matrix(j->S00, "Suu", fp);
-    gretl_xml_put_matrix(j->S11, "Svv", fp);
-    gretl_xml_put_matrix(j->S01, "Suv", fp);
-    gretl_xml_put_matrix(j->evals, "evals", fp);
-    gretl_xml_put_matrix(j->Beta, "Beta", fp);
-    gretl_xml_put_matrix(j->Alpha, "Alpha", fp);
-    gretl_xml_put_matrix(j->Bvar, "Bvar", fp);
-    gretl_xml_put_matrix(j->Bse, "Bse", fp);
-    gretl_xml_put_matrix(j->R, "R", fp);
-    gretl_xml_put_matrix(j->q, "q", fp);
-    gretl_xml_put_matrix(j->Ra, "Ra", fp);
-    gretl_xml_put_matrix(j->qa, "qa", fp);
+    gretl_matrix_serialize(j->R0, "u", fp);
+    gretl_matrix_serialize(j->R1, "v", fp);
+    gretl_matrix_serialize(j->S00, "Suu", fp);
+    gretl_matrix_serialize(j->S11, "Svv", fp);
+    gretl_matrix_serialize(j->S01, "Suv", fp);
+    gretl_matrix_serialize(j->evals, "evals", fp);
+    gretl_matrix_serialize(j->Beta, "Beta", fp);
+    gretl_matrix_serialize(j->Alpha, "Alpha", fp);
+    gretl_matrix_serialize(j->Bvar, "Bvar", fp);
+    gretl_matrix_serialize(j->Bse, "Bse", fp);
+    gretl_matrix_serialize(j->R, "R", fp);
+    gretl_matrix_serialize(j->q, "q", fp);
+    gretl_matrix_serialize(j->Ra, "Ra", fp);
+    gretl_matrix_serialize(j->qa, "qa", fp);
 
     fputs("</gretl-johansen>\n", fp);
 }
@@ -4335,17 +4329,17 @@ int gretl_VAR_serialize (const GRETL_VAR *var, SavedObjectFlags flags,
 
     if (var->X != NULL && var->Y != NULL) {
 	/* could be fiddly to reconstruct, needed for IRF bootstrap */
-	gretl_xml_put_matrix(var->X, "X", fp);
-	gretl_xml_put_matrix(var->Y, "Y", fp);
+	gretl_matrix_serialize(var->X, "X", fp);
+	gretl_matrix_serialize(var->Y, "Y", fp);
     }  
 
     if (var->ord != NULL) {
-	gretl_xml_put_matrix(var->ord, "ord", fp);
+	gretl_matrix_serialize(var->ord, "ord", fp);
     }
 
     if (var->ci == VECM) {
 	/* this is hard to reconstruct for VECMs */
-	gretl_xml_put_matrix(var->A, "A", fp);
+	gretl_matrix_serialize(var->A, "A", fp);
     }
 
     gretl_pop_c_numeric_locale();
