@@ -1127,13 +1127,8 @@ static int finalize_ods_import (DATASET *dset,
     }
 
     if (!err) {
-	gretlopt merge_opt = 0;
-
-	if (merge && (opt & OPT_T)) {
-	    merge_opt = OPT_T;
-	}
-	err = merge_or_replace_data(dset, &sheet->dset, 
-				    merge_opt, prn);
+	err = merge_or_replace_data(dset, &sheet->dset,
+				    opt, prn);
     }  
 
     if (!err && !merge) {
