@@ -1942,7 +1942,6 @@ int sav_get_data (const char *fname, DATASET *dset,
 	destroy_dataset(newset);
     } else {
 	int merge = (dset->Z != NULL);
-	gretlopt merge_opt = 0;
 
 	if (fix_varname_duplicates(newset)) {
 	    pputs(prn, _("warning: some variable names were duplicated\n"));
@@ -1960,11 +1959,7 @@ int sav_get_data (const char *fname, DATASET *dset,
 	    sdat.descrip = NULL;
 	}
 
-	if (merge && (opt & OPT_T)) {
-	    merge_opt = OPT_T;
-	}
-
-	err = merge_or_replace_data(dset, &newset, merge_opt, prn);
+	err = merge_or_replace_data(dset, &newset, opt, prn);
 
 	if (!err && !merge) {
 	    dataset_add_import_info(dset, fname, GRETL_SAV);
