@@ -4,7 +4,7 @@
    Based on stataread.c from the GNU R "foreign" package with the 
    following original info:
 
-     * $Id: stata_import.c,v 1.57 2014/06/02 14:13:00 allin Exp $
+     * $Id: stata_import.c,v 1.58 2014/08/13 23:08:50 allin Exp $
   
      (c) 1999, 2000, 2001, 2002 Thomas Lumley. 
      2000 Saikat DebRoy
@@ -954,7 +954,6 @@ int dta_get_data (const char *fname, DATASET *dset,
 	}	
     } else {
 	int merge = (dset->Z != NULL);
-	gretlopt merge_opt = 0;
 	int nvtarg = newset->v - 1;
 
 	if (nvread < nvtarg) {
@@ -970,11 +969,7 @@ int dta_get_data (const char *fname, DATASET *dset,
 	    gretl_string_table_destroy(st);
 	}
 
-	if (merge && (opt & OPT_T)) {
-	    merge_opt = OPT_T;
-	}
-
-	err = merge_or_replace_data(dset, &newset, merge_opt, prn);
+	err = merge_or_replace_data(dset, &newset, opt, prn);
     
 	if (!err && !merge) {
 	    dataset_add_import_info(dset, fname, GRETL_DTA);
