@@ -345,11 +345,13 @@ static GtkWidget *x12a_help_button (GtkWidget *hbox, tx_request *request)
     GtkWidget *button;
 
     button = gtk_button_new_from_stock(GTK_STOCK_HELP);
-    gtk_widget_set_can_default(button, TRUE);
     gtk_container_add(GTK_CONTAINER(hbox), button);
     gtk_button_box_set_child_secondary(GTK_BUTTON_BOX(hbox),
 				       button, TRUE);
-    gtk_widget_show(button);
+#if GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION >= 2
+    gtk_button_box_set_child_non_homogeneous(GTK_BUTTON_BOX(hbox),
+					     button, TRUE);
+#endif
     g_signal_connect(G_OBJECT(button), "clicked", 
 		     G_CALLBACK(request->helpfunc), 
 		     NULL);
@@ -1034,6 +1036,21 @@ static int write_tramo_file (const char *fname,
     return 0;
 }
 
+static int x12_get_subperiod (double x, const DATASET *dset)
+{
+    int i, d = ceil(log10(dset->pd));
+    int ret;
+
+    x -= floor(x);
+    for (i=0; i<d; i++) {
+	x *= 10;
+    }
+
+    ret = (x-floor(x)) >.5 ? ceil(x) : floor(x);
+
+    return ret;
+}
+
 static int write_spc_file (const char *fname, const double *y,
 			   const char *vname,
 			   const DATASET *dset, 
@@ -1058,7 +1075,7 @@ static int write_spc_file (const char *fname, const double *y,
     sprintf(tmp, "%g", x);
     p = strchr(tmp, '.');
     if (p != NULL) {
-	startper = atoi(p + 1);
+	startper = x12_get_subperiod(x, dset);
     } else {
 	startper = 1;
     }
